@@ -1,9 +1,8 @@
 __project_name__ = 'upsies'
 __description__ = 'Media metadata aggregator'
 __homepage__ = 'https://upsies.readthedocs.io'
-__version__ = '2023.05.25'
-__author__ = 'plotski'
-__author_email__ = 'plotski@example.org'
+__changelog__ = 'https://codeberg.org/plotski/upsies/raw/branch/master/NEWS'
+__version__ = '2024.02.27'
 
 
 def application_setup(config):
