@@ -15,16 +15,19 @@ DEFAULT_CACHE_DIRECTORY = os.path.join(XDG_CACHE_HOME, __project_name__)
 GENERIC_TORRENTS_DIRPATH = os.path.join(DEFAULT_CACHE_DIRECTORY, 'generic_torrents')
 """Path to directory that contains cached torrents for re-using piece hashes"""
 
-CONFIG_FILEPATH = os.path.join(XDG_CONFIG_HOME, __project_name__, 'config.ini')
+CONFIG_DIRECTORYPATH = os.path.join(XDG_CONFIG_HOME, __project_name__)
+"""Path to configuration directory"""
+
+CONFIG_FILEPATH = os.path.join(CONFIG_DIRECTORYPATH, 'config.ini')
 """Path to general configuration file"""
 
-TRACKERS_FILEPATH = os.path.join(XDG_CONFIG_HOME, __project_name__, 'trackers.ini')
+TRACKERS_FILEPATH = os.path.join(CONFIG_DIRECTORYPATH, 'trackers.ini')
 """Path to trackers configuration file"""
 
-IMGHOSTS_FILEPATH = os.path.join(XDG_CONFIG_HOME, __project_name__, 'imghosts.ini')
+IMGHOSTS_FILEPATH = os.path.join(CONFIG_DIRECTORYPATH, 'imghosts.ini')
 """Path to image hosting services configuration file"""
 
-CLIENTS_FILEPATH = os.path.join(XDG_CONFIG_HOME, __project_name__, 'clients.ini')
+CLIENTS_FILEPATH = os.path.join(CONFIG_DIRECTORYPATH, 'clients.ini')
 """Path to BitTorrent clients configuration file"""
 
 VIDEO_FILE_EXTENSIONS = ('mkv', 'mp4', 'avi', 'wmv', 'vob', 'm2ts', 'mts', 'ts')
