@@ -107,10 +107,14 @@ class ImageOptimizeError(UpsiesError):
     """Image optimization failed"""
 
 
-class TorrentError(UpsiesError):
+class TorrentCreateError(UpsiesError):
     """Torrent file creation failed"""
 
 
+class TorrentAddError(UpsiesError):
+    """Adding torrent to client failed"""
+
+
 def SubprocessError(exception, original_traceback):
     """Attach `original_traceback` to `exception`"""
     exception.original_traceback = f'Subprocess traceback:\n{original_traceback.strip()}'
