@@ -21,10 +21,18 @@ class ChoiceJob(JobBase):
     attribute:
 
         ``dialog_updated``
-            Emitted when the :attr:`choices`, :attr:`focused` or
-            :attr:`autodetected` properties are set or when :attr:`choices` is
+            Emitted when the :attr:`options`, :attr:`focused` or
+            :attr:`autodetected` properties are set or when :attr:`options` is
             modified. Registered callbacks get the job instance as a positional
             argument.
+
+        ``autodetecting``
+            Emitted when :attr:`autodetect` is called. Registered callbacks
+            get the job instance as a positional argument.
+
+        ``autodetected``
+            Emitted when :attr:`autodetect` returns. Registered callbacks get
+            the job instance as a positional argument.
     """
 
     @property
@@ -36,56 +44,66 @@ class ChoiceJob(JobBase):
         return self._label
 
     @property
-    def choices(self):
-        """
-        Sequence of choices the user can make
+    def question(self):
+        """Text that is displayed alongside the :attr:`options`"""
+        return self._question
 
-        A choice is a :class:`tuple` with 2 items. The first item is a
+    @question.setter
+    def question(self, question):
+        """Text that is displayed alongside the :attr:`options`"""
+        self._question = question
+
+    @property
+    def options(self):
+        """
+        Sequence of options the user can make
+
+        An option is a :class:`tuple` with 2 items. The first item is a
         human-readable :class:`str` and the second item is any object that is
         available as :attr:`choice` when the job is finished.
 
-        Choices may also be passed as a flat iterable of :class:`str`, in which
+        Options may also be passed as a flat iterable of :class:`str`, in which
         case both items in the tuple are identical.
 
         When setting this property, focus is preserved if the value of the
-        focused choice exists in the new choices. Otherwise, the first choice is
+        focused option exists in the new options. Otherwise, the first option is
         focused.
         """
-        return getattr(self, '_choices', ())
+        return getattr(self, '_options', ())
 
-    @choices.setter
-    def choices(self, choices):
-        # Build new list of choices
-        valid_choices = []
-        for choice in choices:
-            if isinstance(choice, str):
-                valid_choices.append((choice, choice))
-            elif isinstance(choice, collections.abc.Sequence):
-                if len(choice) != 2:
-                    raise ValueError(f'Choice must be 2-tuple, not {choice!r}')
+    @options.setter
+    def options(self, options):
+        # Build new list of options
+        valid_options = []
+        for option in options:
+            if isinstance(option, str):
+                valid_options.append((option, option))
+            elif isinstance(option, collections.abc.Sequence):
+                if len(option) != 2:
+                    raise ValueError(f'Option must be 2-tuple, not {option!r}')
                 else:
-                    valid_choices.append((str(choice[0]), choice[1]))
+                    valid_options.append((str(option[0]), option[1]))
             else:
-                raise ValueError(f'Choice must be 2-tuple, not {choice!r}')
+                raise ValueError(f'Option must be 2-tuple, not {option!r}')
 
-        if len(valid_choices) < 2:
-            raise ValueError(f'There must be at least 2 choices: {choices!r}')
+        if len(valid_options) < 2:
+            raise ValueError(f'There must be at least 2 options: {options!r}')
 
         # Remember current focus
         prev_focused = self.focused
 
-        # Set new choices
-        self._choices = utils.MonitoredList(
-            valid_choices,
+        # Set new options
+        self._options = utils.MonitoredList(
+            valid_options,
             callback=lambda _: self.signal.emit('dialog_updated', self),
         )
 
         # Try to restore focus if the previously focused item still exists,
-        # default to first choice
+        # default to first option
         self._focused_index = 0
         if prev_focused:
             prev_label, prev_value = prev_focused
-            for index, (label, value) in enumerate(valid_choices):
+            for index, (label, value) in enumerate(valid_options):
                 if value == prev_value:
                     self._focused_index = index
                     break
@@ -94,9 +112,9 @@ class ChoiceJob(JobBase):
 
     def get_index(self, thing):
         """
-        Return index in :attr:`choices`
+        Return index in :attr:`options`
 
-        :param thing: Identifier of a choice in :attr:`choices`
+        :param thing: Identifier of a option in :attr:`options`
 
             This can be:
 
@@ -106,32 +124,32 @@ class ChoiceJob(JobBase):
             an index (:class:`int`)
                Return `thing`, but limited to the minimum/maximum valid index.
 
-            one of the 2-tuples in :attr:`choices`
-                Return the index of `thing` in :attr:`choices`.
+            one of the 2-tuples in :attr:`options`
+                Return the index of `thing` in :attr:`options`.
 
-            an item of one of the 2-tuples in :attr:`choices`
-                Return the index of the first 2-tuple in :attr:`choices` that
+            an item of one of the 2-tuples in :attr:`options`
+                Return the index of the first 2-tuple in :attr:`options` that
                 contains `thing`.
 
             a :func:`regular expression <re.compile>`
-                Return the index of the first 2-tuple in :attr:`choices` that
+                Return the index of the first 2-tuple in :attr:`options` that
                 contains something that matches `thing`. Non-string values are
                 converted to :class:`str` for matching against the regular
                 expression.
 
-        :raise ValueError: if `thing` is not found in :attr:`choices`
+        :raise ValueError: if `thing` is not found in :attr:`options`
         """
         if thing is None:
             return None
 
         elif isinstance(thing, int):
-            return max(0, min(thing, len(self.choices) - 1))
+            return max(0, min(thing, len(self.options) - 1))
 
-        elif thing in self.choices:
-            return self.choices.index(thing)
+        elif thing in self.options:
+            return self.options.index(thing)
 
         else:
-            for i, (label, value) in enumerate(self.choices):
+            for i, (label, value) in enumerate(self.options):
                 # Focus by human-readable text or value
                 if thing == label or thing == value:
                     return i
@@ -142,42 +160,42 @@ class ChoiceJob(JobBase):
                     if thing.search(label) or thing.search(value_str):
                         return i
 
-        raise ValueError(f'No such choice: {thing!r}')
+        raise ValueError(f'No such option: {thing!r}')
 
-    def get_choice(self, thing):
+    def get_option(self, thing):
         """
-        Return item in :attr:`choices`
+        Return item in :attr:`options`
 
         :param thing: See :meth:`get_index`
 
-        If `thing` is `None`, return the currently focused choice, which is
+        If `thing` is `None`, return the currently focused option, which is
         indicated by :attr:`focused_index`.
         """
-        choice_index = self.get_index(thing)
-        if choice_index is None:
-            choice_index = self.focused_index
-        return self.choices[choice_index]
+        option_index = self.get_index(thing)
+        if option_index is None:
+            option_index = self.focused_index
+        return self.options[option_index]
 
     @property
     def focused_index(self):
-        """Index of currently focused choice in :attr:`choices`"""
+        """Index of currently focused option in :attr:`options`"""
         return getattr(self, '_focused_index', None)
 
     @property
     def focused(self):
         """
-        Currently focused choice (2-tuple)
+        Currently focused option (2-tuple)
 
         This property can be set to anything that is a valid value for
         :meth:`get_index`.
         """
         focused_index = self.focused_index
         if focused_index is not None:
-            return self.choices[focused_index]
+            return self.options[focused_index]
 
     @focused.setter
     def focused(self, focused):
-        # focused_index can't be set to None, so we default to first choice
+        # focused_index can't be set to None, so we default to first option
         self._focused_index = self.get_index(focused)
         if self._focused_index is None:
             self._focused_index = 0
@@ -185,20 +203,20 @@ class ChoiceJob(JobBase):
 
     @property
     def autodetected_index(self):
-        """Index of autodetected choice in :attr:`choices` or `None`"""
+        """Index of autodetected option in :attr:`options` or `None`"""
         return getattr(self, '_autodetected_index', None)
 
     @property
     def autodetected(self):
         """
-        Autodetected choice (2-tuple) or `None`
+        Autodetected option (2-tuple) or `None`
 
         This property can be set to anything that is a valid value for
         :meth:`get_index`.
         """
         autodetected_index = self.autodetected_index
         if autodetected_index is not None:
-            return self.choices[autodetected_index]
+            return self.options[autodetected_index]
 
     @autodetected.setter
     def autodetected(self, autodetected):
@@ -211,12 +229,10 @@ class ChoiceJob(JobBase):
         User-chosen value if job is finished, `None` otherwise
 
         While :attr:`~.base.JobBase.output` contains the user-readable string
-        (first item of the chosen 2-tuple in :attr:`choices`), this is the
+        (first item of the chosen 2-tuple in :attr:`options`), this is the
         object attached to it (second item).
         """
-        choice = getattr(self, '_choice', None)
-        if choice is not None:
-            return choice
+        return getattr(self, '_choice', None)
 
     def _set_choice(self, choice):
         # This method is called via `output` signal (see initialize()), which is
@@ -224,72 +240,77 @@ class ChoiceJob(JobBase):
         if hasattr(self, '_choice'):
             raise RuntimeError(f'{self.name}: Choice was already made: {self.choice}')
         else:
-            label, value = self.get_choice(choice)
+            label, value = self.get_option(choice)
             self._choice = value
 
     def make_choice(self, thing):
         """
-        Make a choice and :meth:`~.JobBase.finish` this job
+        Make a choice and :meth:`~.JobBase.finalize` this job
 
-        :param thing: See :meth:`get_choice`
+        :param thing: See :meth:`get_option`
 
         After this method is called, this job :attr:`~.JobBase.is_finished`,
         :attr:`~.JobBase.output` contains the human-readable label of the choice
         and :attr:`choice` is the machine-readable value of the choice.
         """
-        label, value = self.get_choice(thing)
+        label, value = self.get_option(thing)
         self.send(label)
-        self.finish()
+        self.finalize()
 
     def set_label(self, identifier, new_label):
         """
-        Assign new label to choice
+        Assign new label to option
 
-        :param identifier: Choice (2-tuple of (<current label>, <value>) or the
-            current label or value of a choice
-        :param new_label: New label for the choice defined by `identifier`
+        :param identifier: Option (2-tuple of `(<current label>, <value>)`) or the
+            current label or value of an option
+        :param new_label: New label for the option defined by `identifier`
 
-        Do nothing if `identifier` doesn't match any choice.
+        Do nothing if `identifier` doesn't match any option.
         """
-        new_choices = []
-        for label, value in tuple(self.choices):
+        new_options = []
+        for label, value in tuple(self.options):
             if identifier == label or identifier == value or identifier == (label, value):
-                new_choices.append((str(new_label), value))
+                new_options.append((str(new_label), value))
             else:
-                new_choices.append((label, value))
-        if self.choices != new_choices:
-            self.choices = new_choices
+                new_options.append((label, value))
+        if self.options != new_options:
+            self.options = new_options
 
-    def initialize(self, *, name, label, choices, focused=None, autodetected=None,
-                   autodetect=None, autodetect_and_finish=None):
+    def initialize(self, *, name, label, options, question=None,
+                   focused=None, autodetected=None,
+                   autodetect=None, autofinish=False):
         """
         Set internal state
 
         :param name: Name for internal use
         :param label: Name for user-facing use
-        :param choices: Iterable of choices the user can make
+        :param options: Iterable of options the user can pick from
+        :param question: Any text that is displayed alongside the `options`
         :param focused: See :attr:`focused`
         :param autodetected: See :attr:`autodetected`
-        :param autodetect: Callable that sets :attr:`autodetected` in
-            :meth:`.JobBase.start`
+        :param autodetect: Callable that sets :attr:`autodetected` when job is
+            started
 
             `autodetect` gets the job instance (``self``) as a positional
             argument.
-        :param autodetect_and_finish: Same as `autodetect`, but :meth:`finish`
-            this job if the returned value is not `None`
+        :param autofinish: Whether to call :meth:`make_choice` if `autodetect`
+            returns anything that is not `None`
 
-        :raise ValueError: if `choices` is shorter than 2 or `focused` is
+        :raise ValueError: if `options` is shorter than 2 or `focused` is
             invalid
         """
         self._name = str(name)
         self._label = str(label)
         self._autodetect = autodetect
-        self._autodetect_and_finish = autodetect_and_finish
+        self._autofinish = bool(autofinish)
 
         self.signal.add('dialog_updated')
+        self.signal.add('autodetecting')
+        self.signal.add('autodetected')
         self.signal.register('output', self._set_choice)
 
-        self.choices = choices
+        self.options = options
+        self.question = question
         self.autodetected = autodetected
 
         if focused is not None:
@@ -297,22 +318,33 @@ class ChoiceJob(JobBase):
         else:
             self.focused = autodetected
 
-    def start(self):
-        super().start()
+    async def run(self):
+        # Always emitting the autodetecting/autodetected signals makes things
+        # more reliable and the UI doesn't have to handle special cases.
+        self.signal.emit('autodetecting')
 
-        # Job is not started if it is disabled.
-        # Job is finished right away if output was cached previously.
-        if self.is_started and not self.is_finished:
-            if self._autodetect:
-                autodetected = self._autodetect(self)
-                if autodetected is not None:
-                    self.autodetected = self.focused = autodetected
+        if self._autodetect:
+            autodetected = await self._call_autodetect()
+            if autodetected is not None:
+                self.autodetected = self.focused = autodetected
 
-            if self._autodetect_and_finish:
-                autodetected = self._autodetect_and_finish(self)
-                if autodetected is not None:
-                    self.autodetected = self.focused = autodetected
-                    self.make_choice(self.focused)
+        self.signal.emit('autodetected')
+
+        if self._autofinish and self.autodetected is not None:
+            self.make_choice(self.autodetected)
+
+        # Wait for make_choice() getting called. If it was already called via
+        # `autofinish`, finalization() returns immediately.
+        await self.finalization()
+
+    async def _call_autodetect(self):
+        if self._autodetect is not None:
+            if inspect.iscoroutinefunction(self._autodetect):
+                return await self._autodetect(self)
+            elif callable(self._autodetect):
+                return self._autodetect(self)
+            else:
+                raise RuntimeError(f'Bad autodetect value: {self._autodetect!r}')
 
 
 class TextFieldJob(JobBase):
@@ -403,16 +435,16 @@ class TextFieldJob(JobBase):
         :param name: Name for internal use
         :param label: Name for user-facing use
         :param text: Initial text or callable (synchronous or asynchronous)
-            which will be called when the job is :meth:`executed
-            <upsies.jobs.base.JobBase.execute>`. The return value is used as the
+            which will be called when the job is :meth:`started
+            <upsies.jobs.base.JobBase.start>`. The return value is used as the
             initial text.
         :param default: Text to use if `text` is `None`, returns `None` or
             raises `nonfatal_exceptions`
-        :param finish_on_success: Whether to call :meth:`finish` after setting
-            :attr:`text` to `text`
+        :param finish_on_success: Whether to call :meth:`~.JobBase.finalize`
+            after setting :attr:`text` to `text`
 
-            .. note:: :meth:`finish` is not called when :attr:`text` is set to
-                      `default`.
+            .. note:: :meth:`~.JobBase.finalize` is not called if :attr:`text`
+                      is set to `default`.
         :param nonfatal_exceptions: Sequence of exception classes that may be
             raised by `text` without crashing
 
@@ -446,30 +478,25 @@ class TextFieldJob(JobBase):
             # Set text attribute immediately
             self.text = text
 
-        # Get text from callable when job is executed
-        self._execute_arguments = (text, default, finish_on_success, nonfatal_exceptions)
+        self._run_arguments = (text, default, finish_on_success, nonfatal_exceptions)
 
-    def execute(self):
-        text, default, finish_on_success, nonfatal_exceptions = self._execute_arguments
+    async def run(self):
+        text, default, finish_on_success, nonfatal_exceptions = self._run_arguments
 
         if inspect.isawaitable(text):
-            self.attach_task(
-                self.fetch_text(
-                    coro=text,
-                    default=default,
-                    finish_on_success=finish_on_success,
-                    nonfatal_exceptions=nonfatal_exceptions,
-                )
+            await self.fetch_text(
+                coro=text,
+                default=default,
+                finish_on_success=finish_on_success,
+                nonfatal_exceptions=nonfatal_exceptions,
             )
 
         elif inspect.iscoroutinefunction(text):
-            self.attach_task(
-                self.fetch_text(
-                    coro=text(),
-                    default=default,
-                    finish_on_success=finish_on_success,
-                    nonfatal_exceptions=nonfatal_exceptions,
-                )
+            await self.fetch_text(
+                coro=text(),
+                default=default,
+                finish_on_success=finish_on_success,
+                nonfatal_exceptions=nonfatal_exceptions,
             )
 
         else:
@@ -480,6 +507,11 @@ class TextFieldJob(JobBase):
                 nonfatal_exceptions=nonfatal_exceptions,
             )
 
+        # If fetch_text() or set_text() failed for some reason, we must not
+        # finish until the user fixed the situation, usually by entering text
+        # manually that we failed to autodetect.
+        await self.finalization()
+
     def set_text(self, text, default=None, finish_on_success=False, nonfatal_exceptions=()):
         """
         Change :attr:`text` value
@@ -588,11 +620,18 @@ class TextFieldJob(JobBase):
         to :meth:`warn` and do not finish. Otherwise, pass `output` to
         :meth:`~.base.JobBase.send` and :meth:`~.base.JobBase.finish` this job.
         """
+        # Normalize, e.g. remove leading and trailing whitespace.
         output = self._normalizer(output)
+
+        # Remove any warning from previously failed validation.
+        self.clear_warnings()
+
+        # Validate normalized output.
         try:
             self._validator(output)
         except ValueError as e:
             self.warn(e)
         else:
+            # Send normalized and validated output.
             super().send(output)
-            super().finish()
+            self.finalize()
