@@ -2,7 +2,7 @@
 Upload images to image hosting services
 """
 
-from .. import errors
+from .. import errors, utils
 from ..utils.imghosts import ImageHostBase
 from . import QueueJobBase
 
@@ -17,15 +17,16 @@ class ImageHostJob(QueueJobBase):
     label = 'Image URLs'
 
     # Don't cache output and rely on caching in ImageHostBase. Otherwise, a
-    # single failed/cancelled upload would throw away all the gathered URLs.
+    # single failed/cancelled upload would throw away all the gathered URLs
+    # because nothing is cached if a job fails.
     cache_id = None
 
-    def initialize(self, *, imghost, images_total=0, enqueue=()):
+    def initialize(self, *, imghosts, images_total=0, enqueue=()):
         """
         Validate arguments and set internal state
 
-        :param ImageHostBase imghost: Return value of
-            :func:`.utils.imghosts.imghost`
+        :param imghosts: Sequence of :class:`ImageHostBase` subclass instances
+            (see :func:`.utils.imghosts.imghost`)
         :param enqueue: Sequence of image paths (see
             :class:`~.base.QueueJobBase`)
         :param images_total: Number of images that are going to be uploaded. The
@@ -35,17 +36,19 @@ class ImageHostJob(QueueJobBase):
         If `enqueue` is given, the job finishes after all images are uploaded.
 
         If `enqueue` is not given, calls to :meth:`upload` are expected and
-        :meth:`finalize` must be called after the last call.
+        :meth:`~.QueueJobBase.close` must be called after the last call.
 
         `enqueue` and `images_total` must not be given at the same time.
         """
         if enqueue and images_total:
             raise RuntimeError('You must not give both arguments "enqueue" and "images_total".')
         else:
-            assert isinstance(imghost, ImageHostBase), f'Not an ImageHostBase: {imghost!r}'
-            imghost.cache_directory = self.cache_directory
-            self._imghost = imghost
-            self._images_uploaded = 0
+            for imghost in imghosts:
+                assert isinstance(imghost, ImageHostBase), f'Not an ImageHostBase: {imghost!r}'
+                # Force image hosts to cache image URLs in our cache directory
+                imghost.cache_directory = self.cache_directory
+
+            self._imghosts = imghosts
             self._uploaded_images = []
             self._urls_by_file = {}
             if images_total > 0:
@@ -54,17 +57,43 @@ class ImageHostJob(QueueJobBase):
                 self.images_total = len(enqueue)
 
     async def handle_input(self, image_path):
-        try:
-            info = await self._imghost.upload(image_path, cache=not self.ignore_cache)
-        except errors.RequestError as e:
-            self.error(e)
+        if len(self._imghosts) <= 1:
+            await self._upload_to_one(image_path)
         else:
-            _log.debug('Uploaded image: %r', info)
-            self._images_uploaded += 1
-            self._uploaded_images.append(info)
-            self._urls_by_file[image_path] = info
-            image_url = str(info)
-            self.send(image_url)
+            await self._upload_to_any(image_path)
+
+    async def _upload_to_one(self, image_path):
+        # Upload to 1 or 0 image hosts and error out immediately if that fails.
+        for imghost in self._imghosts:
+            try:
+                await self._upload(image_path, imghost)
+            except errors.RequestError as e:
+                self.error(f'{imghost.name}: Upload failed: {utils.fs.basename(image_path)}: {e}')
+
+    async def _upload_to_any(self, image_path):
+        # Try each image host and stop on first successful upload.
+        # Only warn about upload failures.
+        fail = False
+        for imghost in self._imghosts:
+            try:
+                await self._upload(image_path, imghost)
+            except errors.RequestError as e:
+                _log.debug('Failed to upload %s to %s: %r', image_path, imghost.name, e)
+                self.warn(f'{imghost.name}: Upload failed: {utils.fs.basename(image_path)}: {e}')
+                fail = True
+            else:
+                fail = False
+                break
+        if fail:
+            self.error('All upload attempts failed.')
+
+    async def _upload(self, image_path, imghost):
+        info = await imghost.upload(image_path, cache=not self.ignore_cache)
+        _log.debug('Uploaded image: %r', info)
+        self._uploaded_images.append(info)
+        self._urls_by_file[image_path] = info
+        image_url = str(info)
+        self.send(image_url)
 
     @property
     def exit_code(self):
@@ -87,13 +116,13 @@ class ImageHostJob(QueueJobBase):
 
     @property
     def urls_by_file(self):
-        """Map image file paths to image URLs or uploaded images so far"""
+        """Mapping of image file paths to image URLs"""
         return self._urls_by_file.copy()
 
     @property
     def images_uploaded(self):
         """Number of uploaded images"""
-        return self._images_uploaded
+        return len(self._uploaded_images)
 
     @property
     def images_total(self):
