@@ -26,20 +26,36 @@ class MediainfoJob(JobBase):
     # the only/final job.
     hidden = True
 
+    _DEFAULT_FORMAT = '{MEDIAINFO}'
+
     @property
     def cache_id(self):
         """Final segment of `content_path` and `from_all_videos` argument"""
-        return (
+        cache_id = [
             utils.fs.basename(self._content_path),
             'from_all_videos' if self._from_all_videos else 'from_first_video',
-            [
+        ]
+
+        if self._exclude_files:
+            cache_id.append([
                 # `re.Pattern` objects or glob strings
                 (exclude.pattern if hasattr(exclude, 'pattern') else str(exclude))
                 for exclude in self._exclude_files
-            ],
-        )
+            ])
 
-    def initialize(self, *, content_path, from_all_videos=False, exclude_files=()):
+        if self._format != self._DEFAULT_FORMAT:
+            cache_id.append(f'format={self._format}')
+
+        return cache_id
+
+    def initialize(
+            self,
+            *,
+            content_path,
+            from_all_videos=False,
+            exclude_files=(),
+            format='{MEDIAINFO}',
+    ):
         """
         Set internal state
 
@@ -55,22 +71,20 @@ class MediainfoJob(JobBase):
 
             .. note:: Non-video files and stuff like `Sample.mkv` are always
                       excluded by :func:`.video.filter_main_videos`.
+        :param format: String that contains the placeholder ``"{MEDIAINFO}"``,
+            which is replaced by the actual mediainfo
+
+            Any other placeholders are ignored.
         """
         self._content_path = content_path
         self._from_all_videos = from_all_videos
         self._exclude_files = exclude_files
+        self._format = format
         self._mediainfos_by_file = {}
         self.signal.add('mediainfo', record=True)
         self.signal.register('mediainfo', self._store_mediainfos_by_file)
 
-    def execute(self):
-        """Call ``mediainfo`` in an asynchronous thread"""
-        self.attach_task(
-            coro=self._generate_mediainfos(),
-            finish_when_done=True,
-        )
-
-    async def _generate_mediainfos(self):
+    async def run(self):
         # Exclude files specified by the user
         video_filepaths = utils.torrent.filter_files(
             content_path=self._content_path,
@@ -78,6 +92,9 @@ class MediainfoJob(JobBase):
         )
         # Exclude irrelevant files (non-videos, sample.mkv, etc)
         video_filepaths = utils.video.filter_main_videos(video_filepaths)
+        if not video_filepaths:
+            self.error('No video files found')
+
         for video_filepath in video_filepaths:
             # DVD
             if utils.fs.dirname(video_filepath).endswith(f'{os.sep}VIDEO_TS'):
@@ -111,6 +128,9 @@ class MediainfoJob(JobBase):
             if not self._from_all_videos:
                 break
 
+    def send(self, mediainfo):
+        super().send(self._format.replace('{MEDIAINFO}', mediainfo))
+
     def _store_mediainfos_by_file(self, filepath, mediainfo):
         self._mediainfos_by_file[filepath] = mediainfo
 
