@@ -40,17 +40,14 @@ class SceneSearchJob(JobBase):
         self._content_path = content_path
         self.signal.add('search_results')
 
-    def execute(self):
-        """Create background search task"""
-        self.attach_task(self._search(), finish_when_done=True)
-
-    async def _search(self):
+    async def run(self):
         try:
             results = await self._predb.search(
                 query=self._content_path,
                 only_existing_releases=False,
             )
         except (errors.RequestError, errors.SceneError) as e:
+            _log.debug('Caught %r', e)
             self.error(e)
         else:
             if results:
@@ -59,8 +56,6 @@ class SceneSearchJob(JobBase):
             else:
                 self.error('No results')
             self.signal.emit('search_results', results)
-        finally:
-            self.finish()
 
 
 class SceneCheckJob(JobBase):
@@ -118,6 +113,7 @@ class SceneCheckJob(JobBase):
         self._content_path = content_path
         self._predetermined_result = None if force is None else bool(force)
         self._is_scene_release = None
+        self._check_tasks = []
         self.signal.add('ask_release_name')
         self.signal.add('ask_is_scene_release')
         self.signal.add('checked')
@@ -135,27 +131,32 @@ class SceneCheckJob(JobBase):
             # Something like SceneRenamedError or SceneAbbreviatedFilenameError
             self.error(e)
 
-    def execute(self):
-        """Start asynchronous background worker task"""
+    async def run(self):
         if self._predetermined_result is None:
-            self.attach_task(
-                self._catch_errors(
-                    self._verify(),
-                ),
+            # We create a task so we can cancel it if the user makes a decision
+            # while we are checking, e.g. by pressing a key.
+            self._check_tasks.append(
+                self.add_task(
+                    self._catch_errors(
+                        self._verify()
+                    )
+                )
             )
-        elif self._predetermined_result is True:
-            self.finalize(types.SceneCheckResult.true)
-        elif self._predetermined_result is False:
-            self.finalize(types.SceneCheckResult.false)
+        elif self._predetermined_result:
+            self.set_result(types.SceneCheckResult.true)
         else:
-            raise RuntimeError(f'Unexpected predetermined_result: {self._predetermined_result!r}')
+            self.set_result(types.SceneCheckResult.false)
+
+        # Because this job can be interactive, we must wait for a finalize()
+        # call that happens when a decision is made (manually or automatically).
+        await self.finalization()
 
     async def _verify(self):
         _log.debug('Verifying release: %r', self._content_path)
 
         if await self._predb.is_scene_release(self._content_path) is types.SceneCheckResult.false:
             # Try to get a true negative as fast as possible
-            self.finalize(types.SceneCheckResult.false)
+            self.set_result(types.SceneCheckResult.false)
 
         elif predbs.is_mixed_season_pack(self._content_path):
             # We don't want to prompt the user if this is a mixed release
@@ -187,9 +188,11 @@ class SceneCheckJob(JobBase):
         """
         _log.debug('User selected release name: %r', release_name)
         if release_name:
-            self.attach_task(
-                self._catch_errors(
-                    self._verify_release(release_name)
+            self._check_tasks.append(
+                self.add_task(
+                    self._catch_errors(
+                        self._verify_release(release_name)
+                    )
                 )
             )
         else:
@@ -208,19 +211,19 @@ class SceneCheckJob(JobBase):
 
         serious_errors = [e for e in exceptions if not isinstance(e, errors.SceneMissingInfoError)]
         for e in serious_errors:
-            self.error(e, finish=False)
+            self.error(e)
 
-        if serious_errors:
-            self.finish()
-        elif is_scene_release in (types.SceneCheckResult.true, types.SceneCheckResult.false):
-            self.finalize(is_scene_release)
-        else:
-            self.signal.emit('ask_is_scene_release', is_scene_release)
+        # Serious errors terminate this job.
+        if not serious_errors:
+            if is_scene_release in (types.SceneCheckResult.true, types.SceneCheckResult.false):
+                self.set_result(is_scene_release)
+            else:
+                self.signal.emit('ask_is_scene_release', is_scene_release)
 
-    def finalize(self, is_scene_release):
+    def set_result(self, is_scene_release):
         """
         Make the final decision of whether this is a scene release or not and
-        :meth:`finish`
+        finish this job
 
         Must be called by the UI in the handler of the signal
         ``ask_is_scene_release``.
@@ -229,10 +232,23 @@ class SceneCheckJob(JobBase):
         """
         _log.debug('Final scene check decision: %r', is_scene_release)
         self.signal.emit('checked', is_scene_release)
+
         if is_scene_release is types.SceneCheckResult.true:
             self.send('Scene release')
         elif is_scene_release is types.SceneCheckResult.false:
             self.send('Not a scene release')
         else:
             self.send('May be a scene release')
-        self.finish()
+
+        # Unblock run() and finish the job.
+        self.finalize()
+
+    def stop_checking(self):
+        """
+        Cancel any ongoing checking tasks
+
+        This method should be called by the UI if the user makes a decision
+        while we are trying to autodetect.
+        """
+        for task in self._check_tasks:
+            task.cancel()
