@@ -67,28 +67,28 @@ class ScreenshotsJob(JobBase):
         """
         self._content_path = content_path
         self._exclude_files = exclude_files
-        self._screenshots_created = 0
-        self._screenshots_total = -1
         self._timestamps = timestamps
         self._count = count
         self._from_all_videos = from_all_videos
         self._optimize = optimize
+        self._screenshots_created = 0
+        self._screenshots_total = -1
         self._screenshots_process = None
         self._optimize_process = None
         self._screenshots_by_file = collections.defaultdict(lambda: [])
         self.signal.add('screenshots_total', record=True)
 
-    def execute(self):
-        """
-        Execute screenshot creation subprocess
-
-        The subprocess also picks video file(s) and timestamps.
-        """
+    async def run(self):
+        """Execute subprocesses for screenshot creation and optimization"""
+        # Execute subprocesses
+        self._execute_screenshots_process()
         if self._optimize not in ('none', None):
-            self._execute_screenshots_process()
             self._execute_optimize_process()
-        else:
-            self._execute_screenshots_process()
+
+        # Wait for subprocesses
+        await self._screenshots_process.join()
+        if self._optimize_process:
+            await self._optimize_process.join()
 
     def _execute_screenshots_process(self):
         self._screenshots_process = daemon.DaemonProcess(
@@ -113,8 +113,8 @@ class ScreenshotsJob(JobBase):
             name='_optimize_process',
             target=_optimize_process,
             kwargs={
-                # Use default optimization level if None is provided
                 'level': self._optimize,
+                'overwrite': self.ignore_cache,
                 # Ignore missing dependecy if we do "default" optimization
                 'ignore_dependency_error': (
                     True
@@ -134,41 +134,48 @@ class ScreenshotsJob(JobBase):
                 self.signal.emit('screenshots_total', self._screenshots_total)
 
             if 'screenshot_path' in info:
-                self._screenshots_by_file[info['video_path']].append(info['screenshot_path'])
-
                 if self._optimize_process:
-                    _log.debug('Optimizing %s: %.2f KiB',
+                    _log.debug('Screenshot: %s: %.2f KiB',
                                info['screenshot_path'],
                                fs.file_size(info['screenshot_path']) / 1024)
-                    self._optimize_process.send(daemon.MsgType.info, info['screenshot_path'])
+                    self._optimize_process.send(
+                        daemon.MsgType.info,
+                        (info['screenshot_path'], info['video_path']),
+                    )
 
                 else:
-                    self._screenshots_created += 1
-                    self.send(info['screenshot_path'])
+                    _log.debug('Screenshot: %s: %.2f KiB',
+                               info['screenshot_path'],
+                               fs.file_size(info['screenshot_path']) / 1024)
+                    self.send(info['screenshot_path'], info['video_path'])
 
             elif 'optimized_screenshot_path' in info:
                 _log.debug('Optimized %s: %.2f KiB',
                            info['optimized_screenshot_path'],
                            fs.file_size(info['optimized_screenshot_path']) / 1024)
-                self._screenshots_created += 1
-                self.send(info['optimized_screenshot_path'])
+                self.send(info['optimized_screenshot_path'], info['video_path'])
 
             if self.screenshots_created == self.screenshots_total:
-                self.finish()
+                self._screenshots_process.stop()
+                if self._optimize_process:
+                    self._optimize_process.stop()
 
     def _handle_error(self, error):
         if isinstance(error, BaseException):
-            self.exception(error)
+            raise error
         else:
             self.error(error)
 
-    def finish(self):
-        """Terminate screenshot creation subprocess and finish"""
+    def terminate(self):
+        """
+        Stop screenshot creation and optimization subprocesses before
+        terminating the job
+        """
         if self._screenshots_process:
             self._screenshots_process.stop()
         if self._optimize_process:
             self._optimize_process.stop()
-        super().finish()
+        super().terminate()
 
     @property
     def exit_code(self):
@@ -183,7 +190,7 @@ class ScreenshotsJob(JobBase):
                     # If we have cached output, assume the cached number of
                     # screenshots is what the user wanted because the output of
                     # unsuccessful jobs is not cached (see
-                    # JobBase._write_output_cache()).
+                    # JobBase._write_cache()).
                     return 0
                 else:
                     return 1
@@ -206,14 +213,14 @@ class ScreenshotsJob(JobBase):
 
         See also :attr:`from_all_videos`.
 
-        Setting this property when this job :attr:`~.JobBase.is_started` raises
+        Setting this property when this job :attr:`~.JobBase.was_started` raises
         :class:`RuntimeError`.
         """
         return self._content_path
 
     @content_path.setter
     def content_path(self, content_path):
-        if self.is_started:
+        if self.was_started:
             raise RuntimeError('Cannot set content_path after job has been started')
         else:
             self._content_path = content_path
@@ -225,14 +232,14 @@ class ScreenshotsJob(JobBase):
 
         See :meth:`initialize` for more information.
 
-        Setting this property when this job :attr:`~.JobBase.is_started` raises
+        Setting this property when this job :attr:`~.JobBase.was_started` raises
         :class:`RuntimeError`.
         """
         return self._exclude_files
 
     @exclude_files.setter
     def exclude_files(self, exclude_files):
-        if self.is_started:
+        if self.was_started:
             raise RuntimeError('Cannot set exclude_files after job has been started')
         else:
             self._exclude_files = exclude_files
@@ -242,14 +249,14 @@ class ScreenshotsJob(JobBase):
         """
         Whether to make screenshots from all video files or only the first
 
-        Setting this property when this job :attr:`~.JobBase.is_started` raises
+        Setting this property when this job :attr:`~.JobBase.was_started` raises
         :class:`RuntimeError`.
         """
         return self._from_all_videos
 
     @from_all_videos.setter
     def from_all_videos(self, from_all_videos):
-        if self.is_started:
+        if self.was_started:
             raise RuntimeError('Cannot set from_all_videos after job has been started')
         else:
             self._from_all_videos = from_all_videos
@@ -259,14 +266,14 @@ class ScreenshotsJob(JobBase):
         """
         How many screenshots to make per video file
 
-        Setting this property when this job :attr:`~.JobBase.is_started` raises
+        Setting this property when this job :attr:`~.JobBase.was_started` raises
         :class:`RuntimeError`.
         """
         return self._count
 
     @count.setter
     def count(self, count):
-        if self.is_started:
+        if self.was_started:
             raise RuntimeError('Cannot set count after job has been started')
         else:
             self._count = count
@@ -276,14 +283,14 @@ class ScreenshotsJob(JobBase):
         """
         Specific list of timestamps to make
 
-        Setting this property when this job :attr:`~.JobBase.is_started` raises
+        Setting this property when this job :attr:`~.JobBase.was_started` raises
         :class:`RuntimeError`.
         """
         return self._timestamps
 
     @timestamps.setter
     def timestamps(self, timestamps):
-        if self.is_started:
+        if self.was_started:
             raise RuntimeError('Cannot set timestamps after job has been started')
         else:
             self._timestamps = timestamps
@@ -315,10 +322,17 @@ class ScreenshotsJob(JobBase):
             for video_path, screenshot_paths in self._screenshots_by_file.items()
         }
 
+    def send(self, screenshot_path, video_path):
+        self._screenshots_created += 1
+        self._screenshots_by_file[video_path].append(screenshot_path)
+        return super().send(screenshot_path)
 
-def _screenshots_process(output_queue, input_queue,
-                         *, content_path, exclude_files, timestamps, count, from_all_videos,
-                         output_dir, overwrite):
+
+def _screenshots_process(
+        output_queue, input_queue,
+        *,
+        content_path, exclude_files, timestamps, count, from_all_videos, output_dir, overwrite,
+):
     video_files = list(_get_video_files(
         output_queue,
         content_path=content_path,
@@ -355,41 +369,6 @@ def _screenshots_process(output_queue, input_queue,
         pass
 
 
-def _optimize_process(output_queue, input_queue, *, level, ignore_dependency_error):
-    # Get as many queued screenshots as possible. Check if there's a termination
-    # sentinel before optimizing each screenshot to avoid processing all the
-    # screenshots before Ctrl-C takes effect.
-    msgs = []
-    while True:
-        new_msgs = _read_queue_until_empty(input_queue=input_queue)
-        msgs.extend(new_msgs)
-
-        if any(typ is daemon.MsgType.terminate for typ, msg_ in msgs):
-            break
-
-        elif msgs:
-            typ_, screenshot_file = msgs.pop(0)
-            try:
-                image.optimize(screenshot_file, level=level)
-
-            except errors.ImageOptimizeError as e:
-                output_queue.put((daemon.MsgType.error, str(e)))
-
-            except errors.DependencyError as e:
-                if ignore_dependency_error:
-                    # Act like we optimized `screenshot_file`
-                    output_queue.put((daemon.MsgType.info, {
-                        'optimized_screenshot_path': screenshot_file,
-                    }))
-                else:
-                    raise e
-
-            else:
-                output_queue.put((daemon.MsgType.info, {
-                    'optimized_screenshot_path': screenshot_file,
-                }))
-
-
 def _get_video_files(output_queue, *, content_path, exclude_files):
     # Make list of appropriate video file(s) from `content_path`
     try:
@@ -432,21 +411,29 @@ def _make_screenshot(output_queue, *, video_file, timestamp, output_dir, overwri
         output_dir,
         fs.basename(video_file) + f'.{timestamp}.png',
     )
-    try:
-        actual_screenshot_file = image.screenshot(
-            video_file=video_file,
-            screenshot_file=screenshot_file,
-            timestamp=timestamp,
-            overwrite=overwrite,
-        )
-    except errors.ScreenshotError as e:
-        output_queue.put((daemon.MsgType.error, str(e)))
-    else:
+
+    # Screenshot already exists?
+    if not overwrite and os.path.exists(screenshot_file):
         output_queue.put((daemon.MsgType.info, {
             'video_path': video_file,
-            'screenshot_path': actual_screenshot_file,
+            'screenshot_path': screenshot_file,
         }))
 
+    else:
+        try:
+            return_value = image.screenshot(
+                video_file=video_file,
+                screenshot_file=screenshot_file,
+                timestamp=timestamp,
+            )
+        except errors.ScreenshotError as e:
+            output_queue.put((daemon.MsgType.error, str(e)))
+        else:
+            output_queue.put((daemon.MsgType.info, {
+                'video_path': video_file,
+                'screenshot_path': return_value,
+            }))
+
 
 def _map_timestamps(*, video_files, timestamps, count):
     # Map each video_file to a sequence of timestamps
@@ -521,3 +508,81 @@ def _read_queue_until_empty(*, input_queue):
             return msgs
         else:
             msgs.append((typ, msg))
+
+
+def _optimize_process(
+        output_queue, input_queue,
+        *,
+        level, overwrite, ignore_dependency_error,
+):
+    # Get as many queued screenshots as possible. Check if there's a termination
+    # sentinel before optimizing each screenshot to avoid processing all the
+    # screenshots before Ctrl-C takes effect.
+    msgs = []
+    while True:
+        new_msgs = _read_queue_until_empty(input_queue=input_queue)
+        msgs.extend(new_msgs)
+
+        if any(typ is daemon.MsgType.terminate for typ, msg_ in msgs):
+            break
+
+        elif msgs:
+            typ_, (screenshot_file, video_file) = msgs.pop(0)
+            _optimize_screenshot(
+                output_queue=output_queue,
+                screenshot_file=screenshot_file,
+                video_file=video_file,
+                level=level,
+                overwrite=overwrite,
+                ignore_dependency_error=ignore_dependency_error,
+            )
+
+
+def _optimize_screenshot(
+        output_queue,
+        *,
+        screenshot_file, video_file, level, overwrite, ignore_dependency_error,
+):
+    output_file = os.path.join(
+        fs.dirname(screenshot_file),
+        (
+            fs.basename(fs.strip_extension(screenshot_file))
+            + '.'
+            + f'optimized={level}'
+            + '.'
+            + fs.file_extension(screenshot_file)
+        )
+    )
+
+    if not overwrite and os.path.exists(output_file):
+        output_queue.put((daemon.MsgType.info, {
+            'optimized_screenshot_path': output_file,
+            'video_path': video_file,
+        }))
+
+    else:
+        try:
+            return_value = image.optimize(
+                screenshot_file,
+                level=level,
+                output_file=output_file,
+            )
+
+        except errors.ImageOptimizeError as e:
+            output_queue.put((daemon.MsgType.error, str(e)))
+
+        except errors.DependencyError as e:
+            if ignore_dependency_error:
+                # Act like we optimized `screenshot_file`
+                output_queue.put((daemon.MsgType.info, {
+                    'optimized_screenshot_path': screenshot_file,
+                    'video_path': video_file,
+                }))
+            else:
+                raise e
+
+        else:
+            output_queue.put((daemon.MsgType.info, {
+                'optimized_screenshot_path': return_value,
+                'video_path': video_file,
+            }))
