@@ -46,10 +46,8 @@ class SubmitJob(JobBase):
     name = 'submit'
     label = 'Submit'
 
-    @property
-    def cache_id(self):
-        """:attr:`.TrackerBase.name`"""
-        return self.kwargs['tracker'].name
+    # Don't cache output.
+    cache_id = None
 
     def initialize(self, *, tracker, tracker_jobs):
         """
@@ -61,12 +59,16 @@ class SubmitJob(JobBase):
         """
         assert isinstance(tracker, trackers.TrackerBase), f'Not a TrackerBase: {tracker!r}'
         assert isinstance(tracker_jobs, trackers.TrackerJobsBase), f'Not a TrackerJobsBase: {tracker_jobs!r}'
+
+        # Pass through signals from Tracker and TrackerJobs
         for t in (tracker, tracker_jobs):
             t.signal.register('warning', self.warn)
             t.signal.register('error', self.error)
             t.signal.register('exception', self.exception)
+
         self._tracker = tracker
         self._tracker_jobs = tracker_jobs
+
         for signal, message in (('logging_in', 'Logging in'),
                                 ('logged_in', 'Logged in'),
                                 ('uploading', 'Uploading'),
@@ -75,50 +77,47 @@ class SubmitJob(JobBase):
                                 ('logged_out', '')):
             self.signal.add(signal)
             self.signal.register(signal, lambda msg=message: setattr(self, 'info', msg))
+
         self.signal.register('finished', lambda _: setattr(self, 'info', ''))
         self.signal.register('error', lambda _: setattr(self, 'info', ''))
 
         # Create jobs_after_upload now so they can connect to other jobs,
         # e.g. add_torrent_job needs to register for create_torrent_job's output
         # before create_torrent_job finishes.
-        self.jobs_after_upload
+        self.enabled_jobs_after_upload
 
         # Start jobs_after_upload only if the upload succeeds. This also works
-        # nicely when this job is getting it's output from cache (execute() is
-        # not called).
+        # nicely when this job is getting it's output from cache.
         self.signal.register('output', self._start_jobs_after_upload)
 
     def _start_jobs_after_upload(self, _):
-        for job in self.jobs_after_upload:
+        for job in self.enabled_jobs_after_upload:
             job.start()
 
-    def execute(self):
-        """
-        Start the asynchronous background task that waits for jobs and wait for it
-        to finish
-        """
-        if not self.is_finished:
-            self.attach_task(self._run_jobs())
+    async def run(self):
+        # While there is any unfinished job, keep collecting all enabled+started
+        # jobs and wait for them. We do it like this because any job can enable
+        # and/or disable other jobs, and then we have to re-collect all enabled
+        # jobs.
+        def get_unfinished_jobs():
+            return tuple(
+                job
+                for job in self.enabled_jobs_before_upload
+                if job.was_started and not job.is_finished
+            )
 
-    async def _run_jobs(self):
-        # When all jobs are finished, check again if all jobs are finished. This
-        # is necessary because each job can can enable or disable other jobs and
-        # self.jobs_before_upload can return a different set of jobs every time.
-        # We assume that nothing changes once all jobs are finished.
-        jobs_before_upload = self.jobs_before_upload
-        while not all(job.is_finished for job in jobs_before_upload):
-            _log.debug('Waiting for jobs before upload: %r', [j.name for j in jobs_before_upload])
-            await asyncio.gather(*(job.wait() for job in jobs_before_upload))
-            jobs_before_upload = self.jobs_before_upload
-        _log.debug('Done waiting for jobs before upload: %r', [j.name for j in jobs_before_upload])
+        while jobs := get_unfinished_jobs():
+            _log.debug('Waiting for jobs before upload: %r', [j.name for j in jobs])
+            await asyncio.gather(*(job.wait() for job in jobs))
+            jobs = get_unfinished_jobs()
+        _log.debug('Done waiting for jobs before upload: %r', [j.name for j in self.enabled_jobs_before_upload])
 
-        # Wait for any dangling background tasks
+        # Wait for any dangling background tasks attached to Tracker
         await self._tracker.await_tasks()
 
         # Don't submit if self._tracker_jobs thinks that's a bad idea.
         if self._tracker_jobs.submission_ok:
             await self._submit()
-        self.finish()
 
     async def _submit(self):
         _log.debug('%s: Submitting', self._tracker.name)
@@ -126,40 +125,49 @@ class SubmitJob(JobBase):
             self.signal.emit('logging_in')
             await self._tracker.login()
             self.signal.emit('logged_in')
+
             try:
                 self.signal.emit('uploading')
                 torrent_page_url = await self._tracker.upload(self._tracker_jobs)
                 if torrent_page_url:
                     self.send(torrent_page_url)
                     self.signal.emit('uploaded')
+
             finally:
                 self.signal.emit('logging_out')
                 await self._tracker.logout()
                 self.signal.emit('logged_out')
+
         except errors.RequestError as e:
             self.error(e)
 
     @property
-    def jobs_before_upload(self):
+    def enabled_jobs_before_upload(self):
         """
         Sequence of jobs to do before submission
 
         This is the same as :attr:`.TrackerJobsBase.jobs_before_upload` but
         with all `None` values and disabled jobs filtered out.
         """
-        return tuple(job for job in self._tracker_jobs.jobs_before_upload
-                     if job and job.is_enabled)
+        return tuple(
+            job
+            for job in self._tracker_jobs.jobs_before_upload
+            if job and job.is_enabled
+        )
 
     @property
-    def jobs_after_upload(self):
+    def enabled_jobs_after_upload(self):
         """
         Sequence of jobs to do after successful submission
 
         This is the same as :attr:`.TrackerJobsBase.jobs_after_upload` but with
         all `None` values and disabled jobs filtered out.
         """
-        return tuple(job for job in self._tracker_jobs.jobs_after_upload
-                     if job and job.is_enabled)
+        return tuple(
+            job
+            for job in self._tracker_jobs.jobs_after_upload
+            if job and job.is_enabled
+        )
 
     @property
     def hidden(self):
@@ -187,7 +195,7 @@ class SubmitJob(JobBase):
         or the final job cannot be determined at the moment.
         """
         if not self._tracker_jobs.submission_ok:
-            jobs = self.jobs_before_upload
+            jobs = self.enabled_jobs_before_upload
             # Because jobs can enable/disable each other, we can't know the
             # final job until all jobs are finished.
             if jobs and all(job.is_finished for job in jobs):
