@@ -78,18 +78,22 @@ class CreateTorrentJob(base.JobBase):
         self._exclude_files = list(self._tracker.options['exclude'])
         self._exclude_files.extend(exclude_files)
 
+        self._torrent_process = None
+        self._activity = ''
+
         self.signal.add('announce_url')
         self.signal.add('file_tree')
         self.signal.add('file_list')
         self.signal.add('progress_update')
-        self._torrent_process = None
-        self._activity = ''
 
     @property
     def activity(self):
         """
         What is currently being done
 
+        ``"announce_url"``
+            If we are getting the announce URL.
+
         ``"hashing"``
             If we are hashing files.
 
@@ -104,12 +108,12 @@ class CreateTorrentJob(base.JobBase):
         """
         return self._activity
 
-    def execute(self):
+    async def run(self):
         """Get announce URL from `tracker`, then execute torrent creation subprocess"""
-        self._get_announce_url_task = self.attach_task(self._get_announce_url())
-        self._get_announce_url_task.add_done_callback(
-            lambda task: self._start_torrent_creation_process(task.result())
-        )
+        announce_url = await self._get_announce_url()
+        if announce_url:
+            self._start_torrent_process(announce_url)
+            await self._torrent_process.join()
 
     async def _get_announce_url(self):
         self._activity = 'announce_url'
@@ -124,7 +128,7 @@ class CreateTorrentJob(base.JobBase):
         finally:
             self._activity = ''
 
-    def _start_torrent_creation_process(self, announce_url):
+    def _start_torrent_process(self, announce_url):
         self._torrent_process = daemon.DaemonProcess(
             name=self.name,
             target=_torrent_process,
@@ -144,15 +148,14 @@ class CreateTorrentJob(base.JobBase):
             info_callback=self._handle_info_update,
             error_callback=self._handle_error,
             result_callback=self._handle_torrent_created,
-            finished_callback=self.finish,
         )
         self._torrent_process.start()
 
-    def finish(self):
-        """Terminate torrent creation subprocess and finish"""
+    def terminate(self):
+        """Terminate torrent creation subprocess and all tasks"""
         if self._torrent_process:
             self._torrent_process.stop()
-        super().finish()
+        super().terminate()
 
     def cancel_search(self):
         """
@@ -193,10 +196,8 @@ class CreateTorrentJob(base.JobBase):
 
     def _handle_error(self, error):
         if isinstance(error, BaseException):
-            _log.debug('CreateTorrentJob: Raising %r', error)
-            self.exception(error)
+            raise error
         else:
-            _log.debug('CreateTorrentJob: Reporting %r', error)
             self.error(error)
 
 
@@ -238,7 +239,7 @@ def _torrent_process(output_queue, input_queue, *args, **kwargs):
     kwargs['progress_callback'] = progress_callback
     try:
         torrent_path = torrent.create(*args, **kwargs)
-    except errors.TorrentError as e:
+    except errors.TorrentCreateError as e:
         output_queue.put((daemon.MsgType.error, str(e)))
     else:
         output_queue.put((daemon.MsgType.result, torrent_path))
@@ -284,19 +285,12 @@ class AddTorrentJob(base.QueueJobBase):
         self.signal.register('error', lambda _: setattr(self, 'info', ''))
 
     async def handle_input(self, torrent_path):
-        _log.debug('Adding %s to %s', torrent_path, self._btclient.name)
         self.signal.emit('adding', torrent_path)
-
-        response = await self._btclient.add_torrent(torrent_path)
-
-        for warning in response.warnings:
-            self.warn(warning)
-
-        for error in response.errors:
-            self.error(error)
-
-        if response.success:
-            infohash = (response.added + response.already_added)[0]
+        try:
+            infohash = await self._btclient.add_torrent(torrent_path)
+        except errors.TorrentAddError as e:
+            self.error(e)
+        else:
             self.send(infohash)
             self.signal.emit('added', infohash)
 
@@ -361,8 +355,6 @@ class CopyTorrentJob(base.QueueJobBase):
             except OSError as e:
                 msg = e.strerror if e.strerror else str(e)
                 self.error(f'Failed to copy {filepath} to {self._destination}: {msg}')
-                # Default to original torrent path
-                self.send(filepath)
             else:
                 self.send(new_path)
                 self.signal.emit('copied', new_path)
