@@ -34,7 +34,7 @@ class TrackerConfigBase(dict):
         'exclude': utils.configfiles.config_value(
             value=utils.types.ListOf(
                 item_type=utils.types.RegEx,
-                iterable=(),
+                default=(),
             ),
             description='List of regular expressions. Matching files are excluded from generated torrents.',
         ),
