@@ -7,6 +7,7 @@ import builtins
 import collections
 import functools
 import pathlib
+import re
 
 from ... import errors, jobs, utils
 
@@ -27,7 +28,9 @@ class TrackerJobsBase(abc.ABC):
     are created only once per session.
 
     Subclasses that need to run background tasks should pass them to
-    :meth:`.JobBase.attach_task` or to :meth:`.TrackerBase.attach_task`.
+    :meth:`.JobBase.add_task` or to :meth:`.TrackerBase.attach_task`.
+    :meth:`.TrackerBase.attach_task` should only be used if there is no
+    appropriate job for the task.
 
     For a description of the arguments see the corresponding properties.
     """
@@ -35,13 +38,13 @@ class TrackerJobsBase(abc.ABC):
     def __init__(self, *, content_path, tracker,
                  reuse_torrent_path=None, exclude_files=(),
                  btclient=None, torrent_destination=None,
-                 screenshots_optimization=None, image_host=None,
+                 screenshots_optimization=None, image_hosts=None,
                  options=None, common_job_args=None):
         self._content_path = content_path
         self._reuse_torrent_path = reuse_torrent_path
         self._tracker = tracker
         self._screenshots_optimization = screenshots_optimization
-        self._image_host = image_host
+        self._image_hosts = image_hosts
         self._btclient = btclient
         self._torrent_destination = torrent_destination
         self._exclude_files = exclude_files
@@ -54,7 +57,7 @@ class TrackerJobsBase(abc.ABC):
         """
         Content path to generate metadata for
 
-        This is the same object that was passed as a initialization argument.
+        This is the same object that was passed as initialization argument.
         """
         return self._content_path
 
@@ -63,7 +66,7 @@ class TrackerJobsBase(abc.ABC):
         """
         :class:`~.trackers.base.TrackerBase` subclass
 
-        This is the same object that was passed as a initialization argument.
+        This is the same object that was passed as initialization argument.
         """
         return self._tracker
 
@@ -81,7 +84,7 @@ class TrackerJobsBase(abc.ABC):
         """
         Where to copy the generated torrent file to or `None`
 
-        This is the same object that was passed as a initialization argument.
+        This is the same object that was passed as initialization argument.
         """
         return self._torrent_destination
 
@@ -104,25 +107,25 @@ class TrackerJobsBase(abc.ABC):
         """
         Configuration options provided by the user
 
-        This is the same object that was passed as a initialization argument.
+        This is the same object that was passed as initialization argument.
         """
         return self._options
 
     @property
-    def image_host(self):
+    def image_hosts(self):
         """
-        :class:`~.base.ImageHostBase` instance or `None`
+        Sequence of :class:`~.base.ImageHostBase` instances or `None`
 
-        This is the same object that was passed as a initialization argument.
+        This is the same object that was passed as initialization argument.
         """
-        return self._image_host
+        return self._image_hosts
 
     @property
     def btclient(self):
         """
         :class:`~.btclient.BtClient` instance or `None`
 
-        This is the same object that was passed as a initialization argument.
+        This is the same object that was passed as initialization argument.
         """
         return self._btclient
 
@@ -187,9 +190,9 @@ class TrackerJobsBase(abc.ABC):
 
         .. warning:: This is not a foolproof way to find all dependencies.
 
-                     :class:`~.QueueJobBase` instances, for example, don't where
-                     their input comes from, so they don't know anything about
-                     which jobs they depend on. There are also
+                     :class:`~.QueueJobBase` instances, for example, don't
+                     know where their input comes from, so they don't know
+                     anything about which jobs they depend on. There are also
                      :attr:`~.JobBase.precondition`\\ s, which can define
                      arbitrary requirements.
 
@@ -351,7 +354,7 @@ class TrackerJobsBase(abc.ABC):
             return add_torrent_job
 
     def finalize_add_torrent_job(self, _):
-        self.add_torrent_job.finalize()
+        self.add_torrent_job.close()
 
     @functools.cached_property
     def copy_torrent_job(self):
@@ -370,7 +373,17 @@ class TrackerJobsBase(abc.ABC):
             return copy_torrent_job
 
     def finalize_copy_torrent_job(self, _):
-        self.copy_torrent_job.finalize()
+        self.copy_torrent_job.close()
+
+    @property
+    def torrent_filepath(self):
+        """Local path to the torrent file created by :attr:`create_torrent_job`"""
+        return self.get_job_output(self.create_torrent_job, slice=0)
+
+    @functools.cached_property
+    def subtitles(self):
+        """Sequence of :class:`~.subtitle.Subtitle` objects for :attr:`content_path`"""
+        return utils.subtitle.get_subtitles(self.content_path)
 
     @functools.cached_property
     def release_name(self):
@@ -398,17 +411,30 @@ class TrackerJobsBase(abc.ABC):
 
         The text is automatically updated when :attr:`imdb_job` sends an ID.
         """
+        # NOTE: This job should not use the same cache as the `release-name`
+        #       subcommand because a tracker's release_name_job can make
+        #       arbitrary customizations.
         return jobs.dialog.TextFieldJob(
             name=self.get_job_name('release-name'),
             label='Release Name',
-            text=str(self.release_name),
             callbacks={
                 'output': self.release_name.set_release_info,
             },
             precondition=self.make_precondition('release_name_job'),
+            validator=self.validate_release_name,
             **self.common_job_args(),
         )
 
+    def validate_release_name(self, text):
+        if not text.strip():
+            raise ValueError('Release name must not be empty.')
+
+        match = re.search(rf'(?:{utils.release.DELIM}|$)(UNKNOWN_([A-Z_]+))(?:{utils.release.DELIM}|$)', text)
+        if match:
+            placeholder = match.group(1)
+            attribute = match.group(2).replace('_', ' ')
+            raise ValueError(f'Replace "{placeholder}" with the proper {attribute.lower()}.')
+
     async def update_release_name_from(self, webdb, webdb_id):
         """
         Update :attr:`release_name_job` with web DB information
@@ -443,10 +469,6 @@ class TrackerJobsBase(abc.ABC):
         # Update other webdb queries with IMDb info
         self.tracker.attach_task(self._propagate_webdb_info(self.imdb, imdb_id))
 
-        # Upload poster image from appropriate webdb
-        if self._poster_webdb is self.imdb:
-            self._start_poster_job(self.imdb, imdb_id)
-
     @property
     def imdb_id(self):
         """IMDb ID if :attr:`imdb_job` is finished or `None`"""
@@ -466,11 +488,10 @@ class TrackerJobsBase(abc.ABC):
         )
 
     def _handle_tmdb_id(self, tmdb_id):
-        # Update other webdb queries with TMDb info
-        self.tracker.attach_task(self._propagate_webdb_info(self.tmdb, tmdb_id))
-
-        if self._poster_webdb is self.tmdb:
-            self._start_poster_job(self.tmdb, tmdb_id)
+        # Do NOT update other webdb queries with TMDb info. TMDb year seems to
+        # change based on geolocation and is therefore garbage.
+        # self.tracker.attach_task(self._propagate_webdb_info(self.tmdb, tmdb_id))
+        pass
 
     @property
     def tmdb_id(self):
@@ -494,9 +515,6 @@ class TrackerJobsBase(abc.ABC):
         # Update other webdb queries with TVmaze info
         self.tracker.attach_task(self._propagate_webdb_info(self.tvmaze, tvmaze_id))
 
-        if self._poster_webdb is self.tvmaze:
-            self._start_poster_job(self.tvmaze, tvmaze_id)
-
     @property
     def tvmaze_id(self):
         """TVmaze ID if :attr:`tvmaze_job` is finished or `None`"""
@@ -593,9 +611,9 @@ class TrackerJobsBase(abc.ABC):
     @functools.cached_property
     def upload_screenshots_job(self):
         """:class:`~.jobs.imghost.ImageHostJob` instance"""
-        if self.image_host and self.screenshots_job:
+        if self.image_hosts and self.screenshots_job:
             imghost_job = jobs.imghost.ImageHostJob(
-                imghost=self.image_host,
+                imghosts=self.image_hosts,
                 precondition=self.make_precondition('upload_screenshots_job'),
                 **self.common_job_args(),
             )
@@ -610,45 +628,130 @@ class TrackerJobsBase(abc.ABC):
             return imghost_job
 
     def finalize_upload_screenshots_job(self, _):
-        self.upload_screenshots_job.finalize()
+        self.upload_screenshots_job.close()
+
+    poster_max_width = 300
+    """Maximum poster image width"""
+
+    poster_max_height = 600
+    """Maximum poster image height"""
 
     @functools.cached_property
     def poster_job(self):
         """
         :class:`~.jobs.poster.PosterJob` instance
 
-        This is started automatically when an appropriate webdb job finishes.
+        See also :meth:`get_poster`, :meth:`get_poster_from_user`,
+        and :meth:`get_poster_from_webdb`.
         """
         return jobs.poster.PosterJob(
-            autostart=False,
-            webdb_id=None,
-            webdb=None,
-            imghost=self.image_host,
+            precondition=self.make_precondition('poster_job'),
+            getter=self.get_poster,
             width=self.poster_max_width,
             height=self.poster_max_height,
-            precondition=self.make_precondition('poster_job'),
+            write_to=None,
+            imghosts=self.image_hosts,
             **self.common_job_args(),
         )
 
-    def _start_poster_job(self, webdb, webdb_id):
-        self.poster_job.start(
-            webdb=webdb,
-            webdb_id=webdb_id,
-            season=self.release_name.only_season,
-        )
+    async def get_poster(self):
+        """
+        Return poster file or URL or `None`
 
-    poster_max_width = 300
-    """Maximum poster image width"""
+        The default implementation tries to get the poster from the following
+        methods and returns the first truthy return value:
 
-    poster_max_height = 500
-    """Maximum poster image height"""
+            - :meth:`get_poster_from_user`
+            - :meth:`get_poster_from_tracker`
+            - :meth:`get_poster_from_webdb`
+
+        Besides a file or URL, the return value may also be a dictionary with
+        the key ``poster`` and the following optional keys:
+
+            - ``width`` - Resize width in pixels (keep aspeect ratio)
+            - ``height`` - Resize height in pixels (keep aspeect ratio)
+            - ``write_to`` - Write resized poster to this file
+            - ``imghosts`` - Sequence of :class:`~.ImageHostBase` instances to
+              try to upload the poster to
+
+        See :class:`~.PosterJob` for more information.
+        """
+        poster = await self.get_poster_from_user()
+        if poster:
+            return poster
+
+        poster = await self.get_poster_from_tracker()
+        if poster:
+            return poster
+
+        poster = await self.get_poster_from_webdb()
+        if poster:
+            return poster
+
+    async def get_poster_from_user(self):
+        """
+        Get poster from user (e.g. CLI argument)
+
+        The default implementation uses :attr:`options`\\ ``["poster"]``.
+        """
+        return self.options.get('poster', None)
+
+    async def get_poster_from_tracker(self):
+        """
+        Get poster from tracker or any other custom source
+
+        The default implementation always returns `None`.
+        """
+        return None
+
+    async def get_poster_from_webdb(self):
+        """Return poster URL from :attr:`poster_webdb` or `None`"""
+        if self.poster_webdb_job:
+            # We can't pass self.poster_webdb_job via `prejobs` to PosterJob
+            # because self.poster_webdb_job is determined by checking which
+            # webdb job (self.imdb_job, self.tvmaze_job, etc) is contained in
+            # self.jobs_before_upload. But self.jobs_before_upload contains
+            # self.poster_job, resulting in infinite recursion.
+            await self.poster_webdb_job.wait()
+            # Because imdb_job.no_id_ok may be True, we have to handle
+            # poster_webdb_job.output being empty.
+            webdb_id = self.get_job_output(self.poster_webdb_job, slice=0, default=None)
+            if webdb_id:
+                try:
+                    poster = await self.poster_webdb.poster_url(
+                        webdb_id,
+                        season=self.release_name.only_season,
+                    )
+                except errors.RequestError as e:
+                    _log.debug('Failed to get poster from %s: %r', self.poster_webdb, e)
+                else:
+                    if poster:
+                        return poster
 
     @functools.cached_property
-    def _poster_webdb(self):
-        """:class:`~.webdbs.base.WebDbApiBase` instance for finding poster image"""
+    def poster_webdb_job(self):
+        """
+        :class:`~.jobs.base.WebDbSearchJob` instance that is used by
+        :meth:`get_poster_from_webdb` to get a poster image or `None` if no
+        such instance is enabled and contained in :attr:`jobs_before_upload`
+        """
+        webdb, job = self._poster_webdb_and_job
+        return job
+
+    @functools.cached_property
+    def poster_webdb(self):
+        """
+        :class:`~.webdbs.base.WebDbApiBase` instance that is used by
+        :meth:`get_poster_from_webdb` to get a poster image or `None` if no
+        such instance is enabled and contained in :attr:`jobs_before_upload`
+        """
+        webdb, job = self._poster_webdb_and_job
+        return webdb
+
+    @property
+    def _poster_webdb_and_job(self):
         if (
                 self.poster_job.is_enabled
-                and not self.poster_job.is_started
                 and self.poster_job in self.jobs_before_upload
         ):
             if (
@@ -657,22 +760,21 @@ class TrackerJobsBase(abc.ABC):
                     and self.release_name.type in (utils.release.ReleaseType.season,
                                                    utils.release.ReleaseType.episode)
             ):
-                return self.tvmaze
+                return self.tvmaze, self.tvmaze_job
 
             elif (
                     self.imdb_job.is_enabled
                     and self.imdb_job in self.jobs_before_upload
             ):
-                return self.imdb
+                return self.imdb, self.imdb_job
 
             elif (
                     self.tmdb_job.is_enabled
                     and self.tmdb_job in self.jobs_before_upload
             ):
-                return self.tmdb
+                return self.tmdb, self.tmdb_job
 
-            else:
-                raise RuntimeError('You must add a WebDbSearchJob instance to jobs_before_upload')
+        return None, None
 
     @functools.cached_property
     def mediainfo_job(self):
@@ -718,7 +820,7 @@ class TrackerJobsBase(abc.ABC):
         ``screenshot_urls`` is a :class:`list` of screenshot URLs for the
         corresponding video file.
 
-        .. note:: For DVDs (directories with a VIDEO_TS subdirectory), an
+        .. note:: For DVDs (directories with a VIDEO_TS subdirectory),
                   ``mediainfo`` includes an .IFO and a .VOB key while
                   ``screenshot_urls`` is empty for the .IFO key.
 
@@ -858,17 +960,17 @@ class TrackerJobsBase(abc.ABC):
         :return: :class:`list` or :class:`str`
         """
         if not job.is_finished:
-            if default is not self._NO_DEFAULT:
-                return default
-            else:
-                raise RuntimeError(f'Unfinished job: {job.name}')
+            raise RuntimeError(f'Cannot get output from unfinished job: {job.name}')
         else:
             if slice is None:
                 slice = builtins.slice(None, None)
             try:
                 return job.output[slice]
             except IndexError:
-                raise RuntimeError(f'Job finished with insufficient output: {job.name}: {job.output}')
+                if default is not self._NO_DEFAULT:
+                    return default
+                else:
+                    raise RuntimeError(f'Job finished with insufficient output: {job.name}: {job.output}')
 
     def get_job_attribute(self, job, attribute, default=_NO_DEFAULT):
         """
@@ -885,7 +987,7 @@ class TrackerJobsBase(abc.ABC):
             if default is not self._NO_DEFAULT:
                 return default
             else:
-                raise RuntimeError(f'Unfinished job: {job.name}')
+                raise RuntimeError(f'Cannot get attribute from unfinished job: {job.name}')
         else:
             return getattr(job, attribute)
 
@@ -942,3 +1044,19 @@ class TrackerJobsBase(abc.ABC):
 
         # Space between rows
         return vertical_spacer.join(rows)
+
+    def read_nfo(self, strip=False):
+        """
+        Find ``*.nfo`` file beneath :attr:`content_path` and return its contents
+
+        If no ``*.nfo`` file is found, return `None`.
+
+        If an ``*.nfo`` file is found but cannot be read, call
+        :meth:`~.TrackerJobsBase.error`.
+
+        See :func:`.string.read_nfo` for more information.
+        """
+        try:
+            return utils.string.read_nfo(self.content_path, strip=strip)
+        except errors.ContentError as e:
+            self.error(e)
