@@ -4,8 +4,16 @@ Concrete :class:`~.base.TrackerConfigBase` subclass for BHD
 
 import base64
 
-from ...utils import argtypes, configfiles, imghosts, types
-from ..base import TrackerConfigBase
+from ...utils import argtypes, configfiles, is_running_in_development_environment, types
+from ..base import TrackerConfigBase, exclude
+
+BhdImageHost = types.ImageHost(
+    allowed=(
+        ('imgbox', 'ptpimg', 'imgbb', 'dummy')
+        if is_running_in_development_environment() else
+        ('imgbox', 'ptpimg', 'imgbb')
+    ),
+)
 
 
 class BhdTrackerConfig(TrackerConfigBase):
@@ -41,14 +49,29 @@ class BhdTrackerConfig(TrackerConfigBase):
                 'after the upload instead of going live.'
             ),
         ),
-        'image_host': types.Choice('imgbox', options=imghosts.imghost_names()),
+        'image_host': configfiles.config_value(
+            value=types.ListOf(
+                item_type=BhdImageHost,
+                default=('imgbox',),
+                separator=',',
+            ),
+            description=(
+                'List of image hosting service names. The first service is normally used '
+                'with the others as backup if uploading to the first fails.\n'
+                + 'Supported services: ' + ', '.join(BhdImageHost.options)
+            ),
+        ),
         'screenshots': configfiles.config_value(
             value=types.Integer(4, min=3, max=10),
             description='How many screenshots to make.',
         ),
         'exclude': (
-            r'\.(?i:txt|jpg|jpeg|png|sfv|md5)$',
-            r'/(?i:sample)',
+            exclude.checksums,
+            exclude.extras,
+            exclude.images,
+            exclude.nfo,
+            exclude.samples,
+            exclude.subtitles,
         ),
     }
 
