@@ -5,7 +5,7 @@ Concrete :class:`~.base.TrackerJobsBase` subclass for BHD
 import functools
 import io
 
-from ... import __homepage__, __project_name__, errors, jobs, utils
+from ... import __homepage__, __project_name__, jobs, utils
 from ..base import TrackerJobsBase
 
 import logging  # isort:skip
@@ -16,13 +16,12 @@ class BhdTrackerJobs(TrackerJobsBase):
 
     @functools.cached_property
     def jobs_before_upload(self):
+        # NOTE: Keep in mind that the order of jobs is important for
+        #       isolated_jobs: The final job is the overall result, so if
+        #       upload_screenshots_job is listed after description_job,
+        #       --only-description is going to print the list of uploaded
+        #       screenshot URLs.
         return (
-            # Background jobs
-            self.create_torrent_job,
-            self.mediainfo_job,
-            self.screenshots_job,
-            self.upload_screenshots_job,
-
             # Interactive jobs
             self.tmdb_job,
             self.imdb_job,
@@ -30,9 +29,15 @@ class BhdTrackerJobs(TrackerJobsBase):
             self.category_job,
             self.type_job,
             self.source_job,
-            self.description_job,
             self.scene_check_job,
             self.tags_job,
+
+            # Background jobs
+            self.create_torrent_job,
+            self.mediainfo_job,
+            self.screenshots_job,
+            self.upload_screenshots_job,
+            self.description_job,
         )
 
     @property
@@ -62,6 +67,12 @@ class BhdTrackerJobs(TrackerJobsBase):
             # Activate all jobs in jobs_before/after_upload
             return ()
 
+    @functools.cached_property
+    def tmdb_job(self):
+        tmdb_job = super().tmdb_job
+        tmdb_job.no_id_ok = True
+        return tmdb_job
+
     @functools.cached_property
     def category_job(self):
         return jobs.dialog.ChoiceJob(
@@ -72,7 +83,8 @@ class BhdTrackerJobs(TrackerJobsBase):
                 self.release_name_job,
             ),
             autodetect=self.autodetect_category,
-            choices=(
+            autofinish=True,
+            options=(
                 ('Movie', '1'),
                 ('TV', '2'),
             ),
@@ -101,8 +113,9 @@ class BhdTrackerJobs(TrackerJobsBase):
             prejobs=(
                 self.release_name_job,
             ),
-            autodetect_and_finish=self.autodetect_type,
-            choices=(
+            autodetect=self.autodetect_type,
+            autofinish=True,
+            options=(
                 ('UHD 100', 'UHD 100'),
                 ('UHD 66', 'UHD 66'),
                 ('UHD 50', 'UHD 50'),
@@ -127,9 +140,22 @@ class BhdTrackerJobs(TrackerJobsBase):
         )
 
     _autodetect_type_map = {
+        # Directory trees / Images
         'DVD 9': lambda release_name: release_name.source == 'DVD9',
         'DVD 5': lambda release_name: release_name.source == 'DVD5',
+
+        # Remuxes
         'DVD Remux': lambda release_name: release_name.source == 'DVD Remux',
+        'UHD Remux': lambda release_name: (
+            'Remux' in release_name.source
+            and release_name.resolution == '2160p'
+        ),
+        'BD Remux': lambda release_name: (
+            'Remux' in release_name.source
+            and release_name.resolution == '1080p'
+        ),
+
+        # Encodes
         '2160p': lambda release_name: release_name.resolution == '2160p',
         '1080p': lambda release_name: release_name.resolution == '1080p',
         '1080i': lambda release_name: release_name.resolution == '1080i',
@@ -139,7 +165,7 @@ class BhdTrackerJobs(TrackerJobsBase):
         '480p': lambda release_name: release_name.resolution == '480p',
     }
 
-    def autodetect_type(self, _):
+    async def autodetect_type(self, _):
         approved_release_name = self.release_name
         _log.debug('Autodetecting type: Approved resolution: %r', approved_release_name.resolution)
         _log.debug('Autodetecting type: Approved source: %r', approved_release_name.source)
@@ -149,10 +175,6 @@ class BhdTrackerJobs(TrackerJobsBase):
 
     @functools.cached_property
     def source_job(self):
-        # 'output' signal is only emitted when job succeeds while 'finished'
-        # signal is also emitted when job fails (e.g. when Ctrl-c is pressed)
-        self.release_name_job.signal.register('output', self.autodetect_source)
-
         return jobs.dialog.ChoiceJob(
             name=self.get_job_name('source'),
             label='Source',
@@ -160,8 +182,9 @@ class BhdTrackerJobs(TrackerJobsBase):
             prejobs=(
                 self.release_name_job,
             ),
-            autodetect_and_finish=self.autodetect_source,
-            choices=(
+            autodetect=self.autodetect_source,
+            autofinish=True,
+            options=(
                 ('Blu-ray', 'Blu-ray'),
                 ('HD-DVD', 'HD-DVD'),
                 ('WEB', 'WEB'),
@@ -180,7 +203,7 @@ class BhdTrackerJobs(TrackerJobsBase):
         'DVD': lambda release_name: 'DVD' in release_name.source,
     }
 
-    def autodetect_source(self, _):
+    async def autodetect_source(self, _):
         approved_release_name = self.release_name
         _log.debug('Autodetecting source: Approved source: %r', approved_release_name.source)
         for label, is_match in self._autodetect_source_map.items():
@@ -332,12 +355,12 @@ class BhdTrackerJobs(TrackerJobsBase):
             'type': self.get_job_attribute(self.type_job, 'choice'),
             'source': self.get_job_attribute(self.source_job, 'choice'),
             'imdb_id': self.get_job_output(self.imdb_job, slice=0),
-            'tmdb_id': self.get_job_output(self.tmdb_job, slice=0).split('/')[1],
+            'tmdb_id': self.post_data_tmdb_id,
             'description': self.get_job_output(self.description_job, slice=0),
             'edition': self.post_data_edition,
             'custom_edition': self.options['custom_edition'],
             'tags': ','.join(self.get_job_output(self.tags_job, slice=0).split('\n')),
-            'nfo': self.post_data_nfo,
+            'nfo': self.read_nfo(),
             'pack': self.post_data_pack,
             'sd': self.post_data_sd,
             'special': self.post_data_special,
@@ -345,6 +368,14 @@ class BhdTrackerJobs(TrackerJobsBase):
             'live': '0' if self.options['draft'] else '1',
         }
 
+    @functools.cached_property
+    def post_data_tmdb_id(self):
+        # TMDb ID may be 0 for non-existing shows
+        if self.tmdb_job.output:
+            return self.get_job_output(self.tmdb_job, slice=0).split('/')[1]
+        else:
+            return 0
+
     @functools.cached_property
     def post_data_edition(self):
         # The edition of the uploaded release. (Collector, Director, Extended,
@@ -388,15 +419,6 @@ class BhdTrackerJobs(TrackerJobsBase):
         else:
             return '1' if height < 720 else '0'
 
-    max_nfo_size = 500_000
-
-    @property
-    def post_data_nfo(self):
-        try:
-            return utils.string.read_nfo(self.content_path)
-        except errors.ContentError as e:
-            self.error(e)
-
     @property
     def post_data_special(self):
         # The TV special flag for when the torrent contains a TV special. (0 =
@@ -413,10 +435,6 @@ class BhdTrackerJobs(TrackerJobsBase):
     #     # CAN, CEE, CHN, ESP, EUR, FRA, GBR, GER, HKG, ITA, JPN, KOR, NOR,
     #     # NLD, RUS, TWN or USA)
 
-    @property
-    def torrent_filepath(self):
-        return self.get_job_output(self.create_torrent_job, slice=0)
-
     @property
     def mediainfo_filehandle(self):
         mediainfo = self.get_job_output(self.mediainfo_job, slice=0)
