@@ -4,7 +4,7 @@ Concrete :class:`~.base.TrackerBase` subclass for BHD
 
 import re
 
-from ... import __project_name__, errors, utils
+from ... import errors, utils
 from ..base import TrackerBase
 from .config import BhdTrackerConfig
 from .jobs import BhdTrackerJobs
@@ -20,26 +20,26 @@ class BhdTracker(TrackerBase):
     setup_howto_template = (
         '{howto.introduction}\n'
         '\n'
-        '{howto.current_section}. Announce Passkey\n'
+        '{howto.next_section}. Announce Passkey\n'
         '\n'
         '   {howto.current_section}.1 On the website, go to My Settings -> Security -> Passkey\n'
         '       and copy your personal PASSKEY.\n'
-        '   {howto.current_section}.2 $ upsies set trackers.bhd.announce_passkey PASSKEY\n'
-        '{howto.bump_section}'
+        '   {howto.current_section}.2 $ upsies set trackers.{tracker.name}.announce_passkey PASSKEY\n'
         '\n'
-        '{howto.current_section}. API key\n'
+        '{howto.next_section}. API Key\n'
         '\n'
         '   {howto.current_section}.1 On the website, go to My Settings -> Security -> API Key\n'
         '       and copy your personal API_KEY.\n'
-        '   {howto.current_section}.2 $ upsies set trackers.bhd.apikey API_KEY\n'
-        '{howto.bump_section}'
+        '   {howto.current_section}.2 $ upsies set trackers.{tracker.name}.apikey API_KEY\n'
         '\n'
         '{howto.screenshots}\n'
         '\n'
         '{howto.autoseed}\n'
         '\n'
+        '{howto.reuse_torrents}\n'
+        '\n'
         '{howto.upload}\n'
-        f'   $ {__project_name__} submit bhd /path/to/content --draft\n'
+        '   $ {executable} submit {tracker.name} /path/to/content --draft\n'
     )
 
     TrackerConfig = BhdTrackerConfig
@@ -160,10 +160,10 @@ class BhdTracker(TrackerBase):
         # 72 - 190 GB = 8 MB piece size
         #
         # We assume "[MG]B" means "[MG]iB" because piece sizes in MB would be
-        # considered invalid by most BitTorrent clients.
+        # weird.
 
         piece_size_map = {
-            (0, 8 * 1024 * 1024 * 1024 - 1): 1 * 1024 * 1024,                         # 1 MiB
+            (1, 8 * 1024 * 1024 * 1024 - 1): 1 * 1024 * 1024,                         # 1 MiB
             (8 * 1024 * 1024 * 1024, 16 * 1024 * 1024 * 1024 - 1): 2 * 1024 * 1024,   # 2 MiB
             (16 * 1024 * 1024 * 1024, 72 * 1024 * 1024 * 1024 - 1): 4 * 1024 * 1024,  # 4 MiB
             (72 * 1024 * 1024 * 1024, float('inf')): 8 * 1024 * 1024,                 # 8 MiB
@@ -172,3 +172,5 @@ class BhdTracker(TrackerBase):
         for (min_size, max_size), piece_size in piece_size_map.items():
             if min_size <= bytes <= max_size:
                 return piece_size
+
+        raise RuntimeError(f'Cannot calculate piece size for {bytes} bytes')
