@@ -19,24 +19,24 @@ class NblTracker(base.TrackerBase):
     setup_howto_template = (
         '{howto.introduction}\n'
         '\n'
-        '{howto.current_section}. Announce URL\n'
+        '{howto.next_section}. Announce URL\n'
         '\n'
         '   {howto.current_section}.1 On the website, go to Shows -> Upload and copy the ANNOUNCE_URL.\n'
-        '   {howto.current_section}.2 $ upsies set trackers.nbl.announce_url ANNOUNCE_URL\n'
-        '{howto.bump_section}'
+        '   {howto.current_section}.2 $ upsies set trackers.{tracker.name}.announce_url ANNOUNCE_URL\n'
         '\n'
-        '{howto.current_section}. API key\n'
+        '{howto.next_section}. API key\n'
         '\n'
         '   {howto.current_section}.1 On the website, go to USERNAME -> Settings and scroll down\n'
         '       to "API keys".\n'
         '   {howto.current_section}.2 Tick the "New Key" and the "Upload" boxes.\n'
         '   {howto.current_section}.3 Click on "Save Profile".\n'
         '   {howto.current_section}.4 Scroll down to "API keys" again and copy the new API_KEY.\n'
-        '   {howto.current_section}.5 $ upsies set trackers.nbl.apikey API_KEY\n'
-        '{howto.bump_section}'
+        '   {howto.current_section}.5 $ upsies set trackers.{tracker.name}.apikey API_KEY\n'
         '\n'
         '{howto.autoseed}\n'
         '\n'
+        '{howto.reuse_torrents}\n'
+        '\n'
         '{howto.upload}\n'
     )
 
@@ -104,8 +104,6 @@ class NblTracker(base.TrackerBase):
 
         # NOTE: It seems that none of the keys in the returned JSON are
         #       guaranteed to exist.
-        _log.debug('Upload JSON response: %r', json)
-        _log.debug(json.get('status'))
         if json.get('status') == 'success':
             return tracker_jobs.torrent_filepath
         elif json.get('message'):
