@@ -6,7 +6,7 @@ import functools
 import re
 from datetime import datetime
 
-from ... import errors, jobs, utils
+from ... import errors, jobs, uis, utils
 from ...utils.release import ReleaseType
 from ..base import TrackerJobsBase
 from . import metadata
@@ -19,15 +19,12 @@ class PtpTrackerJobs(TrackerJobsBase):
 
     @functools.cached_property
     def jobs_before_upload(self):
+        # NOTE: Keep in mind that the order of jobs is important for
+        #       isolated_jobs: The final job is the overall result, so if
+        #       upload_screenshots_job is listed after description_job,
+        #       --only-description is going to print the list of uploaded
+        #       screenshot URLs.
         return (
-            # Common background jobs
-            self.create_torrent_job,
-            self.mediainfo_job,
-            self.screenshots_job,
-            self.upload_screenshots_job,
-            self.ptp_group_id_job,
-            self.description_job,
-
             # Common interactive jobs
             self.type_job,
             self.imdb_job,
@@ -40,16 +37,28 @@ class PtpTrackerJobs(TrackerJobsBase):
             # Interactive jobs that only run if movie does not exists on PTP yet
             self.title_job,
             self.year_job,
+            self.edition_job,
             self.plot_job,
             self.tags_job,
-            self.cover_art_job,
+            self.poster_job,
+
+            # Background jobs
+            self.create_torrent_job,
+            self.mediainfo_job,
+            self.screenshots_job,
+            self.upload_screenshots_job,
+            self.ptp_group_id_job,
+            self.audio_languages_job,
+            self.subtitle_languages_job,
+            self.trumpable_job,
+            self.description_job,
         )
 
     @property
     def isolated_jobs(self):
         """
-        Sequence of job attribute names (e.g. "imdb_job") that were singled
-        out by the user, e.g. with a CLI argument
+        Sequence of job attribute names (e.g. "title_job") that were singled
+        out by the user, e.g. with --only-title
         """
         if self.options.get('only_description', False):
             return self.get_job_and_dependencies(
@@ -71,7 +80,7 @@ class PtpTrackerJobs(TrackerJobsBase):
             label='Type',
             precondition=self.make_precondition('type_job'),
             autodetect=self.autodetect_type,
-            choices=(
+            options=(
                 ('Feature Film', 'Feature Film'),
                 ('Short Film', 'Short Film'),
                 ('Miniseries', 'Miniseries'),
@@ -94,12 +103,19 @@ class PtpTrackerJobs(TrackerJobsBase):
         #       duration of all .VOB files, but they notoriously lie about their
         #       duration. For now, the user can always correct the autodetected
         #       type.
-        main_video = utils.video.filter_main_videos(
+        main_video = tuple(utils.video.filter_main_videos(
             tuple(utils.video.find_videos(self.content_path))
-        )[0]
+        ))[0]
         if utils.video.duration(main_video) <= 45 * 60:
             return 'Short Film'
 
+    @functools.cached_property
+    def imdb_job(self):
+        imdb_job = super().imdb_job
+        # Disabled until #7 is fixed.
+        imdb_job.no_id_ok = False
+        return imdb_job
+
     def update_imdb_query(self, type_job_):
         """
         Set :attr:`~.webdbs.common.Query.type` on
@@ -116,13 +132,162 @@ class PtpTrackerJobs(TrackerJobsBase):
                 _log.debug('Updating IMDb query type: %r', utils.release.ReleaseType.series)
                 self.imdb_job.query.type = utils.release.ReleaseType.series
 
+    @functools.cached_property
+    def audio_languages_job(self):
+        return jobs.custom.CustomJob(
+            name=self.get_job_name('audio-languages'),
+            label='Audio Languages',
+            precondition=self.make_precondition('audio_languages_job'),
+            worker=self.autodetect_audio_languages,
+            no_output_is_ok=True,
+            catch=(
+                errors.ContentError,
+            ),
+            **self.common_job_args(ignore_cache=True),
+        )
+
+    async def autodetect_audio_languages(self, job):
+        audio_languages = utils.video.languages(
+            self.content_path,
+            default='?',
+            exclude_commentary=True,
+        ).get('Audio', ())
+        _log.debug('Audio languages: %r', audio_languages)
+        for language in audio_languages:
+            self.audio_languages_job.send(language)
+
+    @functools.cached_property
+    def subtitle_languages_job(self):
+        return jobs.custom.CustomJob(
+            name=self.get_job_name('subtitle-languages'),
+            label='Subtitle Languages',
+            precondition=self.make_precondition('subtitle_languages_job'),
+            worker=self.autodetect_subtitle_languages,
+            no_output_is_ok=True,
+            **self.common_job_args(ignore_cache=True),
+        )
+
+    async def autodetect_subtitle_languages(self, job):
+        # Unlike video.languages(), the subtitles property from the parent class
+        # includes external subtitles, e.g. from .srt or .idx/.sub files.
+        subtitle_languages = [
+            subtitle.language or '?'
+            for subtitle in self.subtitles
+        ]
+        _log.debug('Subtitle languages: %r', subtitle_languages)
+        for language in subtitle_languages:
+            self.subtitle_languages_job.send(language)
+
+        # If there are any subtitle tracks without a language tag, tell the user
+        # to manually check the subtitle boxes on the website.
+        if '?' in subtitle_languages:
+            self.subtitle_languages_job.warn(
+                "Some subtitle tracks don't have a language tag.\n"
+                'Please add any missing subtitle languages manually\n'
+                'on the website after uploading.'
+            )
+
+    @functools.cached_property
+    def trumpable_job(self):
+        return jobs.custom.CustomJob(
+            name=self.get_job_name('trumpable'),
+            label='Trumpable',
+            precondition=self.make_precondition('trumpable_job'),
+            prejobs=(
+                self.audio_languages_job,
+                self.subtitle_languages_job,
+            ),
+            worker=self.autodetect_trumpable,
+            no_output_is_ok=True,
+            **self.common_job_args(ignore_cache=True),
+        )
+
+    async def autodetect_trumpable(self, job):
+        reasons = []
+
+        if self.options.get('hardcoded_subtitles', None) in (True, False):
+            # User specified "Hardcoded subtitles"
+            if self.options['hardcoded_subtitles']:
+                reasons.append(metadata.PtpTrumpableReason.HARDCODED_SUBTITLES)
+        # Autodetection of hardcoded subtitles would go here
+
+        if self.options.get('no_english_subtitles', None) in (True, False):
+            # User specified "No English subtitles"
+            if self.options['no_english_subtitles']:
+                reasons.append(metadata.PtpTrumpableReason.NO_ENGLISH_SUBTITLES)
+        else:
+            # Autodetect "No English subtitles"
+            assert self.audio_languages_job.is_finished
+            assert self.subtitle_languages_job.is_finished
+            audio_languages = self.audio_languages_job.output
+            subtitle_languages = self.subtitle_languages_job.output
+            if (
+                    # Not a silent film?
+                    audio_languages
+                    # No properly tagged English audio track?
+                    and 'en' not in audio_languages
+            ):
+                # English subtitles are required.
+                if 'en' not in subtitle_languages:
+                    # No English subtitles found. If there are any audio or
+                    # subtitle tracks that could be English, ask the user.
+                    prompt = self.get_trumpable_no_english_subtitles_prompt(audio_languages, subtitle_languages)
+                    if prompt:
+                        self.trumpable_job.add_prompt(prompt)
+                    else:
+                        # Every audio and subtitle track has a language tag and
+                        # none of them are English.
+                        reasons.append(metadata.PtpTrumpableReason.NO_ENGLISH_SUBTITLES)
+
+        _log.debug('Trumpable reasons from autodetection: %r', reasons)
+        return reasons
+
+    def get_trumpable_no_english_subtitles_prompt(self, audio_languages, subtitle_languages):
+        audio_languages = self.audio_languages_job.output
+        subtitle_languages = self.subtitle_languages_job.output
+        question = ''
+
+        if '?' in audio_languages and '?' in subtitle_languages:
+            question = (
+                'Does this release have an English audio track or\n'
+                'English subtitles for the main audio track?\n'
+                '(Commentary and the like do not count.)'
+            )
+
+        elif '?' in audio_languages:
+            question = (
+                'Does this release have an English audio track?\n'
+                '(Commentary and the like do not count.)'
+            )
+
+        elif '?' in subtitle_languages:
+            question = 'Does this release have English subtitles for the main audio track?'
+
+        if question:
+            return uis.prompts.RadioListPrompt(
+                question=question,
+                options=(
+                    ('Yes', None),
+                    ('No', metadata.PtpTrumpableReason.NO_ENGLISH_SUBTITLES),
+                ),
+                callbacks=(
+                    self.trumpable_no_english_subtitles_prompt_callback,
+                ),
+            )
+
+    def trumpable_no_english_subtitles_prompt_callback(self, answer):
+        _log.debug('Trumpable rasons from user: %r', answer)
+        text, reason = answer
+        if reason:
+            self.trumpable_job.send(reason)
+
     @functools.cached_property
     def description_job(self):
         return jobs.dialog.TextFieldJob(
             name=self.get_job_name('description'),
             label='Description',
             precondition=self.make_precondition('description_job'),
-            prejobs=(
+            prejobs=self.get_job_and_dependencies(
                 self.mediainfo_job,
                 self.upload_screenshots_job,
             ),
@@ -201,29 +366,46 @@ class PtpTrackerJobs(TrackerJobsBase):
         else:
             return self.options['screenshots_from_episode']
 
-    @property
-    def imdb_id(self):
-        if self.imdb_job.is_finished:
-            return self.imdb_job.output[0]
-
     @functools.cached_property
     def ptp_group_id_job(self):
         return jobs.custom.CustomJob(
             name=self.get_job_name('ptp-group-id'),
             label='PTP Group ID',
             precondition=self.make_precondition('ptp_group_id_job'),
-            prejobs=(
+            prejobs=self.get_job_and_dependencies(
                 self.imdb_job,
             ),
-            worker=self.fetch_ptp_group_id,
-            catch=(errors.RequestError,),
+            worker=self.get_ptp_group_id,
+            catch=(
+                errors.RequestError,
+            ),
             **self.common_job_args(),
         )
 
-    async def fetch_ptp_group_id(self, _):
-        assert self.imdb_job.is_finished
+    async def get_ptp_group_id(self, _):
+        # Get group ID from PTP by IMDb ID.
+        assert self.imdb_job.is_finished, self.imdb_job
         group_id = await self.tracker.get_ptp_group_id_by_imdb_id(self.imdb_id)
-        return '' if group_id is None else group_id
+        if group_id:
+            _log.debug('PTP group ID via IMDb ID: %r -> %r', self.imdb_id, group_id)
+            return group_id
+        else:
+            # If there is no existing PTP group, ask the user. Usually this
+            # happens if PTP doesn't know this movie yet. The user can just
+            # press enter to create a new PTP group.
+            #
+            # But it's also possible that PTP has a group for this movie while
+            # it has no IMDb ID.
+            self.ptp_group_id_job.add_prompt(uis.prompts.TextPrompt(
+                callbacks=(
+                    self.ptp_group_id_prompt_callback,
+                ),
+            ))
+            self.ptp_group_id_job.info = 'Leave this empty to create a new PTP group.'
+
+    def ptp_group_id_prompt_callback(self, ptp_group_id):
+        _log.debug('PTP group ID from user: %r', ptp_group_id)
+        self.ptp_group_id_job.send(ptp_group_id.strip())
 
     @property
     def ptp_group_id(self):
@@ -237,7 +419,7 @@ class PtpTrackerJobs(TrackerJobsBase):
 
     def ptp_group_does_not_exist(self):
         """
-        Whether the server already has a group for :attr:`imdb_id`
+        Whether no releases of :attr:`imdb_id` already exist
 
         :attr:`ptp_group_id_job` must be finished when this is called.
 
@@ -245,7 +427,7 @@ class PtpTrackerJobs(TrackerJobsBase):
         needed if the server doesn't have any releases for this :attr:`imdb_id`
         yet.
         """
-        assert self.ptp_group_id_job.is_finished
+        assert self.ptp_group_id_job.is_finished, self.ptp_group_id_job
         if self.ptp_group_id:
             return False
         else:
@@ -258,11 +440,13 @@ class PtpTrackerJobs(TrackerJobsBase):
             label='Title',
             precondition=self.make_precondition(
                 'title_job',
+                # Don't run this job if PTP already knows this movie.
                 precondition=self.ptp_group_does_not_exist,
             ),
-            prejobs=(
+            prejobs=self.get_job_and_dependencies(
+                # We need to wait for the PTP group ID to become available
+                # before we can check if it exists.
                 self.ptp_group_id_job,
-                self.imdb_job,
             ),
             text=self.fetch_title,
             normalizer=self.normalize_title,
@@ -271,7 +455,7 @@ class PtpTrackerJobs(TrackerJobsBase):
         )
 
     async def fetch_title(self):
-        assert self.imdb_job.is_finished
+        assert self.imdb_job.is_finished, self.imdb_job
         return await self.tracker.get_ptp_metadata(self.imdb_id, key='title')
 
     def normalize_title(self, text):
@@ -279,7 +463,7 @@ class PtpTrackerJobs(TrackerJobsBase):
 
     def validate_title(self, text):
         if not text:
-            raise ValueError('Title must not be empty')
+            raise ValueError('Title must not be empty.')
 
     @functools.cached_property
     def year_job(self):
@@ -288,11 +472,13 @@ class PtpTrackerJobs(TrackerJobsBase):
             label='Year',
             precondition=self.make_precondition(
                 'year_job',
+                # Don't run this job if PTP already knows this movie.
                 precondition=self.ptp_group_does_not_exist,
             ),
-            prejobs=(
+            prejobs=self.get_job_and_dependencies(
+                # We need to wait for the PTP group ID to become available
+                # before we can check if it exists.
                 self.ptp_group_id_job,
-                self.imdb_job,
             ),
             text=self.fetch_year,
             normalizer=self.normalize_year,
@@ -301,20 +487,86 @@ class PtpTrackerJobs(TrackerJobsBase):
         )
 
     async def fetch_year(self):
-        assert self.imdb_job.is_finished
+        assert self.imdb_job.is_finished, self.imdb_job
         return await self.tracker.get_ptp_metadata(self.imdb_id, key='year')
 
     def normalize_year(self, text):
         return text.strip()
 
     def validate_year(self, text):
+        if not text:
+            raise ValueError('Year must not be empty.')
         try:
             year = int(text)
         except ValueError:
-            raise ValueError('Year is not a number')
+            raise ValueError('Year is not a number.')
         else:
             if not 1800 < year < datetime.now().year + 10:
-                raise ValueError('Year is not reasonable')
+                raise ValueError('Year is not reasonable.')
+
+    @functools.cached_property
+    def edition_job(self):
+        return jobs.dialog.TextFieldJob(
+            name=self.get_job_name('edition'),
+            label='Edition',
+            precondition=self.make_precondition('edition_job'),
+            text=self.autodetect_edition,
+            finish_on_success=True,
+            **self.common_job_args(ignore_cache=True),
+        )
+
+    async def autodetect_edition(self):
+        # List of keys in metadata.ptp_editions. The corresponding values are
+        # returned in the same order as they are specified in
+        # metadata.ptp_editions.
+        edition_keys = []
+
+        for key, is_edition in self.autodetect_edition_map.items():
+            if is_edition(self):
+                edition_keys.append(key)
+
+        return ' / '.join(
+            metadata.ptp_editions[key]
+            for key in edition_keys
+        )
+
+    autodetect_edition_map = {
+        'collection.criterion': lambda self: 'Criterion' in self.release_name.edition,
+        # 'collection.masters': lambda self: False,
+        # 'collection.warner': lambda self: False,
+
+        'edition.dc': lambda self: "Director's Cut" in self.release_name.edition,
+        'edition.extended': lambda self: 'Extended Cut' in self.release_name.edition,
+        # 'edition.rifftrax': lambda self: False,
+        'edition.theatrical': lambda self: 'Theatrical Cut' in self.release_name.edition,
+        'edition.uncut': lambda self: 'Uncut' in self.release_name.edition,
+        'edition.unrated': lambda self: 'Unrated' in self.release_name.edition,
+
+        'feature.remux': lambda self: 'Remux' in self.release_name.source,
+        'feature.2in1': lambda self: '2in1' in self.release_name.edition,
+        # 'feature.2disc': lambda self: False,
+        # 'feature.3d_anaglyph': lambda self: False,
+        # 'feature.3d_full_sbs': lambda self: False,
+        # 'feature.3d_half_ou': lambda self: False,
+        # 'feature.3D_half_sbs': lambda self: False,
+        'feature.4krestoration': lambda self: '4k Restored' in self.release_name.edition,
+        'feature.4kremaster': lambda self: '4k Remastered' in self.release_name.edition,
+        'feature.10bit': lambda self: (
+            utils.video.bit_depth(self.content_path, default=None) == 10
+            and 'HDR' not in self.release_name.hdr_format
+            and 'DV' not in self.release_name.hdr_format
+        ),
+        # 'feature.extras': lambda self: False,
+        # 'feature.2d3d_edition': lambda self: False,
+        'feature.dtsx': lambda self: 'DTS:X' in self.release_name.audio_format,
+        'feature.dolby_atmos': lambda self: 'Atmos' in self.release_name.audio_format,
+        'feature.dolby_vision': lambda self: 'DV' in self.release_name.hdr_format,
+        'feature.hdr10': lambda self: re.search(r'HDR10(?!\+)', self.release_name.hdr_format),
+        'feature.hdr10+': lambda self: 'HDR10+' in self.release_name.hdr_format,
+        'feature.dual_audio': lambda self: self.release_name.has_dual_audio,
+        # 'feature.english_dub': lambda self: False,
+        'feature.commentary': lambda self: self.release_name.has_commentary,
+    }
 
     @functools.cached_property
     def tags_job(self):
@@ -323,11 +575,13 @@ class PtpTrackerJobs(TrackerJobsBase):
             label='Tags',
             precondition=self.make_precondition(
                 'tags_job',
+                # Don't run this job if PTP already knows this movie.
                 precondition=self.ptp_group_does_not_exist,
             ),
-            prejobs=(
+            prejobs=self.get_job_and_dependencies(
+                # We need to wait for the PTP group ID to become available
+                # before we can check if it exists.
                 self.ptp_group_id_job,
-                self.imdb_job,
             ),
             text=self.fetch_tags,
             normalizer=self.normalize_tags,
@@ -336,12 +590,12 @@ class PtpTrackerJobs(TrackerJobsBase):
         )
 
     async def fetch_tags(self):
-        assert self.imdb_job.is_finished
+        assert self.imdb_job.is_finished, self.imdb_job
         return await self.tracker.get_ptp_metadata(self.imdb_id, key='tags')
 
     def normalize_tags(self, text):
         tags = [tag.strip() for tag in text.split(',')]
-        deduped = list(dict.fromkeys(tags))
+        deduped = sorted(dict.fromkeys(tags))
         return ', '.join(tag for tag in deduped if tag)
 
     def validate_tags(self, text):
@@ -351,26 +605,29 @@ class PtpTrackerJobs(TrackerJobsBase):
                 raise ValueError(f'Tag is not valid: {tag}')
 
     @functools.cached_property
-    def cover_art_job(self):
-        return jobs.dialog.TextFieldJob(
-            name=self.get_job_name('cover_art'),
-            label='Cover Art',
-            precondition=self.make_precondition(
-                'cover_art_job',
-                precondition=self.ptp_group_does_not_exist,
-            ),
-            prejobs=(
-                self.ptp_group_id_job,
-                self.imdb_job,
-            ),
-            text=self.fetch_cover_art,
-            finish_on_success=True,
-            **self.common_job_args(),
+    def poster_job(self):
+        job = super().poster_job
+        # Hide poster_job until imdb_job and ptp_group_id_job are done.
+        job.prejobs += (
+            self.imdb_job,
+            self.ptp_group_id_job,
         )
+        return job
 
-    async def fetch_cover_art(self):
-        assert self.imdb_job.is_finished
-        return await self.tracker.get_ptp_metadata(self.imdb_id, key='art')
+    async def get_poster_from_tracker(self):
+        """Get poster URL from PTP API"""
+        await self.imdb_job.wait()
+        _log.debug('Getting poster from PTP API: %r', self.imdb_id)
+        poster = await self.tracker.get_ptp_metadata(self.imdb_id, key='art')
+        if poster:
+            # Return original poster URL without resizing or re-uploading.
+            return {
+                'poster': poster,
+                'width': None,
+                'height': None,
+                'imghosts': (),
+                'write_to': None,
+            }
 
     @functools.cached_property
     def plot_job(self):
@@ -379,21 +636,23 @@ class PtpTrackerJobs(TrackerJobsBase):
             label='Plot',
             precondition=self.make_precondition(
                 'plot_job',
+                # Don't run this job if PTP already knows this movie.
                 precondition=self.ptp_group_does_not_exist,
             ),
-            prejobs=(
+            prejobs=self.get_job_and_dependencies(
+                # We need to wait for the PTP group ID to become available
+                # before we can check if it exists.
                 self.ptp_group_id_job,
-                self.imdb_job,
             ),
             text=self.fetch_plot,
-            finish_on_success=True,
             normalizer=self.normalize_plot,
             validator=self.validate_plot,
+            finish_on_success=True,
             **self.common_job_args(),
         )
 
     async def fetch_plot(self):
-        assert self.imdb_job.is_finished
+        assert self.imdb_job.is_finished, self.imdb_job
         return await self.tracker.get_ptp_metadata(self.imdb_id, key='plot')
 
     def normalize_plot(self, text):
@@ -401,23 +660,23 @@ class PtpTrackerJobs(TrackerJobsBase):
 
     def validate_plot(self, text):
         if not text:
-            raise ValueError('Plot must not be empty')
+            raise ValueError('Plot must not be empty.')
 
     @functools.cached_property
     def source_job(self):
         return jobs.dialog.TextFieldJob(
             name=self.get_job_name('source'),
-            label='Video Codec',
+            label='Source',
             precondition=self.make_precondition('source_job'),
             text=self.autodetect_source,
-            finish_on_success=True,
             normalizer=self.normalize_source,
             validator=self.validate_source,
+            finish_on_success=True,
             **self.common_job_args(),
         )
 
     _known_sources = {
-        'Blu-ray': re.compile(r'^(?i:blu-?ray)$'),
+        'Blu-ray': re.compile(r'(?i:blu-?ray)'),  # (UHD) BluRay (Remux)
         'DVD': re.compile(r'^(?i:dvd)'),  # DVD(Rip|5|9|...)
         'WEB': re.compile(r'^(?i:web)'),  # WEB(-DL|Rip)
         'HD-DVD': re.compile(r'^(?i:hd-?dvd)$'),
@@ -430,7 +689,7 @@ class PtpTrackerJobs(TrackerJobsBase):
         # Find autodetected source in valid PTP sources
         source = self.release_name.source
         for known_source, regex in self._known_sources.items():
-            if regex.match(source):
+            if regex.search(source):
                 # Finish the job without prompting the user
                 return known_source
 
@@ -447,6 +706,8 @@ class PtpTrackerJobs(TrackerJobsBase):
     def validate_source(self, text):
         if not text or text == 'UNKNOWN_SOURCE':
             raise ValueError('You must provide a source.')
+        elif text not in self._known_sources:
+            raise ValueError(f'Source is not valid: {text}')
 
     # # POST parameter "codec" seems to be optional.
 
@@ -627,35 +888,36 @@ class PtpTrackerJobs(TrackerJobsBase):
     @property
     def _post_data_common(self):
         return {
-            # Feature Film, Miniseries, Short Film, etc
-            'type': self.get_job_attribute(self.type_job, 'choice'),
+            **{
+                # Feature Film, Miniseries, Short Film, etc
+                'type': self.get_job_attribute(self.type_job, 'choice'),
 
-            # Mediainfo and Screenshots
-            'release_desc': self.get_job_output(self.description_job, slice=0),
+                # Mediainfo and Screenshots
+                'release_desc': self.get_job_output(self.description_job, slice=0),
 
-            # Is not main movie (bool)
-            'special': '1' if self.options['not_main_movie'] else None,
+                # Is not main movie (bool)
+                'special': '1' if self.options['not_main_movie'] else None,
 
-            # Is personal rip (bool)
-            'internalrip': '1' if self.options['personal_rip'] else None,
+                # Is personal rip (bool)
+                'internalrip': '1' if self.options['personal_rip'] else None,
 
-            # Is scene Release (bool)
-            'scene': '1' if self.get_job_attribute(self.scene_check_job, 'is_scene_release') else None,
+                # Is scene Release (bool)
+                'scene': '1' if self.get_job_attribute(self.scene_check_job, 'is_scene_release') else None,
 
-            # .nfo file content
-            'nfo_text': self.read_nfo(),
+                # .nfo file content
+                'nfo_text': self.read_nfo(),
 
-            # Upload token from staff
-            'uploadtoken': self.options.get('upload_token', None),
-        } | (
-            # Merge dictionaries (https://peps.python.org/pep-0584)
-            self._post_data_common_source
-            | self._post_data_common_codec
-            | self._post_data_common_container
-            | self._post_data_common_resolution
-            | self._post_data_common_remaster
-            | self._post_data_common_subtitles
-        )
+                # Upload token from staff
+                'uploadtoken': self.options.get('upload_token', None),
+            },
+            **self._post_data_common_source,
+            **self._post_data_common_codec,
+            **self._post_data_common_container,
+            **self._post_data_common_resolution,
+            **self._post_data_common_edition,
+            **self._post_data_common_subtitles,
+            **self._post_data_common_trumpable,
+        }
 
     @property
     def _post_data_common_source(self):
@@ -716,21 +978,18 @@ class PtpTrackerJobs(TrackerJobsBase):
 
     @property
     def _post_data_common_subtitles(self):
-        # Find all subtitle languages
-        subtitles = utils.subtitle.get_subtitles(self.content_path)
-
         # Translate BCP47 codes into PTP numeric codes from upload.php
         codes = []
-        for sub in subtitles:
+        for sub in self.subtitles:
             ptp_code = None
 
-            if ptp_code is None and sub.territory and sub.forced:
+            if ptp_code is None and sub.region and sub.forced:
                 # Language, country and "forced" flag, e.g. "pt-BR (forced)"
-                ptp_code = metadata.ptp_subtitles.get(f'{sub.language}-{sub.territory} (forced)')
+                ptp_code = metadata.ptp_subtitles.get(f'{sub.language}-{sub.region} (forced)')
 
-            if ptp_code is None and sub.territory:
+            if ptp_code is None and sub.region:
                 # Language and "forced" flag, e.g. "pt-BR"
-                ptp_code = metadata.ptp_subtitles.get(f'{sub.language}-{sub.territory}')
+                ptp_code = metadata.ptp_subtitles.get(f'{sub.language}-{sub.region}')
 
             if ptp_code is None and sub.forced:
                 # Language and "forced" flag, e.g. "pt (forced)"
@@ -743,7 +1002,9 @@ class PtpTrackerJobs(TrackerJobsBase):
             if ptp_code:
                 codes.append(ptp_code)
 
-        # Explicitly send "No subtitles" if there aren't any
+        # Send "No subtitles" if there aren't any or if none of the existing
+        # subtitles are supported in metadata.ptp_subtitles, i.e. we couldn't
+        # find any `codes`.
         if not codes:
             codes.append(metadata.ptp_subtitles['No Subtitles'])
 
@@ -751,16 +1012,29 @@ class PtpTrackerJobs(TrackerJobsBase):
         return {'subtitles[]': codes}
 
     @property
-    def _post_data_common_remaster(self):
+    def _post_data_common_edition(self):
+        text = self.get_job_output(self.edition_job, slice=0)
         return {
-            # 'remaster_title': ...,  # Edition Information ("Director's Cut", "Dual Audio", etc.)
+            # Edition Information ("Director's Cut", "Dual Audio", etc.)
+            'remaster': 'on' if text else None,
+            'remaster_title': text if text else None,
             # 'remaster_year': ...,
             # 'remaster_other_input': ...,
         }
 
+    @property
+    def _post_data_common_trumpable(self):
+        return {
+            'trumpable[]': [
+                # Convert pretty strings to API values
+                metadata.PtpTrumpableReason.from_string(string).value
+                for string in self.trumpable_job.output
+            ],
+        }
+
     @property
     def _post_data_add_movie_format(self):
-        # Upload release to movie that exists on PTP
+        # Upload another release to existing movie group
         return {
             'groupid': self.ptp_group_id,
         }
@@ -782,7 +1056,7 @@ class PtpTrackerJobs(TrackerJobsBase):
             # Youtube ID
             # 'trailer': ...,
             # Poster URL
-            'image': self.get_job_output(self.cover_art_job, slice=0),
+            'image': self.get_job_output(self.poster_job, slice=0),
         }
 
         # TODO: Is this required?
@@ -806,13 +1080,3 @@ class PtpTrackerJobs(TrackerJobsBase):
         #         post_data['role'].append(name)
 
         return post_data
-
-    def read_nfo(self):
-        try:
-            return utils.string.read_nfo(self.content_path)
-        except errors.ContentError as e:
-            self.error(e)
-
-    @property
-    def torrent_filepath(self):
-        return self.get_job_output(self.create_torrent_job, slice=0)
