@@ -2,6 +2,7 @@
 Concrete :class:`~.base.TrackerBase` subclass for PTP
 """
 
+import asyncio
 import math
 import re
 import urllib
@@ -22,31 +23,30 @@ class PtpTracker(TrackerBase):
     setup_howto_template = (
         '{howto.introduction}\n'
         '\n'
-        '{howto.current_section}. Login Credentials\n'
+        '{howto.next_section}. Login Credentials\n'
         '\n'
-        '   {howto.current_section}.1 $ upsies set trackers.ptp.username USERNAME\n'
-        '   {howto.current_section}.2 $ upsies set trackers.ptp.password PASSWORD\n'
-        '{howto.bump_section}'
+        '   {howto.current_section}.1 $ upsies set trackers.{tracker.name}.username USERNAME\n'
+        '   {howto.current_section}.2 $ upsies set trackers.{tracker.name}.password PASSWORD\n'
         '\n'
-        '{howto.current_section}. Announce URL\n'
+        '{howto.next_section}. Announce URL\n'
         '\n'
         '   The announce URL is required because the passkey is used for 2FA.\n'
         '\n'
-        '   {howto.current_section}.1 $ upsies set trackers.ptp.announce_url ANNOUNCE_URL\n'
-        '{howto.bump_section}'
+        '   {howto.current_section}.1 $ upsies set trackers.{tracker.name}.announce_url ANNOUNCE_URL\n'
         '\n'
-        '{howto.current_section}. Screenshots\n'
+        '{howto.next_section}. Screenshots\n'
         '\n'
         '   {howto.current_section}.1 Configure ptpimg.me API\n'
         '       $ upsies upload-images ptpimg --help\n'
         '\n'
         '   {howto.current_section}.2 Specify how many screenshots to make. (optional)\n'
-        '       $ upsies set trackers.ptp.screenshots_from_movie NUMBER_OF_MOVIE_SCREENSHOTS\n'
-        '       $ upsies set trackers.ptp.screenshots_from_episode NUMBER_OF_EPISODE_SCREENSHOTS\n'
-        '{howto.bump_section}'
+        '       $ upsies set trackers.{tracker.name}.screenshots_from_movie NUMBER_OF_MOVIE_SCREENSHOTS\n'
+        '       $ upsies set trackers.{tracker.name}.screenshots_from_episode NUMBER_OF_EPISODE_SCREENSHOTS\n'
         '\n'
         '{howto.autoseed}\n'
         '\n'
+        '{howto.reuse_torrents}\n'
+        '\n'
         '{howto.upload}\n'
     )
 
@@ -202,9 +202,6 @@ class PtpTracker(TrackerBase):
             url=f'{self._ajax_url}?action=login',
             data=post_data,
             error_prefix='Login failed',
-
-            # TODO: Remove this before release
-            debug_file='http_dumps.ptp/ptp_login',
         )
         self._anti_csrf_token = self._get_anti_csrf_token(response)
 
@@ -216,9 +213,6 @@ class PtpTracker(TrackerBase):
                 url=self._logout_url,
                 params={'auth': await self._get_auth()},
                 error_prefix='Logout failed',
-
-                # TODO: Remove this before release
-                debug_file='http_dumps.ptp/ptp_logout',
             )
 
         finally:
@@ -229,7 +223,7 @@ class PtpTracker(TrackerBase):
         return self._announce_url
 
     async def upload(self, tracker_jobs):
-        post_data = tracker_jobs.post_data.copy()
+        post_data = tracker_jobs.post_data
         post_data['AntiCsrfToken'] = self._anti_csrf_token
 
         _log.debug('POSTing data:')
@@ -253,11 +247,14 @@ class PtpTracker(TrackerBase):
             # Ignore the HTTP redirect (should be 302 Found) so we can get the
             # torrent URL from the "Location" response header
             follow_redirects=False,
-
-            # TODO: Remove this before release
-            debug_file='http_dumps.ptp/ptp_upload',
         )
 
+        # The server needs some time to process the uploaded metadata before the
+        # torrent is registered by the tracker and we can seed it. If don't
+        # sleep() here, the torrent may be announced before the tracker knows it
+        # exists.
+        await asyncio.sleep(1)
+
         # # Read previously received response for debugging
         # response_filepath = 'ptp_upload.response.content'
         # response = utils.http.Response(
@@ -346,45 +343,58 @@ class PtpTracker(TrackerBase):
 
     async def get_ptp_metadata(self, imdb_id, key=None):
         """
-        Get metadata from PTP website
+        Get metadata as :class:`dict` from PTP website
+
+        :param imdb_id: IMDb ID (e.g. ``tt123456``)
+        :param key: Return this key's value from metadata dictionary
+
+        If there is no metadata for `imdb_id`, an empty :class:`dict` is returned.
+
+        If `key` is provided and it doesn't exist in the metadata, return `None`.
 
         Any :class:`~.RequestError` is caught and passed to
         :meth:`.TrackerBase.error`.
-
-        :param imdb_id: IMDb ID (e.g. ``tt123456``)
         """
         if key:
             _log.debug('%s: Fetching %s from PTP', imdb_id, key)
         else:
             _log.debug('%s: Fetching metadata from PTP', imdb_id)
 
-        assert imdb_id, imdb_id
-
         try:
-            await self.login()
-            response = await self._request(
-                method='GET',
-                url=self._ajax_url,
-                params={
-                    'action': 'torrent_info',
-                    'imdb': self.normalize_imdb_id(imdb_id),
-                },
-                cache=True,
-            )
-            # Raise RequestError if response is not valid JSON
-            results = response.json()
+            if imdb_id:
+                await self.login()
+                response = await self._request(
+                    method='GET',
+                    url=self._ajax_url,
+                    params={
+                        'action': 'torrent_info',
+                        'imdb': self.normalize_imdb_id(imdb_id),
+                    },
+                    cache=True,
+                )
+                _log.debug('%s: Raw metadata: %r', imdb_id, response)
+                if not response:
+                    # PTP has no metadata for this IMDb ID
+                    results = [{}]
+                else:
+                    # Raise RequestError if response is not valid JSON
+                    results = response.json()
+
+            else:
+                # No IMDb ID available - no metadata
+                results = [{}]
 
         except errors.RequestError as e:
             self.error(e)
 
         else:
-            assert len(results) == 1
+            assert len(results) == 1, results
             if key is None:
                 _log.debug('%s: PTP metadata: %s', imdb_id, results[0])
                 return results[0]
             else:
-                _log.debug('%s: PTP %s: %s', imdb_id, key, results[0][key])
-                return results[0][key]
+                _log.debug('%s: PTP %s: %s', imdb_id, key, results[0].get(key))
+                return results[0].get(key)
 
     @staticmethod
     def calculate_piece_size(bytes):
@@ -398,7 +408,7 @@ class PtpTracker(TrackerBase):
         """
         exponent = math.ceil(math.log2(bytes / 1050))
         # Allowed piece size range: 32 KiB ... 16 MiB
-        exponent = min(24, max(15, exponent))
+        exponent = max(15, min(24, exponent))
         return int(math.pow(2, exponent))
 
     @staticmethod
@@ -414,23 +424,29 @@ class PtpTracker(TrackerBase):
 
         :raise ValueError: if `bytes` is negative or otherwise unexpected
         """
-        # All numbers are from PTP's piece size chart.
-        # (MIN_SIZE, MAX_SIZE): (PIECE_SIZE_MIN, PIECE_SIZE_MAX)
-        allowed_piece_sizes = {
-            (0, 147 * 1024 * 1024 - 1): (2**15, 2**16),
-            (147 * 1024 * 1024, 256 * 1024 * 1024 - 1): (2**16, 2**19),
-            (256 * 1024 * 1024, 533 * 1024 * 1024 - 1): (2**17, 2**20),
-            (533 * 1024 * 1024, 1.08 * 1024 * 1024 * 1024 - 1): (2**18, 2**21),
-            (1.08 * 1024 * 1024 * 1024, 2.24 * 1024 * 1024 * 1024 - 1): (2**19, 2**22),
-            (2.24 * 1024 * 1024 * 1024, 4.65 * 1024 * 1024 * 1024 - 1): (2**20, 2**23),
-            (4.65 * 1024 * 1024 * 1024, 8.04 * 1024 * 1024 * 1024 - 1): (2**21, 2**24),
-            (8.04 * 1024 * 1024 * 1024, 148.75 * 1024 * 1024 * 1024 - 1): (2**22, 2**24),
-            (148.75 * 1024 * 1024 * 1024, 257.04 * 1024 * 1024 * 1024 - 1): (2**23, 2**24),
-            (257.04 * 1024 * 1024 * 1024, float('inf')): (2**24, 2**24),
-        }
+        if bytes <= 0:
+            raise ValueError(f'Unexpected size: {bytes!r}')
 
-        for (min_size, max_size), (piece_size_min, piece_size_max) in allowed_piece_sizes.items():
-            if min_size <= bytes <= max_size:
-                return piece_size_min, piece_size_max
+        # NOTE: The algorithm below is from the website's javascript.
 
-        raise ValueError(f'Unexpected size: {bytes!r}')
+        max_exponent = math.ceil(math.log2(bytes / 500))
+        min_exponent = math.floor(math.log2(bytes / 2000))
+
+        # 2^24 or 16MiB is the max for uTorrent 2.x
+        min_exponent = min(24, min_exponent)
+        max_exponent = min(24, max_exponent)
+
+        # Compatibility for uTorrent 2.x creator which only supports up to 4 MiB piece sizes
+        if min_exponent > 22 and bytes <= 128 * 1024 * 1024 * 1024:
+            min_exponent = 22
+        if min_exponent > 22 and bytes <= 256 * 1024 * 1024 * 1024:
+            min_exponent = 23
+
+        # Tiny torrents < ~40MiB need a sane lower bound
+        min_exponent = max(15, min_exponent)
+        max_exponent = max(18, max_exponent)
+
+        return (
+            math.pow(2, min_exponent),
+            math.pow(2, max_exponent),
+        )
