@@ -5,7 +5,6 @@ Abstract base class for commands
 import abc
 import argparse
 import collections
-import os
 import re
 import sys
 import textwrap
@@ -96,7 +95,7 @@ class CommandBase(abc.ABC):
     _keybindings_description = (
         "You can cancel any command with Escape, Ctrl+c or Ctrl+g.\n"
         "\n"
-        "In the TUI, the usual Emacs keybindings are supported. Here is an incomplete list:\n"
+        "In the TUI, the usual readline/Emacs keybindings are supported. Here is an incomplete list:\n"
         "\n"
         "  Ctrl+f         Move forward character-wise\n"
         "  Ctrl+b         Move backward character-wise\n"
@@ -403,11 +402,12 @@ class CommandBase(abc.ABC):
         Passed as `cache_directory` argument to :class:`.jobs.base.JobBase`
         instances that are instantiated by this class
 
-        The default implementation appends ``.cache`` to :attr:`home_directory`
-        if the ``CONTENT`` or ``RELEASE`` argument exists and returns
+        The default implementation returns :attr:`home_directory`
+        (e.g. ``~/.cache/upsies/Foo.2012-ASDF.upsies/``) if the ``CONTENT`` or
+        ``RELEASE`` argument exists and returns
         ``config.main.cache_directory`` otherwise.
         """
         if hasattr(self.args, 'CONTENT') or hasattr(self.args, 'RELEASE'):
-            return os.path.join(self.home_directory, '.cache')
+            return self.home_directory
         else:
             return self.config['config']['main']['cache_directory']
