@@ -28,21 +28,29 @@ class poster(CommandBase):
             'default': None,
         },
         ('--upload-to', '-u'): {
-            'type': utils.argtypes.imghost,
-            'metavar': 'IMAGEHOST',
-            'help': ('Case-insensitive name of image hosting service\n'
+            'type': type(utils.types.ListOf(
+                item_type=utils.argtypes.imghost,
+                separator=',',
+            )),
+            'metavar': 'IMAGE_HOST',
+            'help': ('Comma-separated list of case-insensitive names of image hosting services\n'
+                     'Only the URL from the first successfull upload is used.\n'
                      'Supported services: ' + ', '.join(utils.imghosts.imghost_names())),
-            'default': None,
+            'default': (),
         },
         ('--width', '-w'): {
-            'help': 'Maximum poster width in pixels (default: 500)',
+            'help': 'Poster width in pixels (default: 0 (no resizing))',
             'type': utils.argtypes.integer,
-            'default': 500,
+            'default': 0,
         },
         ('--height', '-t'): {
-            'help': 'Maximum poster height in pixels (default: 1000)',
+            'help': 'Poster height in pixels (default: 0 (no resizing))',
             'type': utils.argtypes.integer,
-            'default': 1000,
+            'default': 0,
+        },
+        ('--output', '-o'): {
+            'help': 'Write poster to file OUTPUT',
+            'default': None,
         },
     }
 
@@ -61,13 +69,15 @@ class poster(CommandBase):
             return utils.webdbs.webdb('imdb')
 
     @functools.cached_property
-    def imghost(self):
-        if self.args.upload_to:
-            return utils.imghosts.imghost(
-                name=self.args.upload_to,
-                options=self.config['imghosts'][self.args.upload_to],
+    def imghosts(self):
+        return tuple(
+            utils.imghosts.imghost(
+                name=imghost_name,
+                options=self.config['imghosts'][imghost_name],
                 cache_directory=self.cache_directory,
             )
+            for imghost_name in self.args.upload_to
+        )
 
     @functools.cached_property
     def jobs(self):
@@ -84,27 +94,31 @@ class poster(CommandBase):
             ignore_cache=self.args.ignore_cache,
             query=utils.webdbs.Query.from_release(self.release_name),
             db=self.webdb,
-            callbacks={
-                'output': self.start_poster_job,
-            },
         )
 
-    def start_poster_job(self, webdb_id):
-        self.poster_job.start(
-            webdb_id=webdb_id,
-            season=self.release_name.only_season,
-        )
+    @property
+    def webdb_id(self):
+        if self.webdb_job.is_finished and self.webdb_job.output:
+            return self.webdb_job.output[0]
 
     @functools.cached_property
     def poster_job(self):
         return jobs.poster.PosterJob(
-            home_directory=self.home_directory if self.imghost else '.',
+            home_directory=self.home_directory if self.imghosts else '.',
             cache_directory=self.cache_directory,
             ignore_cache=self.args.ignore_cache,
-            autostart=False,
-            webdb_id=None,
-            webdb=self.webdb,
-            imghost=self.imghost,
+            prejobs=(
+                self.webdb_job,
+            ),
+            getter=self.get_poster,
             width=self.args.width,
             height=self.args.height,
+            write_to=self.args.output,
+            imghosts=self.imghosts,
+        )
+
+    async def get_poster(self):
+        return await self.webdb.poster_url(
+            self.webdb_id,
+            season=self.release_name.only_season,
         )
