@@ -50,21 +50,24 @@ class release_name(CommandBase):
         return jobs.dialog.TextFieldJob(
             name='release-name',
             label='Release Name',
-            text=str(self.release_name),
             home_directory=self.home_directory,
             cache_directory=self.cache_directory,
+            cache_id=f'separator={self.args.separator}',
             ignore_cache=self.args.ignore_cache,
+            prejobs=(
+                self.imdb_job,
+            ),
+            text=self.get_release_name,
         )
 
-    def _update_release_name(self, imdb_id):
-        self.release_name_job.attach_task(
-            self.release_name_job.fetch_text(
-                coro=self.release_name.fetch_info(webdb=self.imdb, webdb_id=imdb_id),
-                nonfatal_exceptions=(
-                    errors.RequestError,
-                ),
-                default=str(self.release_name),
+    async def get_release_name(self):
+        imdb_id = self.imdb_job.output[0]
+        await self.release_name_job.fetch_text(
+            coro=self.release_name.fetch_info(webdb=self.imdb, webdb_id=imdb_id),
+            nonfatal_exceptions=(
+                errors.RequestError,
             ),
+            default=self.release_name,
         )
 
     @functools.cached_property
@@ -77,7 +80,6 @@ class release_name(CommandBase):
             ignore_cache=self.args.ignore_cache,
             query=utils.webdbs.Query.from_path(self.args.RELEASE),
             db=self.imdb,
-            callbacks={'output': self._update_release_name},
         )
 
     @functools.cached_property
