@@ -56,10 +56,12 @@ class screenshots(CommandBase):
             'help': f'File size optimization level: {", ".join(utils.image.optimization_levels)}',
         },
         ('--upload-to', '-u'): {
-            'type': utils.argtypes.imghost,
-            'metavar': 'IMAGEHOST',
-            'help': ('Case-insensitive name of image hosting service\n'
-                     'Supported services: ' + ', '.join(utils.imghosts.imghost_names())),
+            'type': utils.argtypes.imghosts,
+            'metavar': 'IMAGE_HOSTS',
+            'help': (
+                'Comma-separated list of case-insensitive image hosting service names\n'
+                'Supported services: ' + ', '.join(utils.imghosts.imghost_names())
+            ),
         },
         ('--output-directory', '-o'): {
             'default': '',  # Current working directory
@@ -96,10 +98,7 @@ class screenshots(CommandBase):
                 home_directory=self.home_directory,
                 cache_directory=self.cache_directory,
                 ignore_cache=self.args.ignore_cache,
-                imghost=utils.imghosts.imghost(
-                    name=self.args.upload_to,
-                    options=self.config['imghosts'][self.args.upload_to],
-                ),
+                imghosts=self.imghosts,
             )
             # Timestamps and number of screenshots are determined in a
             # subprocess, we have to wait for that before we can set the number
@@ -108,9 +107,19 @@ class screenshots(CommandBase):
             # Pass ScreenshotsJob's output to ImageHostJob input.
             self.screenshots_job.signal.register('output', imghost_job.enqueue)
             # Tell imghost_job to finish the current upload and then finish.
-            self.screenshots_job.signal.register('finished', lambda _: imghost_job.finalize())
+            self.screenshots_job.signal.register('finished', lambda _: imghost_job.close())
             return imghost_job
 
+    @functools.cached_property
+    def imghosts(self):
+        return tuple(
+            utils.imghosts.imghost(
+                name=name,
+                options=self.config['imghosts'][name],
+            )
+            for name in self.args.upload_to
+        )
+
     @functools.cached_property
     def jobs(self):
         return (
