@@ -3,6 +3,7 @@ Change or show configuration file options
 """
 
 import functools
+import os
 import textwrap
 
 from .... import constants, defaults, errors, jobs, utils
@@ -33,7 +34,7 @@ def _make_pretty_option_list():
         elif option_type_name == 'str':
             option_type_name = 'string'
         elif option_type_name == 'choice':
-            text.append(f'  Choices: {", ".join(default_value.options)}')
+            text.append(f'  Options: {", ".join(default_value.options)}')
 
         # Add description indented
         description = getattr(option_type, 'description', '')
@@ -59,9 +60,9 @@ class set(CommandBase):
     Without any arguments, all options are listed with their current values.
 
     OPTION consists of three segments which are delimited with a period (".").
-    The first segment in OPTION is the file name without the extension. The
-    second segment is the section name in that file. The third segment is the
-    option name.
+    The first segment in OPTION is the INI file name without the extension.
+    The second segment is the section name in that file. The third segment
+    is the option name.
 
     List values are given as one argument per list item. If non-list values are
     given as multiple arguments, they are concatenated with single spaces. In
@@ -90,12 +91,28 @@ class set(CommandBase):
             'help': 'Reset OPTION to default value',
             'group': 'value',
         },
+        ('--dump', '-d'): {
+            'type': utils.argtypes.one_of(tuple(defaults.defaults)),
+            'nargs': '*',
+            'metavar': 'FILE',
+            'help': (
+                f'Write current configuration to '
+                f'{utils.fs.tildify_path(os.path.join(constants.CONFIG_DIRECTORYPATH, "FILE"))}.ini\n'
+                'with commented-out default values.\n'
+                f'FILE may be ' + ", ".join(sorted(f'"{x}"' for x in defaults.defaults)) +
+                ' or omitted to write all configuration files.\n'
+                'WARNING: You will lose any comments and custom structure in your .ini files.'
+            ),
+            'group': 'value',
+        },
         ('--fetch-ptpimg-apikey',): {
             'nargs': 2,
             'metavar': ('EMAIL', 'PASSWORD'),
-            'help': ('Fetch ptpimg API key and save it in '
-                     f'{utils.fs.tildify_path(constants.IMGHOSTS_FILEPATH)}\n'
-                     '(EMAIL and PASSWORD are not saved)'),
+            'help': (
+                'Fetch ptpimg API key and save it in '
+                f'{utils.fs.tildify_path(constants.IMGHOSTS_FILEPATH)}\n'
+                '(EMAIL and PASSWORD are not saved)'
+            ),
         },
     }
 
@@ -112,9 +129,19 @@ class set(CommandBase):
                     # strings (and vice versa) depending on the default type.
                     value=self.args.VALUE,
                     reset=self.args.reset,
+                    dump=self.dump_sections,
                 ),
             )
 
+    @property
+    def dump_sections(self):
+        if self.args.dump:
+            return self.args.dump
+        elif self.args.dump is None:
+            return None
+        else:
+            return tuple(self.config)
+
     @functools.cached_property
     def fetch_ptpimg_apikey_job(self):
         return jobs.custom.CustomJob(
@@ -141,4 +168,3 @@ class set(CommandBase):
             job.send(apikey)
             self.config['imghosts']['ptpimg']['apikey'] = apikey
             self.config.write('imghosts.ptpimg.apikey')
-            job.finish()
