@@ -39,7 +39,7 @@ class submit(base.CommandBase):
                         'help': 'Path to release content',
                     },
                     ('--is-scene',): {
-                        'type': utils.argtypes.is_scene,
+                        'type': utils.argtypes.bool_or_none,
                         'default': None,
                         'help': ('Whether this is a scene release (usually autodetected)\n'
                                  'Valid values: '
@@ -140,15 +140,15 @@ class submit(base.CommandBase):
         of :mod:`.trackers`
         """
         return self.tracker.TrackerJobs(
+            tracker=self.tracker,
             options=self.tracker_options,
             content_path=self.args.CONTENT,
             reuse_torrent_path=(
                 tuple(self.args.reuse_torrent)
                 + tuple(self.config['config']['torrent-create']['reuse_torrent_paths'])
             ),
-            tracker=self.tracker,
             screenshots_optimization=self.config['config']['screenshots']['optimize'],
-            image_host=self._get_imghost(),
+            image_hosts=self._get_image_hosts(),
             btclient=self._get_btclient(),
             torrent_destination=self._get_torrent_destination(),
             exclude_files=(
@@ -163,27 +163,36 @@ class submit(base.CommandBase):
             },
         )
 
-    def _get_imghost(self):
-        imghost_name = self.tracker_options.get('image_host', None)
-        if imghost_name:
-            # Get global image host options
-            options = self.config['imghosts'][imghost_name].copy()
+    def _get_image_hosts(self):
+        return tuple(
+            self._get_image_host(name)
+            for name in self.tracker_options.get('image_host', ())
+        )
 
-            # Apply tracker-specific options that are common for all image hosts
-            options.update(self.tracker.TrackerJobs.image_host_config.get('common', {}))
+    def _get_image_host(self, name):
+        # Get global image host options from imghosts.ini
+        name = str(name).lower()
+        options = self.config['imghosts'][name].copy()
 
-            # Apply tracker-specific options for the used image host
-            options.update(self.tracker.TrackerJobs.image_host_config.get(imghost_name, {}))
+        # Apply tracker-specific options that are common for all image hosts
+        # (e.g. thumbnail size)
+        options.update(self.tracker.TrackerJobs.image_host_config.get('common', {}))
 
-            return utils.imghosts.imghost(
-                name=imghost_name,
-                options=options,
-            )
+        # Apply tracker-specific options for the used image host
+        options.update(self.tracker.TrackerJobs.image_host_config.get(name, {}))
+
+        return utils.imghosts.imghost(
+            name=name,
+            options=options,
+            cache_directory=self.home_directory,
+        )
 
     def _get_btclient_name(self):
-        return (getattr(self.args, 'add_to', None)
-                or self.tracker_options.get('add_to', None)
-                or None)
+        return (
+            getattr(self.args, 'add_to', None)
+            or self.tracker_options.get('add_to', None)
+            or None
+        )
 
     def _get_btclient(self):
         client_name = self._get_btclient_name()
@@ -194,11 +203,17 @@ class submit(base.CommandBase):
                 url=options['url'],
                 username=options['username'],
                 password=options['password'],
-                download_path=utils.fs.dirname(os.path.abspath(self.args.CONTENT)),
+                # See torrent_create.download_path for some reasoning.
+                download_path=options['translate_path'].translate(
+                    os.path.abspath(utils.fs.dirname(self.args.CONTENT))
+                ),
                 check_after_add=options['check_after_add'],
+                category=options.get('category', ''),
             )
 
     def _get_torrent_destination(self):
-        return (getattr(self.args, 'copy_to', None)
-                or self.tracker_options.get('copy_to', None)
-                or None)
+        return (
+            getattr(self.args, 'copy_to', None)
+            or self.tracker_options.get('copy_to', None)
+            or None
+        )
