@@ -32,25 +32,41 @@ class torrent_add(CommandBase):
 
     @functools.cached_property
     def jobs(self):
-        client_name = self.args.CLIENT
-        options = self.get_options('clients', client_name)
         return (
             jobs.torrent.AddTorrentJob(
                 home_directory=self.home_directory,
                 cache_directory=self.cache_directory,
                 ignore_cache=self.args.ignore_cache,
                 btclient=utils.btclient.BtClient(
-                    name=client_name,
-                    url=options['url'],
-                    username=options['username'],
-                    password=options['password'],
-                    download_path=self.args.download_path,
-                    check_after_add=options['check_after_add'],
+                    name=self.args.CLIENT,
+                    url=self.client_options['url'],
+                    username=self.client_options['username'],
+                    password=self.client_options['password'],
+                    download_path=self.download_path,
+                    check_after_add=self.client_options['check_after_add'],
+                    category=self.client_options.get('category', ''),
                 ),
                 enqueue=self.args.TORRENT,
             ),
         )
 
+    @functools.cached_property
+    def client_options(self):
+        return self.get_options('clients', self.args.CLIENT)
+
+    @functools.cached_property
+    def download_path(self):
+        if self.args.download_path:
+            # Make provided --download-path absolute before doing path
+            # translation to get the path the client should use, which may run
+            # on a different computer.
+            return self.client_options['translate_path'].translate(
+                os.path.abspath(self.args.download_path)
+            )
+        else:
+            # Use client's default path
+            return None
+
 
 class torrent_create(CommandBase):
     """
@@ -129,6 +145,16 @@ class torrent_create(CommandBase):
     def tracker_name(self):
         return self.args.subcommand.lower()
 
+    @functools.cached_property
+    def tracker(self):
+        return trackers.tracker(
+            name=self.tracker_name,
+            options={
+                **self.config['trackers'][self.tracker_name],
+                **vars(self.args),
+            },
+        )
+
     @property
     def home_directory(self):
         """Create torrent file in current working directory"""
@@ -145,34 +171,41 @@ class torrent_create(CommandBase):
                 tuple(self.args.reuse_torrent)
                 + tuple(self.config['config']['torrent-create']['reuse_torrent_paths'])
             ),
-            tracker=trackers.tracker(
-                name=self.tracker_name,
-                options={**self.config['trackers'][self.tracker_name],
-                         **vars(self.args)},
-            ),
+            tracker=self.tracker,
             exclude_files=(
-                tuple(self.config['trackers'][self.tracker_name]['exclude'])
-                + tuple(self.args.exclude_files)
+                tuple(self.args.exclude_files)
                 + tuple(self.args.exclude_files_regex)
             ),
+            callbacks={
+                'announce_url': self.logout_after_getting_announce_url,
+            },
         )
 
+    def logout_after_getting_announce_url(self, announce_url):
+        # The "announce_url" signal is emitted twice: First we attempt to get
+        # the announce URL and the signal callback is called with `Ellipsis`.
+        # Then the signal is emitted again with the announce URL.
+        if announce_url is not Ellipsis:
+            async def logout():
+                await self.tracker.logout()
+
+            self.create_torrent_job.add_task(logout())
+
     @functools.cached_property
     def add_torrent_job(self):
         if self.args.add_to:
-            client_name = self.args.add_to
-            options = self.get_options('clients', client_name)
             add_torrent_job = jobs.torrent.AddTorrentJob(
                 home_directory=self.home_directory,
                 cache_directory=self.cache_directory,
                 ignore_cache=self.args.ignore_cache,
                 btclient=utils.btclient.BtClient(
-                    name=client_name,
-                    url=options['url'],
-                    username=options['username'],
-                    password=options['password'],
-                    download_path=utils.fs.dirname(os.path.abspath(self.args.CONTENT)),
-                    check_after_add=options['check_after_add'],
+                    name=self.args.add_to,
+                    url=self.client_options['url'],
+                    username=self.client_options['username'],
+                    password=self.client_options['password'],
+                    download_path=self.download_path,
+                    check_after_add=self.client_options['check_after_add'],
+                    category=self.client_options.get('category', ''),
                 ),
             )
 
@@ -182,10 +215,29 @@ class torrent_create(CommandBase):
             # Finish AddTorrentJob when CreateTorrentJob is finished. The
             # torrent will be enqueued and AddTorrentJob will finish after
             # adding the enqueued torrent.
-            self.create_torrent_job.signal.register('finished', lambda _: add_torrent_job.finalize())
+            self.create_torrent_job.signal.register('finished', lambda _: add_torrent_job.close())
 
             return add_torrent_job
 
+    @functools.cached_property
+    def client_options(self):
+        return self.get_options('clients', self.args.add_to)
+
+    @functools.cached_property
+    def download_path(self):
+        # By default, we assume we are running on the same computer as the
+        # client, in which case the download_path is identical to the parent
+        # directory of the provided CONTENT path.
+        #
+        # If the client is running on a different computer, we apply a
+        # user-configured search-and-replace path translation.
+        #
+        # In either case, we first make the download_path absolute for easier
+        # trouble-shooting and to make the path translation predictable.
+        return self.client_options['translate_path'].translate(
+            os.path.abspath(utils.fs.dirname(self.args.CONTENT))
+        )
+
     @functools.cached_property
     def copy_torrent_job(self):
         if self.args.copy_to:
@@ -198,7 +250,7 @@ class torrent_create(CommandBase):
             # Pass CreateTorrentJob output to CopyTorrentJob input.
             self.create_torrent_job.signal.register('output', copy_torrent_job.enqueue)
             # Tell CopyTorrentJob to finish when CreateTorrentJob is done.
-            self.create_torrent_job.signal.register('finished', lambda _: copy_torrent_job.finalize())
+            self.create_torrent_job.signal.register('finished', lambda _: copy_torrent_job.close())
             return copy_torrent_job
 
     @functools.cached_property
