@@ -15,7 +15,7 @@ class upload_images(CommandBase):
 
     argument_definitions = {}
 
-    subcommand_name = 'IMGHOST'
+    subcommand_name = 'IMAGE_HOST'
     subcommands = {
         imghost.name: {
             'description': imghost.description,
@@ -60,9 +60,11 @@ class upload_images(CommandBase):
                 home_directory=self.home_directory,
                 cache_directory=self.cache_directory,
                 ignore_cache=self.args.ignore_cache,
-                imghost=utils.imghosts.imghost(
-                    name=self.imghost_name,
-                    options=self.imghost_options,
+                imghosts=(
+                    utils.imghosts.imghost(
+                        name=self.imghost_name,
+                        options=self.imghost_options,
+                    ),
                 ),
                 enqueue=self.args.IMAGE,
             ),
