@@ -7,9 +7,10 @@ returns `False`.
 Headless usage is limited to :attr:`~.jobs.base.JobBase.hidden` jobs.
 """
 
+import asyncio
 import sys
 
-from ... import errors, utils
+from ... import errors
 
 import logging  # isort:skip
 _log = logging.getLogger(__name__)
@@ -18,7 +19,6 @@ _log = logging.getLogger(__name__)
 class Headless:
     def __init__(self):
         self._jobs = {}
-        self._exception = None
 
     def add_jobs(self, *jobs):
         """Add :class:`~.jobs.base.JobBase` instances"""
@@ -49,14 +49,11 @@ class Headless:
         """
         self.add_jobs(*jobs)
 
-        for job in self._enabled_jobs:
-            _log.debug('Starting job: %r', job.name)
-            job.start()
-        utils.get_aioloop().run_until_complete(self._wait_for_jobs())
+        asyncio.run(self._wait_for_jobs())
 
         # First non-zero exit_code is the application exit_code
         for job in self._enabled_jobs:
-            _log.debug('Checking exit_code of %r: %r', job.name, job.exit_code)
+            _log.debug('%s: Exit code: %r', job.name, job.exit_code)
             if job.exit_code != 0:
                 for error in job.errors:
                     print(f'{job.name}: {error}', file=sys.stderr)
@@ -70,23 +67,26 @@ class Headless:
     def _exit_if_all_jobs_finished(self, *_):
         if all(job.is_finished for job in self._enabled_jobs):
             _log.debug('All jobs finished')
-            self._finish_jobs()
+            self._exit()
 
     def _exit_if_job_failed(self, job):
         if job.is_finished and job.exit_code != 0:
             _log.debug('Terminating application because of failed job: %r', job.name)
-            self._finish_jobs()
+            self._exit()
 
     async def _wait_for_jobs(self):
-        _log.debug('Terminating all jobs')
         for job in self._enabled_jobs:
-            if job.is_started and not job.is_finished:
+            _log.debug('Starting job: %r', job.name)
+            job.start()
+
+        for job in self._enabled_jobs:
+            if job.was_started and not job.is_finished:
                 _log.debug('Waiting for %r', job.name)
                 await job.wait()
                 _log.debug('Done waiting for %r', job.name)
 
-    def _finish_jobs(self):
+    def _exit(self):
         for job in self._enabled_jobs:
             if not job.is_finished:
-                _log.debug('Finishing %s', job.name)
-                job.finish()
+                _log.debug('Terminating %s', job.name)
+                job.terminate()
