@@ -3,10 +3,11 @@ import functools
 
 from prompt_toolkit.filters import Condition
 from prompt_toolkit.key_binding import KeyBindings
-from prompt_toolkit.layout.containers import ConditionalContainer, HSplit, Window, to_container
+from prompt_toolkit.layout.containers import ConditionalContainer, DynamicContainer, HSplit, Window, to_container
 from prompt_toolkit.layout.controls import FormattedTextControl
 from prompt_toolkit.layout.layout import walk
 
+from ... import prompts
 from .. import widgets
 
 import logging  # isort:skip
@@ -16,7 +17,7 @@ _log = logging.getLogger(__name__)
 class JobWidgetBase(abc.ABC):
     """User-interaction and information display for :class:`~.jobs.JobBase` instance"""
 
-    _no_runtime_widget = Window(
+    _empty_widget = Window(
         dont_extend_height=True,
         style='class:info',
     )
@@ -24,15 +25,19 @@ class JobWidgetBase(abc.ABC):
     def __init__(self, job, app):
         self._job = job
         self._app = app
+        self._prompts = []
         self.setup()
+
         self.job.signal.register('info', lambda _: self.invalidate())
         self.job.signal.register('warning', lambda _: self.invalidate())
+        self.job.signal.register('prompt', self.add_prompt)
+
         main_widget = HSplit(
             children=[
                 # Status information or user interaction
                 ConditionalContainer(
                     filter=Condition(lambda: not self.job.is_finished),
-                    content=self._runtime_widget or self._no_runtime_widget,
+                    content=self._prompt_or_runtime_widget,
                 ),
                 # Result
                 ConditionalContainer(
@@ -58,12 +63,14 @@ class JobWidgetBase(abc.ABC):
                 ),
             ],
         )
+
         label = widgets.HLabel(
             group='jobs',
             text=self.job.label,
             style='class:label',
             content=main_widget,
         )
+
         self._container = ConditionalContainer(
             filter=Condition(lambda: self.job.errors or not self.job.hidden),
             content=label,
@@ -86,20 +93,31 @@ class JobWidgetBase(abc.ABC):
     @abc.abstractmethod
     def runtime_widget(self):
         """
-        Interactive or status that is displayed while this job is running
+        Interactive or status widget that is displayed while this job is running
 
         :return: :class:`~.prompt_toolkit.layout.containers.Window` object or
             `None`
         """
 
     @functools.cached_property
-    def _runtime_widget(self):
-        runtime_widget = self.runtime_widget
-        if runtime_widget:
-            container = to_container(runtime_widget)
+    def _runtime_widget_with_local_keybindings(self):
+        if self.runtime_widget:
+            container = to_container(self.runtime_widget)
             container.key_bindings = self.keybindings_local
             return container
 
+    @functools.cached_property
+    def _prompt_or_runtime_widget(self):
+        def get_content():
+            if self.current_prompt_widget:
+                return self.current_prompt_widget
+            elif self._runtime_widget_with_local_keybindings:
+                return self._runtime_widget_with_local_keybindings
+            else:
+                return self._empty_widget
+
+        return DynamicContainer(get_content)
+
     @property
     def output_widget(self):
         """
@@ -159,13 +177,69 @@ class JobWidgetBase(abc.ABC):
             wrap_lines=True,
         )
 
-    @property
+    def add_prompt(self, prompt):
+        self._prompts.append(prompt)
+        self._clear_cached_prompt_widget()
+
+    @functools.cached_property
+    def current_prompt_widget(self):
+        if self._prompts:
+            current_prompt = self._prompts[0]
+            return self._prompt_widget_factory(
+                current_prompt,
+                callback=self._handle_prompt_accepted,
+            )
+
+    def _prompt_widget_factory(self, prompt, callback):
+        if isinstance(prompt, prompts.RadioListPrompt):
+            return widgets.RadioList(
+                question=prompt.parameters['question'],
+                options=prompt.parameters['options'],
+                on_accepted=callback,
+            )
+        elif isinstance(prompt, prompts.TextPrompt):
+            input_field = widgets.InputField(
+                on_accepted=lambda buffer: callback(buffer.text),
+            )
+            # The question or prompt message is optional, but HLabel always puts
+            # space between the question and the input field. This looks like
+            # the input field is weirdly indented if the question is empty.
+            if prompt.parameters['question']:
+                return widgets.HLabel(
+                    text=prompt.parameters['question'],
+                    content=input_field,
+                )
+            else:
+                return input_field
+        else:
+            raise ValueError(f'Unsupported prompt: {type(prompt).__name__!r}: {prompt!r}')
+
+    def _clear_cached_prompt_widget(self):
+        try:
+            del self.current_prompt_widget
+        except AttributeError:
+            pass
+        self.invalidate()
+
+    def _handle_prompt_accepted(self, result):
+        current_prompt = self._prompts[0]
+        self._prompts.remove(current_prompt)
+        self._clear_cached_prompt_widget()
+        current_prompt.set_result(result)
+
+    @functools.cached_property
     def is_interactive(self):
-        """Whether this job needs user interaction"""
-        if self.runtime_widget:
-            for c in walk(to_container(self.runtime_widget), skip_hidden=True):
-                if isinstance(c, Window) and c.content.is_focusable():
-                    return True
+        """
+        Whether this job may require user interaction at any point
+
+        Subclasses should hardcode this as a class attribute to `True` or
+        `False`. The default implementation tries to guess interactivity by
+        finding a focusable widget. This is expensive and can be inaccurate
+        because widgets come and go.
+        """
+        for c in walk(to_container(self._prompt_or_runtime_widget), skip_hidden=True):
+            if isinstance(c, Window) and c.content.is_focusable():
+                return True
         return False
 
     @functools.cached_property
@@ -200,6 +274,11 @@ class JobWidgetBase(abc.ABC):
 
     def invalidate(self):
         """Schedule redrawing of the TUI"""
+        try:
+            del self.is_interactive
+        except AttributeError:
+            pass
+
         self._app.invalidate()
 
     def __pt_container__(self):
