@@ -13,6 +13,9 @@ _log = logging.getLogger(__name__)
 
 
 class SceneSearchJobWidget(JobWidgetBase):
+
+    is_interactive = False
+
     def setup(self):
         pass
 
@@ -22,13 +25,13 @@ class SceneSearchJobWidget(JobWidgetBase):
 
 
 class SceneCheckJobWidget(JobWidgetBase):
+
     is_interactive = True
 
     def setup(self):
         self._question = FormattedTextControl('')
         self._radiolist = widgets.RadioList()
         self._activity_indicator = widgets.ActivityIndicator(
-            active=True,
             format='\n'.join((
                 '{indicator} Checking for scene release',
                 'Press <Ctrl-s n> to mark this as a non-scene release.',
@@ -36,6 +39,7 @@ class SceneCheckJobWidget(JobWidgetBase):
             style='class:info',
             extend_width=True,
         )
+        self.job.signal.register('running', lambda _: self._activity_indicator.enable())
         self.job.signal.register('finished', lambda _: self._activity_indicator.disable())
         self.job.signal.register('ask_release_name', self._ask_release_name)
         self.job.signal.register('ask_is_scene_release', self._ask_is_scene_release)
@@ -45,7 +49,8 @@ class SceneCheckJobWidget(JobWidgetBase):
             filter=Condition(lambda: self._question.text == ''),
         )
         def _(event):
-            self.job.finalize(SceneCheckResult.false)
+            self.job.stop_checking()
+            self.job.set_result(SceneCheckResult.false)
 
     def _ask_release_name(self, release_names):
         _log.debug('Asking for release name: %r', release_names)
@@ -55,7 +60,7 @@ class SceneCheckJobWidget(JobWidgetBase):
         def handle_release_name(choice):
             _log.debug('Got release name: %r', choice)
             # Remove current question
-            self._radiolist.choices = ()
+            self._radiolist.options = ()
             self.invalidate()
 
             # Verify user choice
@@ -63,10 +68,15 @@ class SceneCheckJobWidget(JobWidgetBase):
             self._activity_indicator.active = True
 
         self._radiolist.on_accepted = handle_release_name
-        self._radiolist.choices = [(release_name, release_name) for release_name in release_names]
-        self._radiolist.choices.append(('This is not a scene release', None))
+        self._radiolist.options = [(release_name, release_name) for release_name in release_names]
+        self._radiolist.options.append(('This is not a scene release', None))
 
         self._question.text = 'Pick the correct release name:'
+
+        # We must tell the TUI to invalidate the job list because we are now
+        # focusable, and the keypress focus may still be unset (keypresses are
+        # ignored).
+        self.job.signal.emit('refresh_job_list')
         self.invalidate()
 
     def _ask_is_scene_release(self, is_scene_release):
@@ -88,14 +98,19 @@ class SceneCheckJobWidget(JobWidgetBase):
 
         def handle_is_scene_release(choice):
             _log.debug('Got decision from user: %r', choice)
-            self.job.finalize(choice[1])
+            self.job.set_result(choice[1])
             self.invalidate()
 
-        self._radiolist.choices = (make_choice(True), make_choice(False))
-        self._radiolist.focused_choice = make_choice(is_scene_release)
+        self._radiolist.options = (make_choice(True), make_choice(False))
+        self._radiolist.focused_option = make_choice(is_scene_release)
         self._radiolist.on_accepted = handle_is_scene_release
 
         self._question.text = 'Is this a scene release?'
+
+        # We must tell the TUI to invalidate the job list because we are now
+        # focusable, and the keypress focus may still be unset (keypresses are
+        # ignored).
+        self.job.signal.emit('refresh_job_list')
         self.invalidate()
 
     @functools.cached_property
@@ -107,7 +122,7 @@ class SceneCheckJobWidget(JobWidgetBase):
                     content=self._activity_indicator,
                 ),
                 ConditionalContainer(
-                    filter=Condition(lambda: bool(self._radiolist.choices)),
+                    filter=Condition(lambda: bool(self._radiolist.options)),
                     content=HSplit(
                         children=[
                             Window(self._question),
