@@ -14,6 +14,9 @@ _log = logging.getLogger(__name__)
 
 
 class CreateTorrentJobWidget(JobWidgetBase):
+
+    is_interactive = False
+
     def setup(self):
         self._progress = widgets.ProgressBar()
         self._file_tree = FormattedTextControl()
@@ -68,7 +71,7 @@ class CreateTorrentJobWidget(JobWidgetBase):
         self._runtime_widget = HSplit(
             children=[
                 ConditionalContainer(
-                    filter=Condition(lambda: self._progress.percent > 0),
+                    filter=Condition(lambda: not self.job.output and self._progress.percent > 0),
                     content=HSplit([
                         # Estimated times, progress per second, etc
                         status,
@@ -96,15 +99,14 @@ class CreateTorrentJobWidget(JobWidgetBase):
                     widgets.hspacer,
                     # Current file name (torrent or content file)
                     ConditionalContainer(
-                        filter=Condition(lambda: self._current_file_name),
+                        filter=Condition(lambda: not self.job.output and self._current_file_name),
                         content=Window(
                             FormattedTextControl(lambda: self._current_file_name),
-                            style='class:info.readout',
                         ),
                     ),
                 ]),
                 ConditionalContainer(
-                    filter=Condition(lambda: bool(self._file_tree.text)),
+                    filter=Condition(lambda: not self.job.output and self._file_tree.text),
                     content=Window(self._file_tree),
                 ),
             ],
@@ -122,6 +124,7 @@ class CreateTorrentJobWidget(JobWidgetBase):
         self.job.signal.register('file_tree', self.handle_file_tree)
         self.job.signal.register('progress_update', self.handle_progress_update)
         self.job.signal.register('finished', lambda _: self.invalidate())
+        self.job.signal.register('output', lambda _: self.invalidate())
 
     def make_readout(self, label, attribute, value_align='left', width=None, label_side='left'):
         readout_widget = Window(
@@ -165,7 +168,7 @@ class CreateTorrentJobWidget(JobWidgetBase):
         self._seconds_total.text = timedelta_string(info.seconds_total)
 
         if isinstance(info, torrent.CreateTorrentProgress):
-            # Torrent is hashed from fresh files - yummy!
+            # Torrent is hashed from content files
             throughput = info.bytes_per_second.format(
                 prefix='binary',
                 decimal_places=2,
@@ -176,7 +179,7 @@ class CreateTorrentJobWidget(JobWidgetBase):
             self._current_file_name = fs.basename(info.filepath)
 
         elif isinstance(info, torrent.FindTorrentProgress):
-            # Torrent hashes are searched for in existing torrents
+            # Torrent is searched by hash in existing torrents
             if info.exception:
                 self.job.warn(info.exception)
             elif info.status == 'verifying':
@@ -200,6 +203,12 @@ class CreateTorrentJobWidget(JobWidgetBase):
             activity = 'Getting announce URL'
         elif self.job.activity == 'searching':
             activity = 'Press <Ctrl-t h> to skip searching for a reusable torrent'
+        elif self.job.output:
+            # If have output (i.e. the torrent file path), but we are not
+            # finished yet, the only reason this can happen is if
+            # get_announce_url() called login() and we have to wait for
+            # logout().
+            activity = 'Logging out'
         else:
             activity = self.job.activity.capitalize()
 
@@ -211,6 +220,9 @@ class CreateTorrentJobWidget(JobWidgetBase):
 
 
 class AddTorrentJobWidget(JobWidgetBase):
+
+    is_interactive = False
+
     def setup(self):
         pass
 
@@ -220,6 +232,9 @@ class AddTorrentJobWidget(JobWidgetBase):
 
 
 class CopyTorrentJobWidget(JobWidgetBase):
+
+    is_interactive = False
+
     def setup(self):
         pass
 
