@@ -4,9 +4,10 @@ Entry point
 
 import asyncio
 import sys
+import threading
 
-from ... import __homepage__, __project_name__, application_setup, application_shutdown, errors
-from ...utils import get_aioloop, update
+from ... import __changelog__, __homepage__, __project_name__, application_setup, application_shutdown, errors
+from ...utils import update
 from . import commands, utils
 
 
@@ -15,15 +16,15 @@ def main(args=None):
 
 
 def _main(args=None):
-    get_newer_version_task = None
     cmd = None
+    newer_version_thread = None
 
     try:
         if utils.is_tty():
             from .tui import TUI
             ui = TUI()
-            # Find latest version and generate update message in a background task
-            get_newer_version_task = get_aioloop().create_task(update.get_newer_version())
+            # Find latest version in a background thread
+            newer_version_thread = _UpgradeCheck()
         else:
             from .headless import Headless
             ui = Headless()
@@ -42,8 +43,7 @@ def _main(args=None):
         return 1
 
     except Exception as e:
-        # Unexpected exception; expected exceptions are handled by JobBase child
-        # classes by calling their error() or exception() methods.
+        # Unexpected exception
         import traceback
         traceback.print_exception(type(e), e, e.__traceback__)
         print()
@@ -67,20 +67,16 @@ def _main(args=None):
                         print('\n'.join(j.output))
                     break
 
-        # Print update message if there is a newer version available
-        if get_newer_version_task:
-            try:
-                # don't wait for get_newer_version_task to prevent annoying wait
-                # times before the user gets their prompt back.
-                newer_version = get_aioloop().run_until_complete(
-                    asyncio.wait_for(get_newer_version_task, timeout=0)
-                )
-            except (asyncio.TimeoutError, errors.RequestError):
-                pass
-            else:
-                if newer_version:
-                    msg = f'{__project_name__} {newer_version} has been released.'
-                    print(msg, file=sys.stderr)
+        # If we found a newer version, inform the user about it.
+        if newer_version_thread and newer_version_thread.result:
+            msg = (
+                '\n' + ('━' * 78) + '\n'
+                + '\n'
+                + f'  \\O/   {__project_name__} {newer_version_thread.result} has been released.\n'
+                + '   |\n'
+                + f'  / \\   Changes: {__changelog__}\n'
+            )
+            print(msg, file=sys.stderr)
 
         return exit_code
 
@@ -88,3 +84,21 @@ def _main(args=None):
         if cmd is not None:
             # Cleanup cache, close HTTP session, etc.
             application_shutdown(cmd.config)
+
+
+class _UpgradeCheck(threading.Thread):
+    def __init__(self):
+        self.result = None
+        # daemon=True allows the application to exit if this thread is still
+        # running. We don't want to wait for slow/unresponsive web servers.
+        super().__init__(daemon=True)
+        self.start()
+
+    def run(self):
+        try:
+            newer_version = asyncio.run(update.get_newer_version())
+        except errors.RequestError:
+            pass
+        else:
+            if newer_version:
+                self.result = newer_version
