@@ -2,6 +2,7 @@
 Custom TUI widgets
 """
 
+import asyncio
 import collections
 import itertools
 import textwrap
@@ -10,14 +11,12 @@ from prompt_toolkit.application import get_app
 from prompt_toolkit.buffer import Buffer
 from prompt_toolkit.document import Document
 from prompt_toolkit.filters import Condition
-from prompt_toolkit.formatted_text import to_formatted_text
+from prompt_toolkit.formatted_text import ANSI, to_formatted_text
 from prompt_toolkit.key_binding.key_bindings import KeyBindings
-from prompt_toolkit.layout.containers import HSplit, VSplit, Window, WindowAlign
+from prompt_toolkit.layout.containers import ConditionalContainer, HSplit, VSplit, Window, WindowAlign
 from prompt_toolkit.layout.controls import BufferControl, FormattedTextControl
 from prompt_toolkit.layout.dimension import Dimension
 
-from ...utils import get_aioloop
-
 import logging  # isort:skip
 _log = logging.getLogger(__name__)
 
@@ -97,7 +96,7 @@ class InputField:
 
     def __init__(self, text='', width=None, extend_width=True, read_only=False,
                  on_accepted=None, on_changed=None, style=''):
-        self._activity_indicator = ActivityIndicator(callback=self.set_text)
+        self._activity_indicator = ActivityIndicator()
         self.read_only = read_only
         self.on_accepted = on_accepted
         self.buffer = Buffer(
@@ -106,12 +105,23 @@ class InputField:
             on_text_changed=on_changed,
             read_only=Condition(lambda: self.read_only),
         )
-        self.container = Window(
-            content=BufferControl(self.buffer),
-            always_hide_cursor=Condition(lambda: self.read_only),
-            width=width,
-            dont_extend_height=True,
-            dont_extend_width=not extend_width,
+        self.container = VSplit(
+            children=[
+                ConditionalContainer(
+                    filter=Condition(lambda: self._activity_indicator.active),
+                    content=VSplit([
+                        self._activity_indicator,
+                        hspacer,
+                    ]),
+                ),
+                Window(
+                    content=BufferControl(self.buffer),
+                    always_hide_cursor=Condition(lambda: self.read_only),
+                    width=width,
+                    dont_extend_height=True,
+                    dont_extend_width=not extend_width,
+                ),
+            ],
             style=style,
         )
         if text:
@@ -170,28 +180,31 @@ class InputField:
 
 class RadioList:
     """
-    List of choices the user can select from
+    List of options the user can select from
 
-    :param choices: Sequence of choices the user can make
-    :param focused: Focused index or `None` to focus the first choice
-    :param on_accepted: Callback that gets the user-picked choice
+    :param options: Sequence of options the user can make
+    :param question: Text that is displayed above the `options` or any falsy
+        value
+    :param focused: Focused index or `None` to focus the first option
+    :param on_accepted: Callback that gets the user choice
 
-    Each choice item must be a :class:`str` or a sequence with one or more
-    items. The first item is presented to the user. `on_accept` gets the
-    complete choice item.
+    Each option item must be a :class:`str` or a sequence with one or more
+    items. The first item is presented to the user. `on_accepted` gets the
+    complete option item.
     """
 
-    def __init__(self, choices=(), focused=None, autodetected=None, on_accepted=None):
-        self.choices = choices
+    def __init__(self, options=(), question=None, focused=None, autodetected=None, on_accepted=None):
+        self.options = options
+        self.question = question
         self.on_accepted = on_accepted
 
         if focused:
-            self.focused_index = choices.index(focused)
+            self.focused_index = options.index(focused)
         else:
             self.focused_index = 0
 
         if autodetected:
-            self.autodetected_index = choices.index(autodetected)
+            self.autodetected_index = options.index(autodetected)
         else:
             self.autodetected_index = None
 
@@ -209,7 +222,7 @@ class RadioList:
         @kb.add('j')
         @kb.add('tab')
         def _(event):
-            self.focused_index = min(len(self.choices) - 1, self.focused_index + 1)
+            self.focused_index = min(len(self.options) - 1, self.focused_index + 1)
 
         @kb.add('enter')
         @kb.add('c-j')
@@ -217,50 +230,60 @@ class RadioList:
         def _(event):
             if self.on_accepted is not None:
                 try:
-                    self.on_accepted(self.choices[self.focused_index])
+                    self.on_accepted(self.options[self.focused_index])
                 except IndexError:
-                    raise RuntimeError(f'No choice at index {self.focused_index}: {self.choices}')
+                    raise RuntimeError(f'No option at index {self.focused_index}: {self.options}')
 
-        self.control = FormattedTextControl(
-            self._get_text_fragments,
-            key_bindings=kb,
-            focusable=True,
-        )
-        self.window = Window(
-            content=self.control,
-            style='class:dialog.choice',
+        choices = Window(
+            content=FormattedTextControl(
+                self._get_text_fragments,
+                key_bindings=kb,
+                focusable=True,
+            ),
             dont_extend_height=True,
             always_hide_cursor=True,
         )
 
-    @property
-    def focused_choice(self):
-        """Currently focused item in :attr:`choices`"""
-        return self.choices[self.focused_index]
+        self.container = HSplit(
+            children=[
+                choices,
+            ],
+            style='class:dialog.choice',
+        )
 
-    @focused_choice.setter
-    def focused_choice(self, choice):
-        if choice not in self.choices:
-            raise ValueError(f'No such choice: {choice!r}')
+        if self.question:
+            self.container.children.insert(0, Window(FormattedTextControl([
+                ('bold', self.question),
+            ])))
+
+    @property
+    def focused_option(self):
+        """Currently focused item in :attr:`options`"""
+        return self.options[self.focused_index]
+
+    @focused_option.setter
+    def focused_option(self, option):
+        if option not in self.options:
+            raise ValueError(f'No such option: {option!r}')
         else:
-            self.focused_index = self.choices.index(choice)
+            self.focused_index = self.options.index(option)
 
     def _get_text_fragments(self):
         fragments = []
-        if not self.choices:
+        if not self.options:
             return fragments
 
-        def choice_as_string(choice):
-            if isinstance(choice, str):
-                return choice
-            elif isinstance(choice, collections.abc.Sequence):
-                return str(choice[0])
+        def option_as_string(option):
+            if isinstance(option, str):
+                return option
+            elif isinstance(option, collections.abc.Sequence):
+                return str(option[0])
             else:
-                raise RuntimeError(f'Invalid choice value: {choice!r}')
+                raise RuntimeError(f'Invalid option value: {option!r}')
 
-        width = max(len(choice_as_string(choice)) for choice in self.choices)
-        for i, choice in enumerate(self.choices):
-            choice_string = choice_as_string(choice)
+        width = max(len(option_as_string(option)) for option in self.options)
+        for i, option in enumerate(self.options):
+            option_string = option_as_string(option)
             if i == self.focused_index:
                 style = 'class:dialog.choice.focused'
                 fragments.append(('[SetCursorPosition]', ''))
@@ -269,7 +292,7 @@ class RadioList:
                 style = 'class:dialog.choice'
                 fragments.append((style, '  '))
             fragments.append((style, ' '))
-            fragments.extend(to_formatted_text(choice_string.ljust(width) + ' ', style=style))
+            fragments.extend(to_formatted_text(option_string.ljust(width) + ' ', style=style))
             if i == self.autodetected_index:
                 fragments.append(('class:dialog.choice', '(autodetected)'))
             fragments.append(('', '\n'))
@@ -279,7 +302,7 @@ class RadioList:
         return fragments
 
     def __pt_container__(self):
-        return self.window
+        return self.container
 
 
 class HLabel:
@@ -370,21 +393,19 @@ class ActivityIndicator:
     :param states: Sequence of strings that are cycled through and passed to
         `callback` individually
     :param float interval: Delay between calls to `callback`
-    :param bool active: Whether `callback` is called right away
     :param str format: See :attr:`format`
     :param str style: prompt-toolkit style
     :param bool extend_width: Whether to maximize horizontally
     """
 
     def __init__(self, *, callback=None, states=('⠷', '⠯', '⠟', '⠻', '⠽', '⠾'),
-                 interval=0.3, active=False, format='{indicator}', style='',
-                 extend_width=False):
+                 interval=0.3, format='{indicator}', style='', extend_width=False):
         self._iterator = itertools.cycle(states)
         self._interval = float(interval)
         self._callback = callback or None
         self._format = format
-        self._text = self._format.format(indicator=states[0])
-        self.active = active
+        self._text = self._format.format(indicator=next(self._iterator))
+        self._iterate_task = None
 
         self.container = Window(
             content=FormattedTextControl(lambda: self.text),
@@ -395,17 +416,24 @@ class ActivityIndicator:
 
     @property
     def active(self):
-        """Whether `callback` is being called continuously"""
-        return getattr(self, '_active', False)
+        """
+        Whether :attr:`text` is continuously updated and `callback` called
+
+        .. note:: This updates :attr:`text` in an asynchronous task. If you
+            try to set this attribute outside of an asynchronous context, you
+            get ``RuntimeError: no running event loop``.
+        """
+        return bool(self._iterate_task)
 
     @active.setter
     def active(self, active):
         if active:
-            if not getattr(self, '_active', None):
-                self._active = True
-                self._iterate()
+            if not self._iterate_task:
+                self._iterate_task = asyncio.get_running_loop().create_task(self._iterate())
         else:
-            self._active = False
+            if self._iterate_task:
+                self._iterate_task.cancel()
+                self._iterate_task = None
 
     def enable(self):
         """Calling this method sets :attr:`active` to `True`"""
@@ -433,14 +461,85 @@ class ActivityIndicator:
         """Formatted :attr:`format` string"""
         return self._text
 
-    def _iterate(self):
-        if self.active:
+    async def _iterate(self):
+        while True:
             self._text = self._format.format(indicator=next(self._iterator))
+            get_app().invalidate()
+
             if self._callback:
                 self._callback(self._text)
 
-            get_aioloop().call_later(self._interval, self._iterate)
-            get_app().invalidate()
+            await asyncio.sleep(self._interval)
+
+    def __pt_container__(self):
+        return self.container
+
+
+class Image:
+    def __init__(self, bytes=b'', width=None, height=None):
+        self._activity_indicator = ActivityIndicator()
+        self._width = width
+        self._height = height
+        self.container = VSplit(
+            children=[
+                ConditionalContainer(
+                    filter=Condition(lambda: self.is_loading),
+                    content=self._activity_indicator,
+                ),
+                ConditionalContainer(
+                    filter=Condition(lambda: not self.is_loading),
+                    content=Window(
+                        content=FormattedTextControl(lambda: self._image),
+                        dont_extend_height=True,
+                        dont_extend_width=True,
+                    ),
+                ),
+            ],
+        )
+
+        self.is_loading = False
+        self.set_image_bytes(bytes)
+
+    def set_image_bytes(self, bytes):
+        if bytes:
+            import io
+
+            import PIL
+            import term_image.image
+            escape_codes = str(
+                # FIXME: This hack doesn't work with KittyImage
+                # objects. Probably because the ANSI class doesn't know the
+                # kitty graphics protocol and interprets the escape sequences as
+                # normal text. To fix this, we always use BlockImage, even on
+                # more capable terminals.
+                term_image.image.BlockImage(
+                    PIL.Image.open(io.BytesIO(bytes)),
+                    width=self._width,
+                    height=self._height,
+                )
+            )
+        else:
+            escape_codes = ''
+        self._image = ANSI(escape_codes)
+
+    # Add `text` property like InputField, TextField, etc. This is useful for
+    # WebDbSearchJobWidget, which handles a lot of widgets. Avoiding a special
+    # case for the poster widget makes things a lot simpler.
+    @property
+    def text(self):
+        return self._image
+
+    @text.setter
+    def text(self, bytes):
+        self.set_image_bytes(bytes)
+
+    @property
+    def is_loading(self):
+        return self._activity_indicator.active
+
+    @is_loading.setter
+    def is_loading(self, value):
+        self._activity_indicator.active = bool(value)
 
     def __pt_container__(self):
         return self.container
