@@ -13,18 +13,13 @@ import os
 import re
 import types as _types
 
+import logging  # isort:skip
+_log = logging.getLogger(__name__)
 
-def get_aioloop():
-    """Return :class:`asyncio.AbstractEventLoop` instance"""
-    # https://docs.python.org/3.10/library/asyncio-eventloop.html
-    try:
-        return asyncio.get_running_loop()
-    # "no running event loop"
-    except RuntimeError:
-        # We need a loop before the application has started.
-        # This mimics get_event_loop(), but that is going to be an alias for
-        # get_running_loop() in future Python releases.
-        return asyncio.get_event_loop_policy().get_event_loop()
+
+def is_running_in_development_environment():
+    """Whether we are running in a development environment or in production"""
+    return bool(os.environ.get('VIRTUAL_ENV', None))
 
 
 def os_family():
@@ -305,9 +300,9 @@ def as_groups(sequence, group_sizes, default=None):
     Iterate over items from `sequence` in equally sized groups
 
     :params sequence: List of items to group
-    :params group_sizes: Sequence of acceptable items in a group
+    :params group_sizes: Sequence of acceptable number of items in a group
 
-        Determine the group size with the lowest number of `default` items in
+        Find the group size with the lowest number of `default` items in
         the last group. That group size is then used for all groups.
     :param default: Value to pad last group with if
         ``len(sequence) % group_size != 0``
@@ -315,11 +310,11 @@ def as_groups(sequence, group_sizes, default=None):
     Example:
 
     >>> sequence = range(1, 10)
-    >>> for group in as_groups(sequence, [4, 5], default='_'):
+    >>> for group in as_groups(sequence, [4, 5], default="_"):
     ...     print(group)
     (1, 2, 3, 4, 5)
     (6, 7, 8, 9, '_')
-    >>> for group in as_groups(sequence, [3, 4], default='_'):
+    >>> for group in as_groups(sequence, [3, 4], default="_"):
     ...     print(group)
     (1, 2, 3)
     (4, 5, 6)
@@ -341,15 +336,6 @@ def as_groups(sequence, group_sizes, default=None):
     yield from itertools.zip_longest(*args, fillvalue=default)
 
 
-def is_regex_pattern(object):
-    """
-    Whether `object` is a regular expression object
-
-    A regular expression object is usually obtained from :func:`re.compile`.
-    """
-    return isinstance(object, re.Pattern)
-
-
 _unsupported_semantic_hash_types = (
     collections.abc.Iterator,
     collections.abc.Iterable,
@@ -394,42 +380,25 @@ def run_task(coro, callback):
     The returned task must be collected (e.g. in a :class:`list`) and awaited or
     cancelled eventually.
 
-    If the returned task is cancelled, the :class:`asyncio.CancelledError` is
-    silently ignored and `callback` is called with `None`.
-
     :param coro: Any awaitable object
-    :param callback: Callable that is called with exception or return value of
-        `coro`
+    :param callback: Callable that is called with the returned task when
+        `coro` returns, is cancelled or raises any other exception
 
     :return: :class:`asyncio.Task` instance
     """
-
-    @functools.wraps(coro)
-    async def wrap_coro(coro):
-        try:
-            return await asyncio.ensure_future(coro)
-        except asyncio.CancelledError:
-            pass
-        except Exception as e:
-            raise e
+    if not callable(callback):
+        raise ValueError(f'Not callable: {callable!r}')
 
     def handle_task_done(task):
+        # Call callback no matter what
         try:
-            result = task.result()
-        except asyncio.CancelledError:
-            # I don't know how to reliably reproduce or unittest this, but
-            # CancelledError was raised here.
-            pass
-        except Exception as e:
-            if callback:
-                callback(e)
-            else:
-                raise
+            task.result()
+        except BaseException:
+            callback(task)
         else:
-            if callback:
-                callback(result)
+            callback(task)
 
-    task = asyncio.ensure_future(wrap_coro(coro))
+    task = asyncio.create_task(coro)
     task.add_done_callback(handle_task_done)
     return task
 
@@ -440,7 +409,7 @@ async def run_async(function, *args, **kwargs):
 
     See :meth:`asyncio.BaseEventLoop.run_in_executor`.
     """
-    loop = get_aioloop()
+    loop = asyncio.get_running_loop()
     wrapped = functools.partial(function, *args, **kwargs)
     return await loop.run_in_executor(None, wrapped)
 
