@@ -73,6 +73,16 @@ def imghost(value):
         raise argparse.ArgumentTypeError(f'Unsupported image hosting service: {value}')
 
 
+def imghosts(value):
+    """Comma-separated list of names of image hosting services from :mod:`~.utils.imghosts`"""
+    names = []
+    for name in value.split(','):
+        name = name.strip()
+        if name:
+            names.append(imghost(name))
+    return names
+
+
 def integer(value):
     """Natural number (:class:`float` is rounded)"""
     try:
@@ -81,6 +91,17 @@ def integer(value):
         raise argparse.ArgumentTypeError(f'Not an integer: {value!r}')
 
 
+def bool_or_none(value):
+    """Convert `value` to :class:`~.types.Bool` or `None` if `value` is `None`"""
+    if value is None:
+        return None
+    else:
+        try:
+            return types.Bool(value)
+        except ValueError as e:
+            raise argparse.ArgumentTypeError(e)
+
+
 @functools.lru_cache(maxsize=None)
 def number_of_screenshots(*, min, max):
     """
@@ -131,12 +152,10 @@ def one_of(values):
 
 def regex(value):
     """:class:`re.Pattern` object"""
-    import re
     try:
-        return re.compile(str(value))
-    except re.error as e:
-        msg = f'Invalid regular expression: {value}: {e}'
-        raise argparse.ArgumentTypeError(msg)
+        return types.RegEx(value)
+    except ValueError as e:
+        raise argparse.ArgumentTypeError(e)
 
 
 def release(value):
@@ -173,22 +192,6 @@ def predb(value):
         raise argparse.ArgumentTypeError(e)
 
 
-def is_scene(value):
-    """
-    User-predetermined scene check result
-
-    Convert `value` to :class:`~.types.Bool` or return `None` if `value` is
-    `None` (autodetect).
-    """
-    if value is None:
-        return None
-    else:
-        try:
-            return types.Bool(value)
-        except ValueError as e:
-            raise argparse.ArgumentTypeError(e)
-
-
 def timestamp(value):
     """See :func:`.timestamp.parse`"""
     from . import timestamp
