@@ -120,6 +120,16 @@ class ConfigFiles(collections.abc.MutableMapping):
 
         return types
 
+    @property
+    def files(self):
+        """
+        Map section name to file paths
+
+        This is essentially the same as the keyword arguments from
+        instantiation.
+        """
+        return self._files.copy()
+
     @functools.cached_property
     def paths(self):
         """Sorted tuple of ``section.subsection.option`` paths"""
@@ -270,7 +280,7 @@ class ConfigFiles(collections.abc.MutableMapping):
                     self._cfg[section][subsection][option] = \
                         self._defaults[section][subsection][option]
 
-    def write(self, *sections):
+    def write(self, *sections, include_defaults=False):
         """
         Save current configuration to file(s)
 
@@ -279,6 +289,7 @@ class ConfigFiles(collections.abc.MutableMapping):
 
         :param sections: Paths to sections, subsections or options to save. Save
             all sections with no arguments.
+        :param bool include_defaults: Whether to include commented defaults
 
         :raise ConfigError: if writing fails
         """
@@ -292,26 +303,35 @@ class ConfigFiles(collections.abc.MutableMapping):
             os.makedirs(parentdir, mode=0o700, exist_ok=True)
             try:
                 with open(self._files[section], 'w') as f:
-                    f.write(self._as_ini(section))
+                    f.write(self._as_ini(section, include_defaults=include_defaults))
             except OSError as e:
                 raise errors.ConfigError(f'{self._files[section]}: {e.strerror or e}')
 
-    def _as_ini(self, section):
+    def _as_ini(self, section, include_defaults=False):
         lines = []
-        for subsection, options in self._cfg.get(section, {}).items():
+        for subsection, options in sorted(self._cfg.get(section, {}).items()):
             lines_subsection = []
-            for option, value in options.items():
-                if value == self._defaults[section][subsection][option]:
-                    continue
-
+            for option, value in sorted(options.items()):
                 if utils.is_sequence(value):
                     if value:
                         values = '\n  '.join(str(v) for v in value)
-                        lines_subsection.append(f'{option} =\n  {values}')
+                        line = f'{option} =\n  {values}'
                     else:
-                        lines_subsection.append(f'{option} =')
+                        line = f'{option} ='
                 else:
-                    lines_subsection.append(f'{option} = {value}')
+                    line = f'{option} = {value}'
+
+                if value == self._defaults[section][subsection][option]:
+                    # `value` is the default value
+                    if include_defaults:
+                        # Commented default value
+                        lines_subsection.append('# ' + line.replace('\n', '\n# '))
+                    else:
+                        # Exclude default value
+                        continue
+                else:
+                    # Non-default value
+                    lines_subsection.append(line)
 
             if lines_subsection:
                 lines.append(f'[{subsection}]')
