@@ -114,7 +114,12 @@ def limit_directory_size(path, max_total_size, min_age=None, max_age=None):
 
         # Actually remove the files
         for file_to_remove in files_to_remove:
-            os.unlink(file_to_remove)
+            # It's possible another process already unlinked this file while we
+            # were still collecting old files.
+            try:
+                os.unlink(file_to_remove)
+            except FileNotFoundError:
+                pass
 
 
 def prune_empty(path, files=False, directories=True):
@@ -132,6 +137,19 @@ def prune_empty(path, files=False, directories=True):
     if not os.path.isdir(path):
         return
 
+    # If multiple instances are running at the same time, one might be pruning
+    # empty directories just after another instance has created their (initially
+    # empty) project directory. To mitigate this, we only prune old directories
+    # (e.g. > 60s).
+    def is_older_than(min_age, path, now=time.time()):
+        try:
+            statinfo = os.stat(path, follow_symlinks=False)
+        except OSError:
+            return False
+        else:
+            age = round(now - statinfo.st_mtime)
+            return age > 60
+
     def raise_error(e, path):
         raise RuntimeError(f'{path}: Failed to prune: {e.strerror or e}')
 
@@ -152,7 +170,11 @@ def prune_empty(path, files=False, directories=True):
             for dirname in dirnames:
                 subdirpath = os.path.join(dirpath, dirname)
                 try:
-                    if not os.path.islink(subdirpath) and not os.listdir(subdirpath):
+                    if (
+                            not os.path.islink(subdirpath)
+                            and not bool(os.listdir(subdirpath))
+                            and is_older_than(60, subdirpath)
+                    ):
                         os.rmdir(subdirpath)
                 except OSError as e:
                     raise_error(e, subdirpath)
@@ -269,7 +291,8 @@ def sanitize_path(path):
     On Windows, the drive (e.g. "C:") is not sanitized to keep the ":".
     """
     # Get drive from Windows paths (e.g. "C:") because we don't want to sanitize
-    # the ":", which is illegal otherwise
+    # the ":", which is illegal otherwise. If there are no drives (e.g. on
+    # Linux), `drive` is empty, so we don't have to handle that case specially.
     drive, path = os.path.splitdrive(str(path))
     segments = str(path).split(os.sep)
     sanitized_segments = [sanitize_filename(segment) for segment in segments]
