@@ -77,13 +77,27 @@ def as_text(html):
     return text
 
 
-def purge_javascript(html):
+def purge_tags(html):
     """
-    Return `html` with <script> tags removed
+    Return `html` with <script> and <style> tags removed
 
     :param str html: HTML string
     """
+    def is_javascript(tag):
+        # Match <script> tags that don't have type="application/ld+json".
+        if tag.name == 'script':
+            if tag.get('type') == 'application/ld+json':
+                return False
+            else:
+                return True
+
+        # Match <style> (CSS) tags.
+        elif tag.name == 'style':
+            return True
+
+        return False
+
     soup = parse(html)
-    for script_tag in soup.find_all('script'):
+    for script_tag in soup.find_all(is_javascript):
         script_tag.decompose()
     return str(soup)
