@@ -203,8 +203,7 @@ async def download(url, filepath, *args, cache=False, **kwargs):
     :param url: Where to download the data from
     :param filepath: Where to save the downloaded data
 
-    Any other arguments are passed to :func:`get`, except for `cache`, which is
-    always `False`.
+    Any other arguments are passed to :func:`get`.
 
     If `filepath` exists, no request is made.
 
@@ -263,7 +262,10 @@ class Response(str):
         try:
             return json.loads(self)
         except ValueError as e:
-            raise errors.RequestError(f'Malformed JSON: {str(self)}: {e}')
+            if self:
+                raise errors.RequestError(f'Malformed JSON: {str(self)!r}: {e}')
+            else:
+                raise errors.RequestError('Malformed JSON: Empty string')
 
     def __repr__(self):
         kwargs = [
@@ -363,7 +365,10 @@ async def _request(
                         )
 
             except httpx.TimeoutException:
-                raise errors.RequestError(f'{url}: Timeout')
+                raise errors.RequestError(
+                    f'{url}: Timeout after {timeout} second'
+                    + ('s' if timeout != 1 else '')
+                )
 
             except ssl.SSLError as e:
                 if hasattr(e, 'verify_message'):
@@ -382,7 +387,8 @@ async def _request(
 
             except httpx.HTTPError as e:
                 _log.debug(f'Unexpected HTTP error: {e!r}')
-                raise errors.RequestError(f'{url}: {e}')
+                msg = re.sub(r'^\[Errno -?\d+\]\s*', r'', str(e))
+                raise errors.RequestError(f'{url}: {msg}')
 
             else:
                 _save_session_cookies(cookies=client.cookies, domain=response.url.host)
@@ -572,7 +578,9 @@ def _to_cache(cache_file, bytes):
         except UnicodeDecodeError:
             pass
         else:
-            string = utils.html.purge_javascript(string)
+            # This is disabled for now because ImdbApi uses a JSON object that
+            # is are stored inside a <script></script> tag.
+            string = utils.html.purge_tags(string)
             bytes = string.encode('utf-8')
 
     try:
