@@ -83,7 +83,7 @@ def _make_screenshot_cmd(video_file, timestamp, screenshot_file):
     )
 
 
-def screenshot(video_file, timestamp, screenshot_file, overwrite=False):
+def screenshot(video_file, timestamp, screenshot_file):
     """
     Create single screenshot from video file
 
@@ -91,7 +91,6 @@ def screenshot(video_file, timestamp, screenshot_file, overwrite=False):
     :param timestamp: Time location in the video
     :type timestamp: int or float or "[[H+:]MM:]SS"
     :param str screenshot_file: Path to screenshot file
-    :param bool overwrite: Whether to overwrite `screenshot_file` if it exists
 
     .. note:: It is important to use the returned file path because it is passed
               through :func:`~.fs.sanitize_path` to make sure it can exist.
@@ -116,11 +115,6 @@ def screenshot(video_file, timestamp, screenshot_file, overwrite=False):
     # Make `screenshot_file` compatible to the file system
     screenshot_file = utils.fs.sanitize_path(screenshot_file)
 
-    # Check for previously created screenshot
-    if not overwrite and os.path.exists(screenshot_file):
-        _log.debug('Screenshot already exists: %s', screenshot_file)
-        return screenshot_file
-
     # Ensure timestamp is within range
     try:
         duration = utils.video.duration(video_file)
@@ -157,7 +151,8 @@ def _make_resize_cmd(image_file, dimensions, resized_file):
         f'file:{resized_file}',
     )
 
-def resize(image_file, width=0, height=0, target_directory=None, target_filename=None):
+
+def resize(image_file, width=0, height=0, target_directory=None, target_filename=None, overwrite=False):
     """
     Resize image, preserve aspect ratio
 
@@ -168,6 +163,8 @@ def resize(image_file, width=0, height=0, target_directory=None, target_filename
         parent directory of `image_file`
     :param target_filename: File name of resized image or `None` to generate a
         name from `image_file`, `width` and `height`
+    :param bool overwrite: Whether to overwrite the resized image file if it
+        already exists
 
     If `width` and `height` are falsy (the default) return `image_file` if
     `target_directory` and `target_filename` are falsy or copy `image_file` to
@@ -224,6 +221,10 @@ def resize(image_file, width=0, height=0, target_directory=None, target_filename
         os.path.join(get_target_directory(), get_target_filename()),
     )
 
+    if not overwrite and os.path.exists(target_filepath):
+        _log.debug('Already resized: %r', target_filepath)
+        return target_filepath
+
     if not width and not height:
         # Nothing to resize
         if target_filepath != str(image_file):
@@ -271,17 +272,20 @@ optimization_levels = tuple(_optimization_levels) + ('none', 'default')
 """Valid `level` arguments for :func:`optimize`"""
 
 
-def optimize(image_file, level=4):
+def optimize(image_file, output_file=None, level=None):
     """
     Optimize PNG image size
 
     `image_file` is overwritten with the smaller image data.
 
     :path image_file: Path to PNG File
-
+    :path output_file: Path to optimized `image_file` or any falsy value to
+        overwrite `image_file`
     :path str,int level: Optimiziation level (``"low"``, ``"medium"``,
         ``"high"``) or ``"default"`` to use recommended level or ``"none"`` /
         `None` to not do any optimization
+
+    :return: path to new optimized PNG file
     """
     if level not in ('none', None):
         if level == 'default':
@@ -299,6 +303,16 @@ def optimize(image_file, level=4):
             '--strip', 'safe',   # Remove irrelevant metadata
             str(image_file),
         ]
+
+        if output_file:
+            sanitized_output_file = utils.fs.sanitize_path(output_file)
+            cmd.extend(('--out', sanitized_output_file))
+            return_value = sanitized_output_file
+        else:
+            return_value = image_file
+
         error = utils.subproc.run(cmd, join_stderr=True)
         if error:
             raise errors.ImageOptimizeError(f'Failed to optimize: {error}')
+        else:
+            return return_value
