@@ -3,8 +3,9 @@ Base class for image uploaders
 """
 
 import abc
-import collections
 import copy
+import glob
+import hashlib
 import os
 
 from ... import __project_name__, constants, errors
@@ -83,7 +84,7 @@ class ImageHostBase(abc.ABC):
     """
     Default user configuration for a subclass
 
-    This always contains :attr:`default_config_common`.
+    Thanks to some magic, this is always an extension of :attr:`default_config_common`.
     """
 
     argument_definitions = {}
@@ -92,11 +93,14 @@ class ImageHostBase(abc.ABC):
     description = ''
     """Any documentation, for example how to get an API key"""
 
-    async def upload(self, image_path, cache=True):
+    async def upload(self, image_path, thumb_width=None, cache=True):
         """
         Upload image file
 
         :param image_path: Path to image file
+        :param int thumb_width: Override ``thumb_width`` in :attr:`options`
+
+            If set to 0, no thumbnail is uploaded.
         :param bool cache: Whether to attempt to get the image URL from cache
 
         :raise RequestError: if the upload fails
@@ -114,13 +118,15 @@ class ImageHostBase(abc.ABC):
             'url': await self._get_image_url(image_path, cache=cache),
         }
 
-        thumb_width = self.options['thumb_width']
+        if thumb_width is None:
+            thumb_width = self.options['thumb_width']
         if thumb_width:
             try:
                 thumbnail_path = image.resize(
                     image_path,
                     width=thumb_width,
                     target_directory=self.cache_directory,
+                    overwrite=not cache,
                 )
             except errors.ImageResizeError as e:
                 raise errors.RequestError(e)
@@ -132,13 +138,9 @@ class ImageHostBase(abc.ABC):
     async def _get_image_url(self, image_path, cache=True):
         url = self._get_url_from_cache(image_path) if cache else None
         if not url:
-            try:
-                url = await self._upload_image(image_path)
-            except errors.RequestError as e:
-                raise errors.RequestError(e)
-            else:
-                _log.debug('Uploaded %r: %r', image_path, url)
-                self._store_url_to_cache(image_path, url)
+            url = await self._upload_image(image_path)
+            _log.debug('Uploaded %r: %r', image_path, url)
+            self._store_url_to_cache(image_path, url)
         else:
             _log.debug('Got URL from cache: %r: %r', image_path, url)
         return url
@@ -148,70 +150,48 @@ class ImageHostBase(abc.ABC):
         """Upload `image_path` and return URL to the image file"""
 
     def _get_url_from_cache(self, image_path):
-        cache_file = self._cache_file(image_path)
-        if os.path.exists(cache_file):
+        cache_file_suffix = self._get_cache_file_suffix(image_path)
+        cache_file_glob = os.path.join(
+            glob.escape(self.cache_directory),
+            f'*.{cache_file_suffix}',
+        )
+        matching_cache_files = glob.glob(cache_file_glob)
+
+        if matching_cache_files:
+            cache_file = matching_cache_files[0]
             _log.debug('Already uploaded: %s', cache_file)
             try:
                 with open(cache_file, 'r') as f:
                     return f.read().strip()
             except OSError:
-                # We'll try to overwrite the bad cache file later
+                # Unreadable cache file. we'll try to overwrite it later.
                 pass
 
     def _store_url_to_cache(self, image_path, url):
-        cache_file = self._cache_file(image_path)
+        # Use file name for easier debugging.
+        stem = fs.basename(image_path)
+        extension = self._get_cache_file_suffix(image_path)
+
+        # Max file name length is usually 255 bytes
+        max_filename_length = 250 - len(extension)
+        filename = f'{stem}.{extension}'
+        cache_file = os.path.join(self.cache_directory, filename[-max_filename_length:])
+
         try:
             fs.mkdir(fs.dirname(cache_file))
             with open(cache_file, 'w') as f:
                 f.write(url)
-        except OSError as e:
+        except (OSError, errors.ContentError) as e:
             msg = e.strerror if getattr(e, 'strerror', None) else e
             raise RuntimeError(f'Unable to write cache {cache_file}: {msg}')
 
-    def _cache_file(self, image_path):
-        # If image_path is in our cache_directory, the image's file name makes
-        # it unique. This is usually the case when we're uploading
-        # screenshots. If image is not in our cache_directory, use the absolute
-        # path as a unique identifier.
-        if fs.dirname(image_path) == self.cache_directory:
-            image_path = fs.basename(image_path)
-        else:
-            image_path = os.path.abspath(image_path)
-
-        # Make cache file even more unique, e.g. imgbox only has one thumbnail
-        # size per uploaded image, so if we want a different thumbnail size, we
-        # need to upload again.
-        cache_id = self._get_cache_id_as_string()
-        if cache_id:
-            image_path += f'.{cache_id}'
-
-        # Max file name length is usually 255 bytes
-        filename = fs.sanitize_filename(image_path[-200:]) + f'.{self.name}.url'
-
-        return os.path.join(self.cache_directory, filename)
-
-    def _get_cache_id_as_string(self):
-        def as_str(obj):
-            if isinstance(obj, collections.abc.Mapping):
-                return ','.join(f'{as_str(k)}={as_str(v)}' for k,v in obj.items())
-            elif isinstance(obj, str):
-                return str(obj)
-            elif isinstance(obj, collections.abc.Sequence):
-                return ','.join(as_str(i) for i in obj)
-            else:
-                return str(obj)
-
-        cache_id = self.cache_id
-        if cache_id is not None:
-            return as_str(cache_id)
-
-    @property
-    def cache_id(self):
-        """
-        Information that makes an upload unique, aside from the file path
-
-        If this returns `None`, the file path is unique enough. Otherwise, the
-        return value should be a string, dictionary, sequence or anything with a
-        readable and unique string representation.
-        """
-        return None
+    def _get_cache_file_suffix(self, image_path):
+        try:
+            # Generate unique ID from the first 10 KiB of image data.
+            with open(image_path, 'rb') as f:
+                unique_id = hashlib.md5(f.read(10 * 1024)).hexdigest()
+        except OSError:
+            # If `image_path` is not readable, get unique ID from the literal
+            # file path.
+            unique_id = hashlib.md5(image_path.encode('utf8')).hexdigest()
+        return unique_id + f'.{self.name}.url'
