@@ -30,10 +30,6 @@ class DummyImageHost(ImageHostBase):
         },
     }
 
-    @property
-    def cache_id(self):
-        return self.options['hostname']
-
     async def _upload_image(self, image_path):
         try:
             fs.assert_file_readable(image_path)
