@@ -32,7 +32,11 @@ class PtpimgImageHost(ImageHostBase):
         '  1. Create an account: https://ptpimg.me/register.php\n'
         f'  2. Store your API key in {fs.tildify_path(constants.IMGHOSTS_FILEPATH)}:\n'
         '\n'
-        f'       $ {__project_name__} set --fetch-ptpimg-apikey EMAIL PASSWORD'
+        f'       $ {__project_name__} set --fetch-ptpimg-apikey EMAIL PASSWORD\n'
+        '\n'
+        '     If you already have your API key, store it like this:\n'
+        '\n'
+        f'       $ {__project_name__} set imghosts.ptpimg.apikey YOUR_API_KEY'
     )
 
     async def _upload_image(self, image_path):
