@@ -28,11 +28,11 @@ class CorruptnetApi(base.PredbApiBase):
     # Seems reasonable?
     _max_pages = 10
 
-    async def _search(self, keywords, group):
+    async def _search(self, query):
         combined_results = []
         max_page_value = self._max_pages * self._results_per_page
         for page in range(0, max_page_value, self._results_per_page):
-            results = list(await self._request_page(keywords, group, page))
+            results = list(await self._request_page(query.keywords, query.group, page))
             combined_results.extend(results)
             if len(results) < self._results_per_page:
                 break
