@@ -31,9 +31,9 @@ class MultiPredbApi:
 
     DEFAULT_PREDB_NAMES = (
         'srrdb',
-        # 'corruptnet',
+        'corruptnet',
         # 'predbovh',
-        'predbclub',
+        # 'predbclub',
         'predbde',
     )
 
@@ -87,17 +87,34 @@ class MultiPredbApi:
         # Call `method_name` on all predbs concurrently and yield the return
         # value or RequestError/NotImplementedError as they occur.
         method_calls = [
-            self._call(predb, method_name, *args, **kwargs)
+            asyncio.create_task(
+                self._call(predb, method_name, *args, **kwargs)
+            )
             for predb in self._predbs
         ]
 
-        for coro in asyncio.as_completed(method_calls):
-            try:
-                result = await coro
-            except (NotImplementedError, errors.RequestError) as e:
-                yield e
-            else:
-                yield result
+        try:
+            for coro in asyncio.as_completed(method_calls):
+                try:
+                    result = await coro
+                except (NotImplementedError, errors.RequestError) as e:
+                    yield e
+                else:
+                    yield result
+        finally:
+            # If our caller stops iterating over our yielded `result`s, we may
+            # still have unfinished tasks that will continue to run in the
+            # background. We cancel them here because
+            #   1. nobody cares about their return values anymore
+            #
+            #   2. if one of the tasks raises an exception, asyncio will
+            #      eventually complain about an unhandled exception. Cancelling
+            #      the tasks seems to prevents that.
+            #
+            # NOTE: This is untested functionality because I couldn't reproduce
+            #       this issue in a test case.
+            for task in method_calls:
+                task.cancel()
 
     @async_lru.alru_cache
     async def search(self, *args, **kwargs):
