@@ -17,8 +17,8 @@ class PredbdeApi(base.PredbApiBase):
     _url_base = b64decode('cHJlZGIuZGU=').decode('ascii')
     _search_url = f'https://{_url_base}/api/'
 
-    async def _search(self, keywords, group):
-        q = self._get_q(keywords, group)
+    async def _search(self, query):
+        q = self._get_q(query.keywords, query.group)
         if q:
             return await self._request_all_pages(q)
         else:
@@ -62,23 +62,28 @@ class PredbdeApi(base.PredbApiBase):
         _log.debug('%s search: %r, %r', self.label, self._search_url, params)
         response = (await http.get(self._search_url, params=params, cache=True)).json()
 
-        # Report API error or return list of release names
-        if response['status'] != 'success':
+        if response.get('data'):
+            # We found at least one release
+            results = tuple(result['release'] for result in response['data'])
+
+        elif (
+                response.get('status') != 'success'
+                and response.get('message')
+                and response['message'].lower() != 'no results'
+        ):
+            # Report error from API
             raise errors.RequestError(f'{self.label}: {response["message"]}')
+
         else:
-            # Extract release names
-            if response['data']:
-                results = tuple(result['release'] for result in response['data'])
-            else:
-                results = ()
+            results = ()
 
-            # Is there another page of results?
-            if len(results) >= self._max_results_per_page:
-                next_page = page + 1
-            else:
-                next_page = -1
+        # Is there another page of results?
+        if len(results) >= self._max_results_per_page:
+            next_page = page + 1
+        else:
+            next_page = -1
 
-            return results, next_page
+        return results, next_page
 
     async def _release_files(self, release_name):
         raise NotImplementedError()
