@@ -17,8 +17,8 @@ class PredbovhApi(base.PredbApiBase):
     _url_base = b64decode('cHJlZGIub3Zo').decode('ascii')
     _search_url = f'https://{_url_base}/api/v1/'
 
-    async def _search(self, keywords, group):
-        q = self._get_q(keywords, group)
+    async def _search(self, query):
+        q = self._get_q(query.keywords, query.group)
         return await self._request_all_pages(q)
 
     def _get_q(self, keywords, group):
