@@ -12,6 +12,7 @@ string.
 
 import asyncio
 import collections
+import copy
 import functools
 import os
 import re
@@ -100,8 +101,8 @@ class ReleaseName(collections.abc.Mapping):
 
     :param str path: Path to release file or directory
 
-        If `path` exists, it is used to read video and audio metadata, e.g. to
-        detect the codecs, resolution, etc.
+        If :attr:`path` exists, it is used to read video and audio metadata,
+        e.g. to detect the codecs, resolution, etc.
 
     :param str name: Path or other string to pass to :class:`ReleaseInfo`
         (defaults to `path`)
@@ -146,6 +147,11 @@ class ReleaseName(collections.abc.Mapping):
         self._info = ReleaseInfo(str(name) if name is not None else self._path)
         self._translate = translate or {}
 
+    @property
+    def release_info(self):
+        """Internal :class:`~.ReleaseInfo` instance"""
+        return self._info
+
     def set_release_info(self, path):
         """
         Update internal :class:`ReleaseInfo` instance
@@ -188,6 +194,11 @@ class ReleaseName(collections.abc.Mapping):
         else:
             raise KeyError(name)
 
+    @property
+    def path(self):
+        """:class:`str` version of the `path` instantiation argument"""
+        return self._path
+
     @property
     def separator(self):
         """
@@ -451,7 +462,7 @@ class ReleaseName(collections.abc.Mapping):
         """
         List of "Director's Cut", "Uncut", "Unrated", etc
 
-        :raise ContentError: if path exists but contains unexpected data
+        :raise ContentError: if :attr:`path` exists but contains unexpected data
         """
         if 'edition' not in self._info:
             self._info['edition'] = []
@@ -462,13 +473,6 @@ class ReleaseName(collections.abc.Mapping):
         if self.has_dual_audio:
             self._info['edition'].append('Dual Audio')
 
-        # HDR format (e.g. "Dolby Vision" or "HDR10")
-        for hdr_format in video.hdr_formats:
-            while hdr_format in self._info['edition']:
-                self._info['edition'].remove(hdr_format)
-        if self.hdr_format:
-            self._info['edition'].append(self.hdr_format)
-
         return self._info['edition']
 
     @edition.setter
@@ -521,7 +525,7 @@ class ReleaseName(collections.abc.Mapping):
         """
         Resolution (e.g. "1080p") or "UNKNOWN_RESOLUTION"
 
-        :raise ContentError: if path exists but contains unexpected data
+        :raise ContentError: if :attr:`path` exists but contains unexpected data
         """
         res = video.resolution(self._path, default=None)
         if res is None:
@@ -537,7 +541,7 @@ class ReleaseName(collections.abc.Mapping):
         """
         Audio format (e.g. "FLAC") or empty string (no audio track)
 
-        :raise ContentError: if path exists but contains unexpected data
+        :raise ContentError: if :attr:`path` exists but contains unexpected data
         """
         af = video.audio_format(self._path, default=None)
         if af is None:
@@ -553,7 +557,7 @@ class ReleaseName(collections.abc.Mapping):
         """
         Audio channels (e.g. "5.1") or empty string (no audio track)
 
-        :raise ContentError: if path exists but contains unexpected data
+        :raise ContentError: if :attr:`path` exists but contains unexpected data
         """
         ac = video.audio_channels(self._path, default=None)
         if ac is None:
@@ -564,12 +568,58 @@ class ReleaseName(collections.abc.Mapping):
     def audio_channels(self, value):
         self._info['audio_channels'] = str(value)
 
+    @_translated_property
+    def hdr_format(self):
+        """
+        HDR format name (e.g. "DV" or "HDR10") or empty string
+
+        If not set explicitly and :attr:`path` exists, this value is
+        autodetected if possible, otherwise default to whatever
+        :class:`ReleaseInfo` detected in the release name.
+
+        Setting this value back to `None` turns on autodetection as described
+        above.
+
+        :raise ContentError: if :attr:`path` exists but contains unexpected data
+        """
+        # Use manually set value unless it is None
+        if getattr(self, '_hdr_format', None) is not None:
+            return self._hdr_format
+
+        # Autodetect from existing files
+        if os.path.exists(self._path):
+            hdr_formats = video.hdr_formats(self._path, default=None)
+            if hdr_formats:
+                self._hdr_formats = ' '.join(hdr_formats)
+                return self._hdr_formats
+
+        # Autodetect from release name
+        guessed_hdr = self._info.get('hdr_format', '')
+        if guessed_hdr:
+            return guessed_hdr
+
+        # No HDR detected
+        return ''
+
+    @hdr_format.setter
+    def hdr_format(self, value):
+        if value is None:
+            # Turn on autodetection
+            self._hdr_format = None
+        elif not value:
+            # No HDR
+            self._hdr_format = ''
+        elif value in video.known_hdr_formats:
+            self._hdr_format = value
+        else:
+            raise ValueError(f'Unknown HDR format: {value!r}')
+
     @_translated_property
     def video_format(self):
         """
         Video format (or encoder in case of x264/x265/XviD) or "UNKNOWN_VIDEO_FORMAT"
 
-        :raise ContentError: if path exists but contains unexpected data
+        :raise ContentError: if :attr:`path` exists but contains unexpected data
         """
         vf = video.video_format(self._path, default=None)
         if vf is None:
@@ -594,17 +644,17 @@ class ReleaseName(collections.abc.Mapping):
         """
         Whether this release has a commentary audio track
 
-        If not set explicitly and the given `path` exists, this value is
+        If not set explicitly and `:attr:`path`` exists, this value is
         autodetected by looking for "commentary" case-insensitively in any audio
         track title.
 
-        If not set explicitly and the given `path` does not exists, default to
+        If not set explicitly and :attr:`path` does not exists, default to
         detection by :class:`ReleaseInfo`.
 
         Setting this value back to `None` turns on autodetection as described
         above.
 
-        :raise ContentError: if path exists but contains unexpected data
+        :raise ContentError: if :attr:`path` exists but contains unexpected data
         """
         # Use manually set value unless it is None
         if getattr(self, '_has_commentary', None) is not None:
@@ -631,14 +681,14 @@ class ReleaseName(collections.abc.Mapping):
         """
         Whether this release has an English and a non-English audio track
 
-        If not set explicitly and the given `path` exists, this value is
+        If not set explicitly and :attr:`path` exists, this value is
         autodetected if possible, otherwise default to whatever
         :class:`ReleaseInfo` detected in the release name.
 
         Setting this value back to `None` turns on autodetection as described
         above.
 
-        :raise ContentError: if path exists but contains unexpected data
+        :raise ContentError: if :attr:`path` exists but contains unexpected data
         """
         # Use manually set value unless it is None
         if getattr(self, '_has_dual_audio', None) is not None:
@@ -660,48 +710,6 @@ class ReleaseName(collections.abc.Mapping):
         else:
             self._has_dual_audio = bool(value)
 
-    @_translated_property
-    def hdr_format(self):
-        """
-        HDR format name (e.g. "Dolby Vision" or "HDR10") or `None`
-
-        If not set explicitly and the given `path` exists, this value is
-        autodetected if possible, otherwise default to whatever
-        :class:`ReleaseInfo` detected in the release name.
-
-        Setting this value back to `None` turns on autodetection as described
-        above.
-
-        :raise ContentError: if path exists but contains unexpected data
-        """
-        # Use manually set value unless it is None
-        if getattr(self, '_hdr_format', None) is not None:
-            return self._hdr_format
-
-        # Autodetect
-        if os.path.exists(self._path):
-            hdr_format = video.hdr_format(self._path, default=None)
-            if hdr_format is not None:
-                self._hdr_format = hdr_format
-                return self._hdr_format
-
-        # Default to ReleaseInfo['edition']
-        guessed_editions = self._info.get('edition', ())
-        for hdr_format in video.hdr_formats:
-            if hdr_format in guessed_editions:
-                return hdr_format
-
-    @hdr_format.setter
-    def hdr_format(self, value):
-        if value is None:
-            self._hdr_format = None
-        elif value == '':
-            self._hdr_format = ''
-        elif value in video.hdr_formats:
-            self._hdr_format = str(value)
-        else:
-            raise ValueError(f'Unknown HDR format: {value!r}')
-
     _needed_attrs = {
         ReleaseType.movie: ('title', 'year', 'resolution', 'source', 'video_format'),
         ReleaseType.season: ('title', 'episodes', 'resolution', 'source', 'video_format'),
@@ -866,6 +874,9 @@ class ReleaseName(collections.abc.Mapping):
         if self.type in (ReleaseType.season, ReleaseType.episode) and not self.date:
             parts.append(str(self.episodes))
 
+        if self.edition:
+            parts.append(' '.join(self.edition))
+
         if self.dvd_encoding:
             parts.append(self.dvd_encoding)
         else:
@@ -876,15 +887,15 @@ class ReleaseName(collections.abc.Mapping):
 
         parts.append(self.source)
 
-        if self.edition:
-            parts.append(' '.join(self.edition))
-
         if self.audio_format:
             parts.append(self.audio_format)
 
         if self.audio_channels:
             parts.append(self.audio_channels)
 
+        if self.hdr_format:
+            parts.append(self.hdr_format)
+
         parts.append(self.video_format)
 
         joined = ' '.join(parts) + f'-{self.group}'
@@ -897,9 +908,9 @@ class ReleaseName(collections.abc.Mapping):
         if sep == '.':
             # Replace "&" with "and" before "&" gets removed
             joined = re.sub(r'&', 'and', joined)
-            # Remove non-word characters like (e.g.: ,:) with some exceptions
+            # Remove most non-word characters (e.g. ",") with some exceptions
             joined = re.sub(r'[^ \w\.-]', '', joined)
-            # Deduplicate "."
+            # Deduplicate "." in case there is a "." at the end of the title
             joined = re.sub(r'\.+', '.', joined)
 
         return joined
@@ -928,6 +939,7 @@ class ReleaseInfo(collections.abc.MutableMapping):
       - ``type`` (:class:`~.types.ReleaseType` enum)
       - ``title``
       - ``aka`` (Also Known As; anything after "AKA" in the title)
+      - ``country``
       - ``year``
       - ``episodes`` (:class:`~.Episodes` instance)
       - ``episode_title``
@@ -938,6 +950,7 @@ class ReleaseInfo(collections.abc.MutableMapping):
       - ``source`` ("BluRay", "WEB-DL", etc)
       - ``audio_codec`` (Audio codec abbreviation)
       - ``audio_channels`` (e.g. "2.0" or "7.1")
+      - ``hdr_format``
       - ``video_codec``
       - ``group``
       - ``has_commentary`` (:class:`bool` or `None` to autodetect)
@@ -949,7 +962,7 @@ class ReleaseInfo(collections.abc.MutableMapping):
     def __init__(self, path, strict=False):
         if strict:
             try:
-                utils.predbs.assert_not_abbreviated_filename(path)
+                utils.predbs.assert_not_abbreviated_filename(str(path))
             except errors.SceneAbbreviatedFilenameError as e:
                 raise errors.ContentError(e)
         self._path = str(path)
@@ -993,6 +1006,12 @@ class ReleaseInfo(collections.abc.MutableMapping):
     def __repr__(self):
         return f'{type(self).__name__}({self._path!r})'
 
+    def copy(self):
+        """Return instance copy"""
+        cp = type(self)(self._path)
+        cp._dict = copy.deepcopy(self._dict)
+        return cp
+
     @property
     def path(self):
         """`path` argument as :class:`str`"""
@@ -1157,6 +1176,9 @@ class ReleaseInfo(collections.abc.MutableMapping):
     def _get_episode_title(self):
         return _as_string(self._guess.get('episode_title', ''))
 
+    def _get_date(self):
+        return _as_string(self._guess.get('date', ''))
+
     _edition_translation = {
         "Collector's Edition": re.compile(r'Collector'),
         'Criterion Collection': re.compile(r'Criterion'),
@@ -1167,17 +1189,8 @@ class ReleaseInfo(collections.abc.MutableMapping):
         'Ultimate Cut': re.compile(r'Ultimate'),
     }
     _proper_repack_regex = re.compile(rf'(?:{DELIM}|^)((?i:proper|repack\d*))(?:{DELIM}|$)')
-    _hdr_regexes = {
-        'DoVi': re.compile(rf'(?:{DELIM}|^)(?i:DV|DoVi|Dolby{DELIM}Vision)(?:{DELIM}|$)'),
-        'HDR10+': re.compile(rf'(?:{DELIM}|^)(?i:HDR10\+)(?:{DELIM}|$)'),
-        'HDR10': re.compile(rf'(?:{DELIM}|^)(?i:HDR10)(?:[^\+]|$)'),
-        'HDR': re.compile(rf'(?:{DELIM}|^)(?i:HDR)(?:[^10\+]|$)'),
-    }
     _remastered_regex = re.compile(rf'(?:{DELIM}|^)((?i:4k{DELIM}+|)(?i:remaster(?:ed|)|restored))(?:{DELIM}|$)')
 
-    def _get_date(self):
-        return _as_string(self._guess.get('date', ''))
-
     def _get_edition(self):
         edition = _as_list(self._guess.get('edition'))
         for edition_fixed, regex in self._edition_translation.items():
@@ -1188,7 +1201,7 @@ class ReleaseInfo(collections.abc.MutableMapping):
         # Revision (guessit doesn't distinguish between REPACK, PROPER, etc)
         match = self._proper_repack_regex.search(self.release_name_params)
         if match:
-            edition.append(match.group(1).capitalize())
+            edition.append(match.group(1).upper())
 
         # Various
         guessit_other = _as_list(self._guess.get('other'))
@@ -1213,12 +1226,6 @@ class ReleaseInfo(collections.abc.MutableMapping):
         elif 'Restored' in edition and is_4k_source():
             edition[edition.index('Restored')] = '4k Restored'
 
-        # HDR format
-        for hdr_format, regex in self._hdr_regexes.items():
-            if regex.search(self.release_name_params):
-                edition.append(hdr_format)
-                break
-
         return edition
 
     def _get_resolution(self):
@@ -1263,15 +1270,14 @@ class ReleaseInfo(collections.abc.MutableMapping):
         return ''
 
     _source_translation = {
-        re.compile(r'(?i:blu-?ray)') : 'BluRay',
-        re.compile(r'(?i:dvd-?rip)') : 'DVDRip',
-        re.compile(r'(?i:tv-?rip)')  : 'TVRip',
-        re.compile(r'(?i:web-?dl)')  : 'WEB-DL',
-        re.compile(r'(?i:web-?rip)') : 'WEBRip',
-        re.compile(r'(?i:web)')      : 'WEB-DL',
+        re.compile(r'(?i:ultra hd blu-?ray)') : 'UHD BluRay',
+        re.compile(r'(?i:blu-?ray)')          : 'BluRay',
+        re.compile(r'(?i:dvd-?rip)')          : 'DVDRip',
+        re.compile(r'(?i:tv-?rip)')           : 'TVRip',
+        re.compile(r'(?i:web-?dl)')           : 'WEB-DL',
+        re.compile(r'(?i:web-?rip)')          : 'WEBRip',
+        re.compile(r'(?i:web)')               : 'WEB-DL',
     }
-    # Look for "Hybrid" after year or season
-    _hybrid_regex = re.compile(rf'{DELIM}hybrid{DELIM}', flags=re.IGNORECASE)
     _web_source_regex = re.compile(rf'{DELIM}(WEB-?(?:DL|Rip))(?:{DELIM}|$)', flags=re.IGNORECASE)
 
     def _get_source(self):
@@ -1365,6 +1371,19 @@ class ReleaseInfo(collections.abc.MutableMapping):
                 return match.group(1)
         return audio_channels
 
+    _hdr_translation = {
+        re.compile(rf'(?:{DELIM}|^)(?i:DV|DoVi|Dolby{DELIM}Vision)(?:{DELIM}|$)'): 'DV',
+        re.compile(rf'(?:{DELIM}|^)(?i:HDR10\+)(?:{DELIM}|$)'): 'HDR10+',
+        re.compile(rf'(?:{DELIM}|^)(?i:HDR10)(?!:\+)(?:{DELIM}|$)'): 'HDR10',
+        re.compile(rf'(?:{DELIM}|^)(?i:HDR)(?!:10\+)(?:{DELIM}|$)'): 'HDR',
+    }
+
+    def _get_hdr_format(self):
+        for regex, hdr_format in self._hdr_translation.items():
+            if regex.search(self.release_name_params):
+                return hdr_format
+        return ''
+
     _x264_regex = re.compile(rf'(?:{DELIM}|^)(?i:x264)(?:{DELIM}|$)')
     _x265_regex = re.compile(rf'(?:{DELIM}|^)(?i:x265)(?:{DELIM}|$)')
 
