@@ -3,7 +3,6 @@ String formatting and parsing
 """
 
 import importlib
-import inspect
 import os
 import re
 import sys
@@ -33,16 +32,19 @@ def group(name):
 
 _max_nfo_size = 1048576
 
-def read_nfo(directory):
+def read_nfo(directory, strip=False):
     """
     Recursively search `directory` for ``*.nfo`` files and read the first
 
-    If `directory` is not a directory, return None.
+    `directory` may also be an nfo file. Otherwise, if `directory` is not
+    a directory, return `None`.
 
     The nfo file is decoded with :func:`decode_nfo`.
 
     Files larger than 1 Mib are ignored.
 
+    :param bool string: See :func:`decode_nfo`
+
     :raise ContentError: if the nfo file is not readable
     """
     error_msgs = []
@@ -50,7 +52,7 @@ def read_nfo(directory):
         try:
             if os.path.getsize(nfo_filepath) <= _max_nfo_size:
                 with open(nfo_filepath, 'rb') as f:
-                    return decode_nfo(f.read())
+                    return decode_nfo(f.read(), strip=strip)
         except OSError as e:
             error_msgs.append(e.strerror if e.strerror else str(e))
 
@@ -58,16 +60,33 @@ def read_nfo(directory):
         raise errors.ContentError(f'Failed to read nfo: {error_msgs[0]}')
 
 
-def decode_nfo(bytes):
+def decode_nfo(bytes, strip=False):
     """
     Return decoded `bytes`
 
     Try to decode as UTF-8 first. If that fails, decode as CP437.
+
+    :param bool string: Whether to remove whitespace at the beginning and end
+
+        .. note:: To preserve ASCII art, spaces at the beginning of the first
+                  non-empty line are kept.
     """
     try:
-        return bytes.decode('utf8', 'strict')
+        text = bytes.decode('utf8', 'strict')
     except UnicodeDecodeError:
-        return bytes.decode('cp437', 'replace')
+        text = bytes.decode('cp437', 'replace')
+
+    if strip:
+        # Remove any whitespace at the end.
+        text = text.rstrip()
+        # Remove empty lines at the beginning while keeping any leading spaces
+        # on the first non-empty line to preserve ASCII art.
+        text = '\n'.join((
+            line for line in text.splitlines()
+            if line.strip()
+        ))
+
+    return text
 
 
 _capitalize_regex = re.compile(r'(\s*)(\S+)(\s*)')
@@ -130,21 +149,32 @@ else:
             return string
 
 
-def evaluate_fstring(template):
+def evaluate_fstring(template, **variables):
     """
     Deferred f-string evaluation
 
+    .. warning:: Because `template` is passed to :func:`eval`, this function
+        must NEVER be called with code that comes from the user. Only use this
+        function to evaluate hardcoded strings.
+
+    :param str template: Unevaluated f-string (i.e. a normal string containing
+        curly braces without the ``f`` in front of the quote)
+    :param dict variables: :class:`dict` instance that maps names that can be
+        used in `template` in curly braces to objects that replace the curly
+        braced name.
+
     Example:
 
-    >>> template = 'Evaluate this: {sum((1, 2, 3)) * 2}'
-    >>> evaluate_fstring(template)
-    12
+    >>> template = 'Evaluate {x}: {sum((1, 2, 3)) * 2}'
+    >>> evaluate_fstring(template, x='this')
+    Evaluate this: 12
+    >>> evaluate_fstring(template, x='that')
+    Evaluate that: 12
 
     References:
 
         https://pypi.org/project/f-yeah/
         https://stackoverflow.com/a/42497694
     """
-    parent_frame = inspect.stack()[1].frame
     fstring = 'f' + repr(template)
-    return eval(fstring, parent_frame.f_globals, parent_frame.f_locals)
+    return eval(fstring, variables)
