@@ -38,11 +38,13 @@ def create(*, content_path, announce, source,
         is usually the tracker's abbreviated name.
     :param str torrent_path: Path of the generated torrent file
     :param exclude: Sequence of glob patterns (:class:`str`) and
-        :class:`re.Pattern` objects (return value from :func:`re.compile`)
+        :class:`re.Pattern` (return value from :func:`re.compile`) or
+        :class:`~.types.RegEx` objects
 
         Files beneath `content_path` are excluded from the torrent.
 
-        Glob patterns are matched case-insensitively.
+        Glob patterns are matched case-insensitively. For case-insensitive
+        matching with regular expressions, use ``(?i:<pattern>)``.
     :param bool use_cache: Whether to get piece hashes from previously created
         torrents or from `reuse_torrent_path`
     :param reuse_torrent_path: Path to existing torrent file to get hashed
@@ -80,14 +82,14 @@ def create(*, content_path, announce, source,
     truthy value. If `progress_callback` returns :data:`SKIP_SEARCHING`, the
     search for a reusable torrent is cancelled and pieces are hashed normally.
 
-    :raise TorrentError: if anything goes wrong
+    :raise TorrentCreateError: if anything goes wrong
 
     :return: `torrent_path` or `None` if cancelled
     """
     if not announce:
-        raise errors.TorrentError('Announce URL is empty')
+        raise errors.TorrentCreateError('Announce URL is empty')
     if not source:
-        raise errors.TorrentError('Source is empty')
+        raise errors.TorrentCreateError('Source is empty')
 
     # Create Torrent object
     torrent = _get_torrent(
@@ -142,8 +144,10 @@ def create(*, content_path, announce, source,
 def _get_exclude_regexs(exclude):
     regexs = []
     for pattern in exclude:
-        if utils.is_regex_pattern(pattern):
+        if isinstance(pattern, re.Pattern):
             regexs.append(pattern)
+        elif isinstance(pattern, types.RegEx):
+            regexs.append(re.compile(pattern.pattern))
         elif isinstance(pattern, str):
             regexs.append(re.compile(fnmatch.translate(str(pattern)), flags=re.IGNORECASE))
         else:
@@ -164,7 +168,7 @@ def _get_torrent(*, content_path, exclude, announce, source, randomize_infohash)
             creation_date=time.time(),
         )
     except torf.TorfError as e:
-        raise errors.TorrentError(str(e))
+        raise errors.TorrentCreateError(str(e))
 
 
 def _generate_hashes(*, torrent, callback):
@@ -175,7 +179,7 @@ def _generate_hashes(*, torrent, callback):
             interval=1.0,
         )
     except torf.TorfError as e:
-        raise errors.TorrentError(str(e))
+        raise errors.TorrentCreateError(str(e))
     else:
         return wrapped_callback.return_value
 
@@ -193,7 +197,7 @@ def _find_hashes(*, torrent, reuse_torrent_path, callback):
             interval=1.0,
         )
     except torf.TorfError as e:
-        raise errors.TorrentError(str(e))
+        raise errors.TorrentCreateError(str(e))
     else:
         return wrapped_callback.return_value
 
@@ -229,7 +233,7 @@ def _write_torrent_path(torrent, torrent_path):
     try:
         torrent.write(torrent_path, overwrite=True)
     except torf.TorfError as e:
-        raise errors.TorrentError(str(e))
+        raise errors.TorrentCreateError(str(e))
 
 
 def _get_generic_torrent_path(torrent, create_directory=True):
@@ -238,7 +242,7 @@ def _get_generic_torrent_path(torrent, create_directory=True):
         try:
             fs.mkdir(directory)
         except errors.ContentError as e:
-            raise errors.TorrentError(f'{directory}: {e}')
+            raise errors.TorrentCreateError(f'{directory}: {e}')
 
     cache_id = _get_torrent_id(torrent)
     filename = f'{torrent.name}.{cache_id}.torrent'
@@ -272,16 +276,24 @@ class Files:
 
     def __init__(self, torrent):
         self._list = self._make_file_list(torrent.filepaths)
-        self._tree = self._make_file_tree(torrent.filetree, parent_path=str(torrent.path.parent))
+        self._tree = self._make_file_tree(
+            torrent.filetree,
+            parent_path=str(torrent.path.parent),
+            strip_leading_sep=False,
+        )
 
     def _make_file_list(self, filepaths):
         return tuple(str(filepath) for filepath in filepaths)
 
-    def _make_file_tree(self, filetree, parent_path=''):
+    def _make_file_tree(self, filetree, parent_path='', strip_leading_sep=True):
         files = []
 
         for name, file in filetree.items():
-            path = os.sep.join((parent_path, name)).strip(os.sep)
+            path = os.sep.join((parent_path, name))
+            if strip_leading_sep:
+                path = path.strip(os.sep)
+            else:
+                path = path.rstrip(os.sep)
 
             if isinstance(file, collections.abc.Mapping):
                 subtree = self._make_file_tree(file)
@@ -450,7 +462,7 @@ class _FindTorrentCallback(_CallbackBase):
             files_per_second=info['items_per_second'],
             filepath=filepath,
             status=status,
-            exception=errors.TorrentError(str(exception)) if exception else None,
+            exception=errors.TorrentCreateError(str(exception)) if exception else None,
             seconds_elapsed=info['seconds_elapsed'],
             seconds_remaining=info['seconds_remaining'],
             seconds_total=info['seconds_total'],
@@ -515,7 +527,7 @@ class FindTorrentProgress(collections.namedtuple(
     """
     :func:`~.collections.namedtuple` with these attributes:
 
-        - ``exception`` (:class:`~.errors.TorrentError` or `None`)
+        - ``exception`` (:class:`~.errors.TorrentCreateError` or `None`)
         - ``filepath`` (:class:`str`)
         - ``files_done`` (:class:`int`)
         - ``files_per_second`` (:class:`int`)
@@ -550,7 +562,7 @@ def filter_files(content_path, exclude=()):
             source=None,
             randomize_infohash=False,
         )
-    except errors.TorrentError as e:
+    except errors.TorrentCreateError as e:
         raise errors.ContentError(e)
     else:
         return [str(f) for f in torrent.filepaths]
