@@ -8,6 +8,11 @@ import collections
 import enum
 import re
 
+from . import imghosts
+
+import logging  # isort:skip
+_log = logging.getLogger(__name__)
+
 
 def Integer(value, min=None, max=None):
     """
@@ -57,7 +62,7 @@ def Integer(value, min=None, max=None):
     return Integer(value)
 
 
-def Choice(value, options, empty_ok=False):
+def Choice(value, options, empty_ok=False, case_sensitive=True):
     """
     Return :class:`str` subclass that can only have instances that are equal to
     an item of `options`
@@ -66,17 +71,24 @@ def Choice(value, options, empty_ok=False):
     :param options: Iterable of allowed instances
     :param bool empty_ok: Whether an emptry string is valid even if it is not in
         `options`
+    :param bool case_sensitive: Whether case is considered
 
     :raise ValueError: if instantiation is attempted with a value that is not in
         `options`
     """
-    options_str = tuple(sorted(str(o) for o in options))
+    if case_sensitive:
+        options_str = tuple(sorted(str(o) for o in options))
+    else:
+        options_str = tuple(sorted(str(o).lower() for o in options))
 
     class Choice(str):
         options = options_str
 
         def __new__(cls, val):
             val_str = str(val)
+            if not case_sensitive:
+                val_str = val_str.lower()
+
             if val_str not in cls.options and (val_str or not empty_ok):
                 raise ValueError(f'Not one of {", ".join(cls.options)}: {val}')
             else:
@@ -91,6 +103,36 @@ def Choice(value, options, empty_ok=False):
     return Choice(value)
 
 
+def ImageHost(allowed=None, disallowed=None):
+    """
+    Return new :class:`Choice` subclass that only accepts `allowed`
+    image host names
+
+    :param allowed: Sequence of allowed image host names or `None`
+        to allow all supported image host names
+    :param disallowed: Sequence of disallowed image host names or `None`
+        to allow all supported image host names
+    """
+    if allowed:
+        options = set(allowed)
+    else:
+        options = set(imghosts.imghost_names())
+
+    if disallowed:
+        for item in disallowed:
+            options.discard(item)
+
+    ImageHost = type(Choice(
+        sorted(options)[0],
+        options=options,
+        case_sensitive=False,
+    ))
+    ImageHost.__name__ = 'ImageHost'
+    ImageHost.__qualname__ = ImageHost.__name__
+    ImageHost.__doc__ = "Name of a supported image hosting service"
+    return ImageHost
+
+
 class Bool(str):
     """
     :class:`str` subclass with boolean value
@@ -299,29 +341,70 @@ class SceneCheckResult(enum.Enum):
         return f'{type(self).__name__}.{self.value}'
 
 
-def RegEx(pattern):
+class RegEx:
     """
-    Create :class:`re.Pattern` (return value of :func:`re.compile`)
+    Special class that behaves like :class:`re.Pattern` (return value of
+    :func:`re.compile`) but more intuitively
+
+    Its string representation is :attr:`re.Pattern.pattern` instead of
+    ``re.compile('<pattern>')``.
+
+    Instead of raising :class:`re.error`, it raises :class:`ValueError` on
+    invalid regular expressions.
 
     :raise ValueError: instead of :class:`re.error`
     """
-    try:
-        return re.compile(pattern)
-    except re.error as e:
-        orig_msg = str(e)
-        msg = orig_msg[0].upper() + orig_msg[1:]
-        raise ValueError(f'{pattern}: {msg}')
+
+    def __init__(self, pattern):
+        if isinstance(pattern, type(self)):
+            self._regex = pattern._regex
+        else:
+            try:
+                self._regex = re.compile(pattern)
+            except re.error as e:
+                orig_msg = str(e)
+                msg = orig_msg[0].upper() + orig_msg[1:]
+                raise ValueError(f'{pattern}: {msg}')
+
+    def __getattr__(self, name):
+        return getattr(self._regex, name)
+
+    def __str__(self):
+        return self._regex.pattern
+
+    def __repr__(self):
+        return f'{type(self).__name__}({self._regex.pattern!r})'
+
+    def __eq__(self, other):
+        if isinstance(other, type(self)):
+            return self._regex.pattern == other._regex.pattern
+        else:
+            return NotImplemented
+
+    def __hash__(self):
+        return hash(self._regex)
+
+    # Prevent maximum recursion error in __getattr__().
+    # See: https://docs.python.org/3/library/pickle.html#object.__reduce__
+    def __reduce__(self):
+        return (type(self), (self._regex.pattern,))
+
+    # If we copy.deepcopy() instances of this class (e.g. `ListOf(RegEx)` that
+    # is used for trackers.*.exclude settings, we end up with `re.compile(...)`
+    # instances for some reason. Implementing __deepcopy__() fixes that.
+    def __deepcopy__(self, memo):
+        return type(self)(self._regex.pattern)
 
 
-def ListOf(item_type, iterable=()):
+def ListOf(item_type, default=(), separator=None):
     """
     Immutable list of `item_type` objects
 
     :param item_type: Any callable that returns a validated object for one of the
         list items or raises :class:`ValueError` or :class:`TypeError`
-    :param iterable: Any iterable of initial items
+    :param default: Any iterable of initial items
 
-        Items in `iterable` are converted with `item_type` as well.
+        Items in `default` are converted to `item_type` as well.
 
     :raise ValueError: if any invalid value is added to the list
 
@@ -330,37 +413,77 @@ def ListOf(item_type, iterable=()):
 
     # Avoid NameError bug (https://github.com/python/cpython/issues/87546)
     item_type_ = item_type
+    separator_ = separator
 
     class ListOf(collections.abc.Sequence):
 
         item_type = item_type_
+        separator = separator_
 
         @classmethod
         def _convert(cls, item):
-            try:
-                return cls.item_type(item)
-            except (ValueError, TypeError):
-                raise ValueError(f'Invalid {cls.item_type.__qualname__} value: {item}')
+            if isinstance(item, str):
+                # Split `item` into multiple items
+                if separator and separator in item:
+                    for subitem in item.split(separator):
+                        yield from cls._convert(subitem)
 
-        def __new__(cls, iterable=()):
+                # Ignore empty `item` if `separator` is specified. This
+                # conveniently converts "foo,,bar,baz" to ["foo", "bar", "baz"].
+                elif item or not separator:
+                    try:
+                        yield cls.item_type(item)
+                    except (ValueError, TypeError):
+                        raise ValueError(f'Invalid value: {item}')
+
+            # `item` is multiple values
+            elif isinstance(item, collections.abc.Iterable):
+                for subitem in item:
+                    yield from cls._convert(subitem)
+
+            # `item` is single value
+            else:
+                try:
+                    yield cls.item_type(item)
+                except (ValueError, TypeError):
+                    raise ValueError(f'Invalid value: {item}')
+
+        def __new__(cls, items=()):
             self = super().__new__(cls)
-            self._seq = tuple(cls._convert(item) for item in iterable)
+            self._sequence = tuple(self._convert(items))
             return self
 
         def __getitem__(self, key):
-            return self._seq[key]
+            return self._sequence[key]
 
         def __len__(self):
-            return len(self._seq)
+            return len(self._sequence)
 
         def __eq__(self, other):
-            if isinstance(other, collections.abc.Sequence):
-                return self._seq == tuple(other)
+            if hasattr(other, 'item_type') and hasattr(other, '_sequence'):
+                return (
+                    self.item_type is other.item_type
+                    and self._sequence == other._sequence
+                )
+            elif isinstance(other, collections.abc.Sequence):
+                return self._sequence == tuple(other)
             else:
                 return NotImplemented
 
+        def __hash__(self):
+            return hash((
+                self._sequence,
+                self.item_type,
+            ))
+
+        def __str__(self):
+            return ', '.join(
+                str(item)
+                for item in self._sequence
+            )
+
         def __repr__(self):
-            return f'{type(self).__name__}({self._seq!r})'
+            return f'{type(self).__name__}({self._sequence!r})'
 
     # Specify class name for easier debugging (e.g. "ListOfSomeClassName")
     ListOf.__name__ = (
@@ -372,4 +495,40 @@ def ListOf(item_type, iterable=()):
     ListOf.__doc__ = f"""Immutable list of {item_type_.__qualname__}"""
 
     # Instantiate custom list with provided items
-    return ListOf(iterable)
+    return ListOf(default)
+
+
+class PathTranslation(collections.namedtuple('PathTranslation', ('local', 'remote'))):
+    """
+    :class:`collections.namedtuple` that takes two paths separated by ``->``
+    and provides them as the attributes :attr:`local` and :attr:`remote`.
+    """
+
+    _regex = re.compile(r'^\s*(.+?)\s*->\s*(.+?)\s*$')
+
+    def __new__(cls, string):
+        match = cls._regex.search(string)
+        if match:
+            local, remote = match.groups()
+            return super().__new__(cls, local, remote)
+        else:
+            raise ValueError(f'Invalid path translation: {string}')
+
+    def __str__(self):
+        return f'{self.local} -> {self.remote}'
+
+
+class PathTranslations(type(ListOf(PathTranslation))):
+    """
+    :class:`~.types.ListOf` :class:`PathTranslation` instances
+    """
+
+    def translate(self, path):
+        """
+        Replace each occurrence of `local` with `remote` in `path` for each
+        :class:`PathTranslation` and return the resulting path.
+        """
+        for translation in self:
+            _log.debug('Translating path = %r.replace(%r, %r)', path, translation.local, translation.remote)
+            path = path.replace(translation.local, translation.remote)
+        return path
