@@ -63,24 +63,44 @@ def mediainfo(path, only_first=True):
         else:
             return mi
 
+    unique_id_default = 'Unique ID                                : 0 (0x0)'
+    unique_id_regex = re.compile(r'^Unique ID\s*:\s*\d+', flags=re.MULTILINE)
+
+    def ensure_unique_id_exists(mi):
+        # Some mediainfo parsers expect a unique ID, but it doesn't always exist.
+        if unique_id_regex.search(mi):
+            return mi
+        else:
+            assert mi.startswith('General\n')
+            return (
+                mi[:len('General\n')]
+                + unique_id_default + '\n'
+                + mi[len('General\n'):]
+            )
+
+    def apply_fixes(mi):
+        return remove_parent_directory(
+            ensure_unique_id_exists(mi)
+        )
+
     if not os.path.isdir(path):
         # If we get a file path, don't complain if it's not a video file. This
         # allows us to get mediainfo for .IFO files even though they are not
         # videos.
         mediainfo = _run_mediainfo(path)
-        return remove_parent_directory(mediainfo)
+        return apply_fixes(mediainfo)
 
     elif only_first:
         video_file = find_videos(path)[0]
         mediainfo = _run_mediainfo(video_file)
-        return remove_parent_directory(mediainfo)
+        return apply_fixes(mediainfo)
 
     else:
         video_files = find_videos(path)
         mediainfos = []
         for video_file in video_files:
             mediainfo = _run_mediainfo(video_file)
-            mediainfos.append(remove_parent_directory(mediainfo))
+            mediainfos.append(apply_fixes(mediainfo))
         return mediainfos
 
 
@@ -288,7 +308,7 @@ def _get_display_height(video_track):
 @functools.lru_cache(maxsize=None)
 def resolution(path, default=NO_DEFAULT_VALUE):
     """
-    Return resolution of video file `path` (e.g. "1080p")
+    Return resolution of video file `path` as :class:`str` (e.g. "1080p")
 
     :param str path: Path to video file or directory
 
@@ -306,6 +326,28 @@ def resolution(path, default=NO_DEFAULT_VALUE):
     scan_type = _scan_type(video_track)
     return f'{std_resolution}{scan_type}'
 
+
+@functools.lru_cache(maxsize=None)
+def resolution_int(path, default=NO_DEFAULT_VALUE):
+    """
+    Return resolution of video file `path` as :class:`int` (e.g. ``1080``)
+
+    :param str path: Path to video file or directory
+
+        For directories, the first video is used. See :func:`find_videos`.
+    :param default: Return value if `path` doesn't exist, raise
+        :exc:`~.ContentError` if not provided
+
+    :raise ContentError: if resolution can't be determined
+    """
+    if default is not NO_DEFAULT_VALUE and not os.path.exists(path):
+        return default
+
+    video_track = default_track('video', path)
+    std_resolution = _get_closest_standard_resolution(video_track)
+    return std_resolution
+
+
 def _scan_type(video_track):
     # "p" or "i", default to "p"
     scan_type = str(video_track.get('ScanType', 'Progressive')).lower()
@@ -400,7 +442,8 @@ def frame_rate(path, default=NO_DEFAULT_VALUE):
 @functools.lru_cache(maxsize=None)
 def bit_depth(path, default=NO_DEFAULT_VALUE):
     """
-    Return bit depth of default video track or ``0`` if it can't be determined
+    Return bit depth of default video track (e.g. ``8`` or ``10``) or ``0`` if
+    it can't be determined
 
     :param str path: Path to video file or directory
 
@@ -420,28 +463,20 @@ def bit_depth(path, default=NO_DEFAULT_VALUE):
         return 0
 
 
-hdr_formats = {
-    'Dolby Vision': {'HDR_Format': re.compile(r'^Dolby Vision$')},
-    # "HDR10+ Profile A" or "HDR10+ Profile B"
-    'HDR10+': {'HDR_Format_Compatibility': re.compile(r'HDR10\+')},
-    'HDR10': (
-        {'HDR_Format_Compatibility': re.compile(r'HDR10(?!\+)')},
-        {'colour_primaries': re.compile(r'BT\.2020')},
-    ),
-    'HDR': (
-        {'HDR_Format_Compatibility': re.compile(r'HDR')},
-        {'HDR_Format': re.compile(r'HDR')},
-    ),
+known_hdr_formats = {
+    'DV',
+    'HDR10+',
+    'HDR10',
+    'HDR',
 }
-"""
-Map HDR format names (e.g. "Dolby Vision") to dictinaries that map mediainfo
-video track fields to regular expressions that must match the fields' values
-"""
+"""Set of valid HDR format names"""
+
 
 @functools.lru_cache(maxsize=None)
-def hdr_format(path, default=NO_DEFAULT_VALUE):
+def hdr_formats(path, default=NO_DEFAULT_VALUE):
     """
-    Return HDR format based on `hdr_formats`, e.g. "HDR10", or empty string
+    Return sequence of HDR formats e.g. ``("HDR10",)``, ``("DV", "HDR10")``
+    or ``()``
 
     :param str path: Path to video file or directory
 
@@ -454,26 +489,48 @@ def hdr_format(path, default=NO_DEFAULT_VALUE):
     if default is not NO_DEFAULT_VALUE and not os.path.exists(path):
         return default
 
-    def is_match(video_track, conditions):
-        if isinstance(conditions, collections.abc.Mapping):
-            # `conditions` maps field names to field value matching regexes
-            for key, regex in conditions.items():
-                value = video_track.get(key, '')
-                if regex.search(value):
-                    return True
-        else:
-            # `conditions` is a sequence of mappings that map field names to
-            # field value matching regexes (see above)
-            for condition in conditions:
-                if is_match(video_track, condition):
-                    return True
-        return False
+    def is_dv(video_track):
+        return bool(
+            # Dolby Vision[ / <more information>]
+            re.search(r'^Dolby Vision', video_track.get('HDR_Format', ''))
+        )
 
+    def is_hdr10p(video_track):
+        return bool(
+            # "HDR10+ Profile A" or "HDR10+ Profile B"
+            re.search(r'HDR10\+', video_track.get('HDR_Format_Compatibility', ''))
+        )
+
+    def is_hdr10(video_track):
+        return bool(
+            re.search(r'HDR10(?!\+)', video_track.get('HDR_Format_Compatibility', ''))
+            or
+            re.search(r'BT\.2020', video_track.get('colour_primaries', ''))
+        )
+
+    def is_hdr(video_track):
+        return bool(
+            re.search(r'HDR(?!10)', video_track.get('HDR_Format_Compatibility', ''))
+            or
+            re.search(r'HDR(?!10)', video_track.get('HDR_Format', ''))
+        )
+
+    hdr_formats = []
     video_track = default_track('video', path)
-    for hdr_format, conditions in hdr_formats.items():
-        if is_match(video_track, conditions):
-            return hdr_format
-    return ''
+
+    # NOTE: DV and HDR(10)(+) can co-exist.
+
+    if is_dv(video_track):
+        hdr_formats.append('DV')
+
+    if is_hdr10p(video_track):
+        hdr_formats.append('HDR10+')
+    elif is_hdr10(video_track):
+        hdr_formats.append('HDR10')
+    elif is_hdr(video_track):
+        hdr_formats.append('HDR')
+
+    return tuple(hdr_formats)
 
 
 @functools.lru_cache(maxsize=None)
@@ -563,6 +620,41 @@ def _get_color_matrix(path, name, default=NO_DEFAULT_VALUE):
     return False
 
 
+@functools.lru_cache(maxsize=None)
+def languages(path, default=NO_DEFAULT_VALUE, exclude_commentary=True):
+    """
+    Extract ``Language`` fields from each track for each track type
+
+    Example return value:
+
+    .. code::
+
+        {
+            'Audio': ['en', 'dk'],
+            'Text': ['en', 'cz', 'pt'],
+        }
+
+    :param str path: Path to video file or directory
+    :param default: Default language for each track that doesn't specify one
+    :param exclude_commentary: Ignore any track with a ``Title`` field that
+        contains the string "commentary" (case-insensitive)
+
+    :raise ContentError: if reading `path` fails
+    """
+    all_tracks = utils.video.tracks(path)
+    languages = collections.defaultdict(list)
+    for type, tracks in all_tracks.items():
+        for track in tracks:
+            title = track.get('Title', '').lower()
+            if not exclude_commentary or 'commentary' not in title:
+                language = track.get('Language', None)
+                if language is not None:
+                    languages[type].append(language)
+                elif default is not NO_DEFAULT_VALUE:
+                    languages[type].append(default)
+    return dict(languages)
+
+
 @functools.lru_cache(maxsize=None)
 def has_commentary(path, default=NO_DEFAULT_VALUE):
     """
@@ -993,9 +1085,11 @@ def for_each_video_ts(path):
 
     Subdirectories of `path` are searched recursively.
     """
+    os_sep_escaped = glob.escape(os.sep)
+    path_escaped = glob.escape(path)
     video_ts_paths = natsort.natsorted(
         glob.glob(
-            f'{path}{utils.fs.OS_SEP_REGEX}**{utils.fs.OS_SEP_REGEX}VIDEO_TS',
+            f'{path_escaped}{os_sep_escaped}**{os_sep_escaped}VIDEO_TS',
             recursive=True,
         )
     )
