@@ -3,8 +3,11 @@ API for imdb.com
 """
 
 import functools
+import json
 import re
 
+import async_lru
+
 from ... import utils
 from ..types import ReleaseType
 from . import common
@@ -43,7 +46,7 @@ class ImdbApi(WebDbApiBase):
         return self._soup_cache[cache_id]
 
     _title_types = {
-        ReleaseType.movie: 'feature,tv_movie,documentary,short,video,tv_short',
+        ReleaseType.movie: 'feature,tv_movie,short,video,tv_short',
         ReleaseType.season: 'tv_series,tv_miniseries',
         # Searching for single episodes is currently not supported
         ReleaseType.episode: 'tv_series,tv_miniseries',
@@ -107,10 +110,10 @@ class ImdbApi(WebDbApiBase):
                 params['release_date'] = f'{query.year}-01-01,{query.year}-12-31'
 
             def is_search_result(tag):
-                if tag.get('class') == ['lister-item-content']:
-                    header = tag.find(class_='lister-item-header')
-                    if not header or 'Episode:' not in header.strings:
-                        return True
+                if 'ipc-metadata-list-summary-item__tc' in tag.get('class', ()):
+                    return True
+
+                return False
 
             soup = await self._get_soup(path, params=params)
             items = soup.find_all(is_search_result)
@@ -122,29 +125,30 @@ class ImdbApi(WebDbApiBase):
 
     _person_url_path_regex = re.compile(r'(/name/nm\d+)')
 
-    def _get_persons(self, tag):
-        a_tags = tag.find_all('a', href=self._person_url_path_regex)
-        persons = []
-        for a_tag in a_tags:
-            if a_tag.string:
-                name = a_tag.string.strip()
-                url_path = self._person_url_path_regex.match(a_tag["href"]).group(1)
-                url = f'{self._url_base.rstrip("/")}/{url_path.lstrip("/")}'
-                persons.append(common.Person(name, url))
-        return tuple(persons)
+    @async_lru.alru_cache
+    async def _get_json(self, id, key=None, default=None):
+        soup = await self._get_soup(f'title/{id}/')
+        tag = soup.find('script', type='application/ld+json')
+        dct = json.loads(tag.string)
+        if dct:
+            if key is None:
+                return dct
+            elif key in dct:
+                return dct[key]
+        return default
+
+    async def _get_persons(self, id, type):
+        infos = await self._get_json(id, key=type, default=())
+        return tuple(
+            common.Person(utils.html.parse(info['name']), info['url'].rstrip('/'))
+            for info in infos
+            if info['@type'] == 'Person'
+        )
 
     async def cast(self, id):
-        cast = []
         if id:
-            soup = await self._get_soup(f'title/{id}/')
-            # New website
-            cast_tag = soup.find(class_='title-cast__grid')
-            if cast_tag is None:
-                # Old website
-                cast_tag = soup.find(class_='cast_list')
-            if cast_tag:
-                cast.extend(self._get_persons(cast_tag))
-        return tuple(cast)
+            return await self._get_persons(id, type='actor')
+        return ()
 
     async def _countries(self, id):
         countries = []
@@ -160,59 +164,31 @@ class ImdbApi(WebDbApiBase):
 
     async def creators(self, id):
         if id:
-            soup = await self._get_soup(f'title/{id}/')
-            # Old website design
-            credits_tags = soup.find_all(class_='credit_summary_item')
-            if not credits_tags:
-                # New website design
-                credits_tags = soup.find_all(class_='ipc-metadata-list__item')
-            for tag in credits_tags:
-                if any(self._creators_label_regex.search(string)
-                       for string in tag.stripped_strings):
-                    return self._get_persons(tag)
+            return await self._get_persons(id, type='creator')
         return ()
 
     _directors_label_regex = re.compile('^Directors?:?$')
 
     async def directors(self, id):
         if id:
-            soup = await self._get_soup(f'title/{id}/')
-            # Old website design
-            credits_tags = soup.find_all(class_='credit_summary_item')
-            if not credits_tags:
-                # New website design
-                credits_tags = soup.find_all(class_='ipc-metadata-list__item')
-            for tag in credits_tags:
-                if any(self._directors_label_regex.search(string)
-                       for string in tag.stripped_strings):
-                    return self._get_persons(tag)
+            return await self._get_persons(id, type='director')
         return ()
 
     async def genres(self, id):
         if id:
-            soup = await self._get_soup(f'title/{id}/')
-            genres = []
-            genre_links = soup.find_all('a', href=re.compile(r'/search/title\?.*genres='))
-            for genre_link in genre_links:
-                match = re.search(r'genres=([^&]+)', genre_link['href'])
-                if match:
-                    genre = match.group(1).lower()
-                    if genre not in genres:
-                        genres.append(genre)
-            return tuple(genres)
+            genres = await self._get_json(id, key='genre', default=())
+            return tuple(
+                genre.casefold()
+                for genre in genres
+            )
         return ()
 
     async def poster_url(self, id, season=None):
-        if id and await self.type(id) != ReleaseType.episode:
-            soup = await self._get_soup(f'title/{id}/reference')
-            tag = soup.select_one('meta [property="og:image"]')
-            if tag:
-                url = tag.get('content')
-                if url:
-                    if '@' in url:
-                        # Remove any cropping instructions and other stuff
-                        url = re.sub(r'_CR[\d+,]+', r'', url)
-                    return url
+        if id:
+            poster_url = await self._get_json(id, key='image', default='')
+            # Request scaled down poster (300 pixels wide)
+            poster_url = re.sub(r'@._V1_*.jpg$', '@._V1_SX300.jpg', poster_url)
+            return poster_url
 
         return ''
 
@@ -221,23 +197,9 @@ class ImdbApi(WebDbApiBase):
 
     async def rating(self, id):
         if id:
-            soup = await self._get_soup(f'title/{id}/')
-
-            # Old website design
-            rating_tag = soup.find(itemprop='ratingValue')
-
-            if not rating_tag:
-                # New website design
-                rating_tag = soup.find(attrs={'data-testid': 'hero-rating-bar__aggregate-rating__score'})
-                if rating_tag and rating_tag.children:
-                    rating_tag = rating_tag.contents[0]
-
-            if rating_tag:
-                try:
-                    return float(rating_tag.string)
-                except (ValueError, TypeError):
-                    pass
-
+            info = await self._get_json(id, key='aggregateRating')
+            if info:
+                return info.get('ratingValue', None)
         return None
 
     _ignored_runtimes_keys = (
@@ -283,34 +245,16 @@ class ImdbApi(WebDbApiBase):
 
     async def summary(self, id):
         if id:
-            soup = await self._get_soup(f'title/{id}/')
+            description = await self._get_json(id, key='description', default='')
+            if not description.endswith('...'):
+                return description
 
-            # Get summary from the top
-            candidates = (
-                # Old website design
-                soup.find(class_='summary_text'),
-                # New website design
-                soup.find(attrs={'data-testid': 'plot-xl'}),
-            )
-            for tag in candidates:
-                if tag:
-                    string = ''.join(tag.stripped_strings).strip()
-                    link_texts = ('See full summary»', 'Read all', 'Add a Plot»')
-                    if all(not string.endswith(text) for text in link_texts):
-                        return string
-
-            # Get summary from dedicated page
-            soup = await self._get_soup(f'title/{id}/plotsummary')
-            candidates = (
-                # Old website design
-                utils.html.get(soup.find(id='plot-summaries-content'), 'li', 'p'),
-                # New website design
-                soup.find(attrs={'data-testid': 'sub-section-summaries'}),
-            )
-            for tag in candidates:
-                if tag:
-                    text = utils.html.as_text(tag)
-                    return re.sub(r'—.*$', '', text).strip()
+            # Try to find the complete description in the trailer section.
+            trailer = await self._get_json(id, key='trailer')
+            if trailer:
+                description = trailer.get('description', '')
+                if not description.endswith('...'):
+                    return description
 
         return ''
 
@@ -339,25 +283,13 @@ class ImdbApi(WebDbApiBase):
 
     async def type(self, id):
         if id:
-            soup = await self._get_soup(f'title/{id}/reference')
-
-            tag = soup.find(class_='ipl-inline-list')
-            text = ' '.join(reversed(tuple(tag.stripped_strings))).lower()
-
-            if 'episode' in text:
+            typ = await self._get_json(id, key='@type')
+            if typ == 'Movie':
+                return ReleaseType.movie
+            elif typ == 'TVSeries':
+                return ReleaseType.season
+            elif typ == 'TVEpisode':
                 return ReleaseType.episode
-            elif 'tv series' in text:
-                return ReleaseType.season
-            elif re.search(r'tv mini[- ]series', text):
-                return ReleaseType.season
-            elif 'tv movie' in text:
-                return ReleaseType.movie
-            elif 'movie' in text:
-                return ReleaseType.movie
-            elif 'short' in text:
-                return ReleaseType.movie
-            elif 'video' in text:
-                return ReleaseType.movie
 
         return ReleaseType.unknown
 
@@ -368,31 +300,30 @@ class ImdbApi(WebDbApiBase):
 
     async def year(self, id):
         if id:
-            soup = await self._get_soup(f'title/{id}/reference')
-
-            tag = soup.find(class_='titlereference-title-year')
-            if tag:
+            soup = await self._get_soup(f'title/{id}/releaseinfo')
+            for tag in soup.find_all(class_="ipc-metadata-list-item__list-content-item"):
                 text = ' '.join(reversed(tuple(tag.stripped_strings))).lower()
                 match = re.search(r'\b(\d{4})\b', text)
                 if match:
                     return match.group(1)
-
         return ''
 
 
 class _ImdbSearchResult(common.SearchResult):
     def __init__(self, *, imdb_api, soup=None, cast=None, countries=None,
-                 directors=None, id=None, genres=None, summary=None, title=None,
+                 directors=None, genres=None, id=None, poster=None, summary=None, title=None,
                  title_english=None, title_original=None, type=None, url=None,
                  year=None):
+        self._imdb_api = imdb_api
         soup = soup or utils.html.parse('')
         id = id or self._get_id(soup)
         return super().__init__(
-            cast=cast or self._get_cast(soup),
+            cast=cast or functools.partial(imdb_api.cast, id),
             countries=countries or functools.partial(imdb_api.countries, id),
-            directors=directors or self._get_directors(soup),
-            genres=genres or self._get_genres(soup),
+            directors=directors or functools.partial(imdb_api.directors, id),
+            genres=genres or functools.partial(imdb_api.genres, id),
             id=id or self._get_id(soup),
+            poster=functools.partial(imdb_api.poster, id),
             summary=summary or self._get_summary(soup),
             title=title or self._get_title(soup),
             title_english=title_english or functools.partial(imdb_api.title_english, id),
@@ -402,23 +333,6 @@ class _ImdbSearchResult(common.SearchResult):
             year=year or self._get_year(soup),
         )
 
-    def _get_cast(self, soup):
-        people = soup.find(string=re.compile(r'Stars?.*'))
-        if people:
-            names = tuple(name.string.strip() for name in people.parent.find_all('a'))
-            return names[1:]
-        else:
-            return ()
-
-    def _get_directors(self, soup):
-        people = soup.find(string=re.compile(r'Director?.*'))
-        if people:
-            director = people.parent.find('a')
-            if director:
-                return (director.string.strip(),)
-        else:
-            return ()
-
     def _get_id(self, soup):
         a_tag = soup.find('a')
         if a_tag:
@@ -426,49 +340,36 @@ class _ImdbSearchResult(common.SearchResult):
             return re.sub(r'^.*/([t0-9]+)/.*$', r'\1', href)
         return ''
 
-    def _get_genres(self, soup):
-        try:
-            genres = soup.find(class_='genre').string.strip()
-        except AttributeError:
-            genres = ''
-        if genres:
-            return tuple(g.strip().casefold() for g in genres.split(','))
-        else:
-            return ()
-
     def _get_summary(self, soup):
         summary = ''
-
-        tags = soup.find_all(class_='text-muted')
-        if len(tags) >= 3:
-            strings = tuple(tags[2].strings)
-            if strings and 'Add a Plot' not in strings:
-                summary = (''.join(strings) or '').strip()
-
-        # Look for "See full summary" link. Preceding text is summary.
-        if not summary:
-            summary_link = soup.find('a', string=re.compile(r'(?i:full\s+summary)'))
-            if summary_link:
-                summary_tag = summary_link.parent
-                if summary_tag:
-                    summary = ''.join(summary_tag.strings)
-
-        summary = re.sub(r'(?i:see full summary).*', '', summary).strip()
-        summary = re.sub(r'\s*\.\.\.\s*$', '…', summary)
+        tag = soup.find('div', attrs={'class': 'ipc-html-content', 'role': 'presentation'})
+        if tag:
+            return tag.string.strip()
 
         return summary
 
     def _get_title(self, soup):
-        a_tag = soup.find('a')
-        if a_tag:
-            return a_tag.string.strip()
+        tag = soup.find(attrs={'class': 'ipc-title__text'})
+        if tag:
+            title = tag.string.strip()
+            # Search result number is prepended to title
+            title = re.sub(r'^\d+\.', '', title)
+            return title.strip()
+
         return ''
 
     def _get_type(self, soup):
-        if soup.find(string=re.compile(r'Directors?:')):
-            return ReleaseType.movie
-        else:
-            return ReleaseType.series
+        tag = soup.find(attrs={'class': 'dli-title-type-data'})
+        if tag:
+            text = tag.string.strip().lower()
+            if any(name in text for name in ('movie', 'video', 'short')):
+                return ReleaseType.movie
+            elif 'series' in text:
+                return ReleaseType.series
+
+        # Sometimes there is no indication of type in search results. It seems
+        # this only happens to movies.
+        return ReleaseType.movie
 
     def _get_url(self, soup):
         id = self._get_id(soup)
@@ -477,19 +378,16 @@ class _ImdbSearchResult(common.SearchResult):
         return ''
 
     def _get_year(self, soup):
-        try:
-            year = soup.find(class_='lister-item-year').string or ''
-        except AttributeError:
-            return ''
-        # Year may be preceded by roman number
-        year = re.sub(r'\([IVXLCDM]+\)\s*', '', year)
-        # Remove parentheses
-        year = year.strip('()')
-        # Possible formats:
-        # - YYYY
-        # - YYYY–YYYY  ("–" is NOT "U+002D / HYPHEN-MINUS" but "U+2013 / EN DASH")
-        # - YYYY–
-        try:
-            return str(int(year[:4]))
-        except (ValueError, TypeError):
-            return ''
+        # New website design
+        # Find the first four-digit number. This covers movies as well as series
+        # in "YYYY-YYYY" and "YYYY-" format.
+        year_regex = re.compile(r'(\d{4})')
+        tags = soup.find_all(attrs={'class': 'dli-title-metadata-item'})
+        for tag in tags:
+            text = tag.string.strip()
+            match = year_regex.search(text)
+            if match:
+                year = match.group(1)
+                return year
+
+        return ''
